@@ -34,12 +34,11 @@
 OC.FileUpload = function(uploader, data) {
 	this.uploader = uploader;
 	this.data = data;
-	var path = '';
+	var basePath = '';
 	if (this.uploader.fileList) {
-		path = OC.joinPaths(this.uploader.fileList.getCurrentDirectory(), this.getFile().name);
-	} else {
-		path = this.getFile().name;
+		basePath = this.uploader.fileList.getCurrentDirectory();
 	}
+	var path = OC.joinPaths(basePath, this.getFile().relativePath || '', this.getFile().name);
 	this.id = 'web-file-upload-' + md5(path) + '-' + (new Date()).getTime();
 };
 OC.FileUpload.CONFLICT_MODE_DETECT = 0;
@@ -129,6 +128,16 @@ OC.FileUpload.prototype = {
 		return OC.joinPaths(this._targetFolder, this.getFile().relativePath || '');
 	},
 
+	/**
+	 * Get full path for the target file,
+	 * including relative path and file name.
+	 *
+	 * @return {String} full path
+	 */
+	getFullFilePath: function() {
+		return OC.joinPaths(this.getFullPath(), this.getFile().name);
+	},
+
 	/**
 	 * Returns conflict resolution mode.
 	 *
@@ -148,15 +157,6 @@ OC.FileUpload.prototype = {
 		this._conflictMode = mode;
 	},
 
-	/**
-	 * Returns whether the upload is in progress
-	 *
-	 * @return {boolean}
-	 */
-	isPending: function() {
-		return this.data.state() === 'pending';
-	},
-
 	deleteUpload: function() {
 		delete this.data.jqXHR;
 	},
@@ -219,13 +219,8 @@ OC.FileUpload.prototype = {
 			this.data.headers['If-None-Match'] = '*';
 		}
 
-		if (file.lastModified) {
-			// preserve timestamp
-			this.data.headers['X-OC-Mtime'] = file.lastModified / 1000;
-		}
-
-		var userName = this.uploader.filesClient.getUserName();
-		var password = this.uploader.filesClient.getPassword();
+		var userName = this.uploader.davClient.getUserName();
+		var password = this.uploader.davClient.getPassword();
 		if (userName) {
 			// copy username/password from DAV client
 			this.data.headers['Authorization'] =
@@ -238,8 +233,8 @@ OC.FileUpload.prototype = {
 			&& this.getFile().size > this.uploader.fileUploadParam.maxChunkSize
 		) {
 			data.isChunked = true;
-			chunkFolderPromise = this.uploader.filesClient.createDirectory(
-				'uploads/' + encodeURIComponent(OC.getCurrentUser().uid) + '/' + encodeURIComponent(this.getId())
+			chunkFolderPromise = this.uploader.davClient.createDirectory(
+				'uploads/' + OC.getCurrentUser().uid + '/' + this.getId()
 			);
 			// TODO: if fails, it means same id already existed, need to retry
 		} else {
@@ -264,9 +259,29 @@ OC.FileUpload.prototype = {
 		}
 
 		var uid = OC.getCurrentUser().uid;
-		return this.uploader.filesClient.move(
-			'uploads/' + encodeURIComponent(uid) + '/' + encodeURIComponent(this.getId()) + '/.file',
-			'files/' + encodeURIComponent(uid) + '/' + OC.joinPaths(this.getFullPath(), this.getFileName())
+		var mtime = this.getFile().lastModified;
+		var size = this.getFile().size;
+		var headers = {};
+		if (mtime) {
+			headers['X-OC-Mtime'] = mtime / 1000;
+		}
+		if (size) {
+			headers['OC-Total-Length'] = size;
+
+		}
+
+		return this.uploader.davClient.move(
+			'uploads/' + uid + '/' + this.getId() + '/.file',
+			'files/' + uid + '/' + OC.joinPaths(this.getFullPath(), this.getFileName()),
+			true,
+			headers
+		);
+	},
+
+	_deleteChunkFolder: function() {
+		// delete transfer directory for this upload
+		this.uploader.davClient.remove(
+			'uploads/' + OC.getCurrentUser().uid + '/' + this.getId()
 		);
 	},
 
@@ -275,12 +290,20 @@ OC.FileUpload.prototype = {
 	 */
 	abort: function() {
 		if (this.data.isChunked) {
-			// delete transfer directory for this upload
-			this.uploader.filesClient.remove(
-				'uploads/' + encodeURIComponent(OC.getCurrentUser().uid) + '/' + encodeURIComponent(this.getId())
-			);
+			this._deleteChunkFolder();
 		}
 		this.data.abort();
+		this.deleteUpload();
+	},
+
+	/**
+	 * Fail the upload
+	 */
+	fail: function() {
+		this.deleteUpload();
+		if (this.data.isChunked) {
+			this._deleteChunkFolder();
+		}
 	},
 
 	/**
@@ -290,7 +313,23 @@ OC.FileUpload.prototype = {
 	 */
 	getResponse: function() {
 		var response = this.data.response();
-		if (typeof response.result !== 'string') {
+		if (response.errorThrown) {
+			// attempt parsing Sabre exception is available
+			var xml = response.jqXHR.responseXML;
+			if (xml.documentElement.localName === 'error' && xml.documentElement.namespaceURI === 'DAV:') {
+				var messages = xml.getElementsByTagNameNS('http://sabredav.org/ns', 'message');
+				var exceptions = xml.getElementsByTagNameNS('http://sabredav.org/ns', 'exception');
+				if (messages.length) {
+					response.message = messages[0].textContent;
+				}
+				if (exceptions.length) {
+					response.exception = exceptions[0].textContent;
+				}
+				return response;
+			}
+		}
+
+		if (typeof response.result !== 'string' && response.result) {
 			//fetch response from iframe
 			response = $.parseJSON(response.result[0].body.innerText);
 			if (!response) {
@@ -361,6 +400,20 @@ OC.Uploader.prototype = _.extend({
 	 */
 	_uploads: {},
 
+	/**
+	 * Count of upload done promises that have not finished yet.
+	 *
+	 * @type int
+	 */
+	_pendingUploadDoneCount: 0,
+
+	/**
+	 * Is it currently uploading?
+	 *
+	 * @type boolean
+	 */
+	_uploading: false,
+
 	/**
 	 * List of directories known to exist.
 	 *
@@ -379,6 +432,13 @@ OC.Uploader.prototype = _.extend({
 	 */
 	filesClient: null,
 
+	/**
+	 * Webdav client pointing at the root "dav" endpoint
+	 *
+	 * @type OC.Files.Client
+	 */
+	davClient: null,
+
 	/**
 	 * Function that will allow us to know if Ajax uploads are supported
 	 * @link https://github.com/New-Bamboo/example-ajax-upload/blob/master/public/index.html
@@ -462,11 +522,13 @@ OC.Uploader.prototype = _.extend({
 				self.filesClient.createDirectory(fullPath).always(function(status) {
 					// 405 is expected if the folder already exists
 					if ((status >= 200 && status < 300) || status === 405) {
-						self.trigger('createdfolder', fullPath);
+						if (status !== 405) {
+							self.trigger('createdfolder', fullPath);
+						}
 						deferred.resolve();
 						return;
 					}
-					OC.Notification.showTemporary(t('files', 'Could not create folder "{dir}"', {dir: fullPath}));
+					OC.Notification.show(t('files', 'Could not create folder "{dir}"', {dir: fullPath}), {type: 'error'});
 					deferred.reject();
 				});
 			}, function() {
@@ -490,6 +552,12 @@ OC.Uploader.prototype = _.extend({
 		});
 	},
 
+	confirmBeforeUnload: function() {
+		if (this._uploading) {
+			return t('files', 'This will stop your current uploads.')
+		}
+	},
+
 	/**
 	 * Show conflict for the given file object
 	 *
@@ -508,9 +576,10 @@ OC.Uploader.prototype = _.extend({
 			return;
 		}
 		// retrieve more info about this file
-		this.filesClient.getFileInfo(fileUpload.getFullPath()).then(function(status, fileInfo) {
+		this.filesClient.getFileInfo(fileUpload.getFullFilePath()).then(function(status, fileInfo) {
 			var original = fileInfo;
 			var replacement = file;
+			original.directory = original.path;
 			OC.dialogs.fileexists(fileUpload, original, replacement, self);
 		});
 	},
@@ -528,7 +597,6 @@ OC.Uploader.prototype = _.extend({
 	 * Clear uploads
 	 */
 	clear: function() {
-		this._uploads = {};
 		this._knownDirs = {};
 	},
 	/**
@@ -547,24 +615,22 @@ OC.Uploader.prototype = _.extend({
 		return null;
 	},
 
-	showUploadCancelMessage: _.debounce(function() {
-		OC.Notification.showTemporary(t('files', 'Upload cancelled.'), {timeout: 10});
-	}, 500),
 	/**
-	 * Checks the currently known uploads.
-	 * returns true if any hxr has the state 'pending'
-	 * @returns {boolean}
+	 * Removes an upload from the list of known uploads.
+	 *
+	 * @param {OC.FileUpload} upload the upload to remove.
 	 */
-	isProcessing:function() {
-		var count = 0;
+	removeUpload: function(upload) {
+		if (!upload || !upload.data || !upload.data.uploadId) {
+			return;
+		}
 
-		jQuery.each(this._uploads, function(i, upload) {
-			if (upload.isPending()) {
-				count++;
-			}
-		});
-		return count > 0;
+		delete this._uploads[upload.data.uploadId];
 	},
+
+	showUploadCancelMessage: _.debounce(function() {
+		OC.Notification.show(t('files', 'Upload cancelled.'), {timeout : 7, type: 'error'});
+	}, 500),
 	/**
 	 * callback for the conflicts dialog
 	 */
@@ -629,8 +695,8 @@ OC.Uploader.prototype = _.extend({
 		// resubmit upload
 		this.submitUploads([upload]);
 	},
-	_trace:false, //TODO implement log handler for JS per class?
-	log:function(caption, e, data) {
+	_trace: false, //TODO implement log handler for JS per class?
+	log: function(caption, e, data) {
 		if (this._trace) {
 			console.log(caption);
 			console.log(data);
@@ -700,6 +766,27 @@ OC.Uploader.prototype = _.extend({
 		});
 	},
 
+	_updateProgressBarOnUploadStop: function() {
+		if (this._pendingUploadDoneCount === 0) {
+			// All the uploads ended and there is no pending operation, so hide
+			// the progress bar.
+			// Note that this happens here only with non-chunked uploads; if the
+			// upload was chunked then this will have been executed after all
+			// the uploads ended but before the upload done handler that reduces
+			// the pending operation count was executed.
+			this._hideProgressBar();
+
+			return;
+		}
+
+		$('#uploadprogressbar .label .mobile').text(t('files', '…'));
+		$('#uploadprogressbar .label .desktop').text(t('files', 'Processing files …'));
+
+		// Nothing is being uploaded at this point, and the pending operations
+		// can not be cancelled, so the cancel button should be hidden.
+		$('#uploadprogresswrapper .stop').fadeOut();
+	},
+
 	_showProgressBar: function() {
 		$('#uploadprogressbar').fadeIn();
 		this.$uploadEl.trigger(new $.Event('resized'));
@@ -739,6 +826,13 @@ OC.Uploader.prototype = _.extend({
 
 		this.fileList = options.fileList;
 		this.filesClient = options.filesClient || OC.Files.getClient();
+		this.davClient = new OC.Files.Client({
+			host: this.filesClient.getHost(),
+			root: OC.linkToRemoteBase('dav'),
+			useHTTPS: OC.getProtocol() === 'https',
+			userName: this.filesClient.getUserName(),
+			password: this.filesClient.getPassword()
+		});
 
 		$uploadEl = $($uploadEl);
 		this.$uploadEl = $uploadEl;
@@ -909,7 +1003,8 @@ OC.Uploader.prototype = _.extend({
 				start: function(e) {
 					self.log('start', e, null);
 					//hide the tooltip otherwise it covers the progress bar
-					$('#upload').tipsy('hide');
+					$('#upload').tooltip('hide');
+					self._uploading = true;
 				},
 				fail: function(e, data) {
 					var upload = self.getUpload(data);
@@ -919,6 +1014,8 @@ OC.Uploader.prototype = _.extend({
 					}
 					self.log('fail', e, upload);
 
+					self.removeUpload(upload);
+
 					if (data.textStatus === 'abort') {
 						self.showUploadCancelMessage();
 					} else if (status === 412) {
@@ -926,23 +1023,26 @@ OC.Uploader.prototype = _.extend({
 						self.showConflict(upload);
 					} else if (status === 404) {
 						// target folder does not exist any more
-						OC.Notification.showTemporary(
-							t('files', 'Target folder "{dir}" does not exist any more', {dir: upload.getFullPath()})
-						);
+						OC.Notification.show(t('files', 'Target folder "{dir}" does not exist any more', {dir: upload.getFullPath()} ), {type: 'error'});
 						self.cancelUploads();
-					} else if (status === 507) {
+					} else if (data.textStatus === 'notenoughspace') {
 						// not enough space
-						OC.Notification.showTemporary(
-							t('files', 'Not enough free space')
-						);
+						OC.Notification.show(t('files', 'Not enough free space'), {type: 'error'});
 						self.cancelUploads();
 					} else {
 						// HTTP connection problem or other error
-						OC.Notification.showTemporary(data.errorThrown, {timeout: 10});
+						var message = t('files', 'An unknown error has occurred');
+						if (upload) {
+							var response = upload.getResponse();
+							if (response) {
+								message = response.message;
+							}
+						}
+						OC.Notification.show(message || data.errorThrown, {type: 'error'});
 					}
 
 					if (upload) {
-						upload.deleteUpload();
+						upload.fail();
 					}
 				},
 				/**
@@ -955,6 +1055,8 @@ OC.Uploader.prototype = _.extend({
 					var that = $(this);
 					self.log('done', e, upload);
 
+					self.removeUpload(upload);
+
 					var status = upload.getResponseStatus();
 					if (status < 200 || status >= 300) {
 						// trigger fail handler
@@ -970,15 +1072,22 @@ OC.Uploader.prototype = _.extend({
 				 */
 				stop: function(e, data) {
 					self.log('stop', e, data);
+					self._uploading = false;
 				}
 			};
 
+			if (options.maxChunkSize) {
+				this.fileUploadParam.maxChunkSize = options.maxChunkSize;
+			}
+
 			// initialize jquery fileupload (blueimp)
 			var fileupload = this.$uploadEl.fileupload(this.fileUploadParam);
 
 			if (this._supportAjaxUploadWithProgress()) {
 				//remaining time
-				var lastUpdate, lastSize, bufferSize, buffer, bufferIndex, bufferTotal;
+				var lastUpdate, lastSize, bufferSize, buffer, bufferIndex, bufferIndex2, bufferTotal;
+
+				var dragging = false;
 
 				// add progress handlers
 				fileupload.on('fileuploadadd', function(e, data) {
@@ -993,11 +1102,11 @@ OC.Uploader.prototype = _.extend({
 					$('#uploadprogressbar').progressbar({value: 0});
 					$('#uploadprogressbar .ui-progressbar-value').
 						html('<em class="label inner"><span class="desktop">'
-							+ t('files', 'Uploading...')
+							+ t('files', 'Uploading …')
 							+ '</span><span class="mobile">'
-							+ t('files', '...')
+							+ t('files', '…')
 							+ '</span></em>');
-                    $('#uploadprogressbar').tipsy({gravity:'n', fade:true, live:true});
+					$('#uploadprogressbar').tooltip({placement: 'bottom'});
 					self._showProgressBar();
 					// initial remaining time variables
 					lastUpdate   = new Date().getTime();
@@ -1005,6 +1114,7 @@ OC.Uploader.prototype = _.extend({
 					bufferSize   = 20;
 					buffer       = [];
 					bufferIndex  = 0;
+					bufferIndex2 = 0;
 					bufferTotal  = 0;
 					for(var i = 0; i < bufferSize; i++){
 						buffer[i]  = 0;
@@ -1024,52 +1134,30 @@ OC.Uploader.prototype = _.extend({
 					lastUpdate = thisUpdate;
 					var diffSize = data.loaded - lastSize;
 					lastSize = data.loaded;
-					diffSize = diffSize / diffUpdate; // apply timing factor, eg. 1MiB/2s = 0.5MiB/s, unit is byte
+					diffSize = diffSize / diffUpdate; // apply timing factor, eg. 1MiB/2s = 0.5MiB/s, unit is byte per second
 					var remainingSeconds = ((data.total - data.loaded) / diffSize);
 					if(remainingSeconds >= 0) {
 						bufferTotal = bufferTotal - (buffer[bufferIndex]) + remainingSeconds;
 						buffer[bufferIndex] = remainingSeconds; //buffer to make it smoother
 						bufferIndex = (bufferIndex + 1) % bufferSize;
+						bufferIndex2++;
 					}
-					if(buffer[bufferSize - 1] === 0){
-						var smoothRemainingSeconds = (bufferTotal / bufferIndex);
-					}else{
-						var smoothRemainingSeconds = (bufferTotal / bufferSize);
-					}
-					var date = new Date(smoothRemainingSeconds * 1000);
-					var timeStringDesktop = "";
-					var timeStringMobile = "";
-					if(date.getUTCHours() > 0){
-						timeStringDesktop = n('files', '{hours}:{minutes}:{seconds} hour left', '{hours}:{minutes}:{seconds} hours left', date.getUTCHours(), {
-							hours:date.getUTCHours(),
-							minutes: ('0' + date.getUTCMinutes()).slice(-2),
-							seconds: ('0' + date.getUTCSeconds()).slice(-2)
-						});
-						timeStringMobile = t('files', '{hours}:{minutes}h' , {
-							hours:date.getUTCHours(),
-							minutes: ('0' + date.getUTCMinutes()).slice(-2),
-							seconds: ('0' + date.getUTCSeconds()).slice(-2)
-						});
-					} else if(date.getUTCMinutes() > 0){
-						timeStringDesktop = n('files', '{minutes}:{seconds} minute left', '{minutes}:{seconds} minutes left', date.getUTCMinutes(), {
-							minutes: date.getUTCMinutes(),
-							seconds: ('0' + date.getUTCSeconds()).slice(-2)
-						});
-						timeStringMobile = t('files', '{minutes}:{seconds}m' , {
-							minutes: date.getUTCMinutes(),
-							seconds: ('0' + date.getUTCSeconds()).slice(-2)
-						});
-					} else if(date.getUTCSeconds() > 0){
-						timeStringDesktop = n('files', '{seconds} second left', '{seconds} seconds left', date.getUTCSeconds(), {
-							seconds: date.getUTCSeconds()
-						});
-						timeStringMobile = t('files', '{seconds}s' , {seconds: date.getUTCSeconds()});
+					var smoothRemainingSeconds;
+					if (bufferIndex2 > 0 && bufferIndex2 < 20) {
+						smoothRemainingSeconds = bufferTotal / bufferIndex2;
+					} else if (bufferSize > 0) {
+						smoothRemainingSeconds = bufferTotal / bufferSize;
 					} else {
-						timeStringDesktop = t('files', 'Any moment now...');
-						timeStringMobile = t('files', 'Soon...');
+						smoothRemainingSeconds = 1;
 					}
-					$('#uploadprogressbar .label .mobile').text(timeStringMobile);
-					$('#uploadprogressbar .label .desktop').text(timeStringDesktop);
+
+					var h = moment.duration(smoothRemainingSeconds, "seconds").humanize();
+					if (!(smoothRemainingSeconds >= 0 && smoothRemainingSeconds < 14400)) {
+						// show "Uploading ..." for durations longer than 4 hours
+						h = t('files', 'Uploading …');
+					}
+					$('#uploadprogressbar .label .mobile').text(h);
+					$('#uploadprogressbar .label .desktop').text(h);
 					$('#uploadprogressbar').attr('original-title',
 						t('files', '{loadedSize} of {totalSize} ({bitrate})' , {
 							loadedSize: humanFileSize(data.loaded),
@@ -1084,34 +1172,15 @@ OC.Uploader.prototype = _.extend({
 					self.log('progress handle fileuploadstop', e, data);
 
 					self.clear();
-					self._hideProgressBar();
+					self._updateProgressBarOnUploadStop();
 					self.trigger('stop', e, data);
 				});
 				fileupload.on('fileuploadfail', function(e, data) {
 					self.log('progress handle fileuploadfail', e, data);
-					//if user pressed cancel hide upload progress bar and cancel button
-					if (data.errorThrown === 'abort') {
-						self._hideProgressBar();
-					}
 					self.trigger('fail', e, data);
 				});
-				var disableDropState = function() {
-					$('#app-content').removeClass('file-drag');
-					$('.dropping-to-dir').removeClass('dropping-to-dir');
-					$('.dir-drop').removeClass('dir-drop');
-					$('.icon-filetype-folder-drag-accept').removeClass('icon-filetype-folder-drag-accept');
-				};
-				var disableClassOnFirefox = _.debounce(function() {
-					disableDropState();
-				}, 100);
 				fileupload.on('fileuploaddragover', function(e){
 					$('#app-content').addClass('file-drag');
-					// dropping a folder in firefox doesn't cause a drop event
-					// this is simulated by simply invoke disabling all classes
-					// once no dragover event isn't noticed anymore
-					if ($.browser['mozilla']) {
-						disableClassOnFirefox();
-					}
 					$('#emptycontent .icon-folder').addClass('icon-filetype-folder-drag-accept');
 
 					var filerow = $(e.delegatedEvent.target).closest('tr');
@@ -1127,47 +1196,97 @@ OC.Uploader.prototype = _.extend({
 						filerow.addClass('dropping-to-dir');
 						filerow.find('.thumbnail').addClass('icon-filetype-folder-drag-accept');
 					}
+
+					dragging = true;
 				});
-				fileupload.on('fileuploaddragleave fileuploaddrop', function (){
+
+				var disableDropState = function() {
 					$('#app-content').removeClass('file-drag');
 					$('.dropping-to-dir').removeClass('dropping-to-dir');
 					$('.dir-drop').removeClass('dir-drop');
 					$('.icon-filetype-folder-drag-accept').removeClass('icon-filetype-folder-drag-accept');
+
+					dragging = false;
+				};
+
+				fileupload.on('fileuploaddragleave fileuploaddrop', disableDropState);
+
+				// In some browsers the "drop" event can be triggered with no
+				// files even if the "dragover" event seemed to suggest that a
+				// file was being dragged (and thus caused "fileuploaddragover"
+				// to be triggered).
+				fileupload.on('fileuploaddropnofiles', function() {
+					if (!dragging) {
+						return;
+					}
+
+					disableDropState();
+
+					OC.Notification.show(t('files', 'Uploading that item is not supported'), {type: 'error'});
 				});
 
 				fileupload.on('fileuploadchunksend', function(e, data) {
 					// modify the request to adjust it to our own chunking
 					var upload = self.getUpload(data);
 					var range = data.contentRange.split(' ')[1];
-					var chunkId = range.split('/')[0];
+					var chunkId = range.split('/')[0].split('-')[0];
 					data.url = OC.getRootPath() +
 						'/remote.php/dav/uploads' +
-						'/' + encodeURIComponent(OC.getCurrentUser().uid) +
-						'/' + encodeURIComponent(upload.getId()) +
-						'/' + encodeURIComponent(chunkId);
+						'/' + OC.getCurrentUser().uid +
+						'/' + upload.getId() +
+						'/' + chunkId;
 					delete data.contentRange;
 					delete data.headers['Content-Range'];
 				});
 				fileupload.on('fileuploaddone', function(e, data) {
 					var upload = self.getUpload(data);
+
+					self._pendingUploadDoneCount++;
+
 					upload.done().then(function() {
+						self._pendingUploadDoneCount--;
+						if (Object.keys(self._uploads).length === 0 && self._pendingUploadDoneCount === 0) {
+							// All the uploads ended and there is no pending
+							// operation, so hide the progress bar.
+							// Note that this happens here only with chunked
+							// uploads; if the upload was non-chunked then this
+							// handler is immediately executed, before the
+							// jQuery upload done handler that removes the
+							// upload from the list, and thus at this point
+							// there is still at least one upload that has not
+							// ended (although the upload stop handler is always
+							// executed after all the uploads have ended, which
+							// hides the progress bar in that case).
+							self._hideProgressBar();
+						}
+
 						self.trigger('done', e, upload);
+					}).fail(function(status, response) {
+						var message = response.message;
+						if (status === 507) {
+							// not enough space
+							OC.Notification.show(message || t('files', 'Not enough free space'), {type: 'error'});
+							self.cancelUploads();
+						} else if (status === 409) {
+							OC.Notification.show(message || t('files', 'Target folder does not exist any more'), {type: 'error'});
+						} else {
+							OC.Notification.show(message || t('files', 'Error when assembling chunks, status code {status}', {status: status}), {type: 'error'});
+						}
+						self.trigger('fail', e, data);
 					});
 				});
 				fileupload.on('fileuploaddrop', function(e, data) {
 					self.trigger('drop', e, data);
+					if (e.isPropagationStopped()) {
+						return false;
+					}
 				});
-
+			}
+			window.onbeforeunload = function() {
+				return self.confirmBeforeUnload();
 			}
 		}
 
-		// warn user not to leave the page while upload is in progress
-		$(window).on('beforeunload', function(e) {
-			if (self.isProcessing()) {
-				return t('files', 'File upload is in progress. Leaving the page now will cancel the upload.');
-			}
-		});
-
 		//add multiply file upload attribute to all browsers except konqueror (which crashes when it's used)
 		if (navigator.userAgent.search(/konqueror/i) === -1) {
 			this.$uploadEl.attr('multiple', 'multiple');
@@ -1176,5 +1295,3 @@ OC.Uploader.prototype = _.extend({
 		return this.fileUploadParam;
 	}
 }, OC.Backbone.Events);
-
-
