@@ -34,6 +34,7 @@
 
 namespace OCA\Files_External\Lib\Storage;
 
+use Icewind\SMB\BasicAuth;
 use Icewind\SMB\Exception\AlreadyExistsException;
 use Icewind\SMB\Exception\ConnectException;
 use Icewind\SMB\Exception\Exception;
@@ -42,8 +43,9 @@ use Icewind\SMB\Exception\InvalidArgumentException;
 use Icewind\SMB\Exception\NotFoundException;
 use Icewind\SMB\Exception\TimedOutException;
 use Icewind\SMB\IFileInfo;
-use Icewind\SMB\NativeServer;
-use Icewind\SMB\Server;
+use Icewind\SMB\Native\NativeServer;
+use Icewind\SMB\ServerFactory;
+use Icewind\SMB\System;
 use Icewind\Streams\CallbackWrapper;
 use Icewind\Streams\IteratorDirectory;
 use OC\Cache\CappedMemoryCache;
@@ -53,18 +55,18 @@ use OCA\Files_External\Lib\Notify\SMBNotifyHandler;
 use OCP\Files\Notify\IChange;
 use OCP\Files\Notify\IRenameChange;
 use OCP\Files\Storage\INotifyStorage;
+use OCP\Files\StorageAuthException;
 use OCP\Files\StorageNotAvailableException;
 use OCP\ILogger;
-use OCP\Util;
 
 class SMB extends Common implements INotifyStorage {
 	/**
-	 * @var \Icewind\SMB\Server
+	 * @var \Icewind\SMB\IServer
 	 */
 	protected $server;
 
 	/**
-	 * @var \Icewind\SMB\Share
+	 * @var \Icewind\SMB\IShare
 	 */
 	protected $share;
 
@@ -74,33 +76,55 @@ class SMB extends Common implements INotifyStorage {
 	protected $root;
 
 	/**
-	 * @var \Icewind\SMB\FileInfo[]
+	 * @var \Icewind\SMB\IFileInfo[]
 	 */
 	protected $statCache;
 
-	public function __construct($params) {
-		if (isset($params['host']) && isset($params['user']) && isset($params['password']) && isset($params['share'])) {
-			if (Server::NativeAvailable()) {
-				$this->server = new NativeServer($params['host'], $params['user'], $params['password']);
-			} else {
-				$this->server = new Server($params['host'], $params['user'], $params['password']);
-			}
-			$this->share = $this->server->getShare(trim($params['share'], '/'));
+	/** @var ILogger */
+	protected $logger;
 
-			$this->root = isset($params['root']) ? $params['root'] : '/';
-			if (!$this->root || $this->root[0] !== '/') {
-				$this->root = '/' . $this->root;
-			}
-			if (substr($this->root, -1, 1) !== '/') {
-				$this->root .= '/';
-			}
-		} else {
-			throw new \Exception('Invalid configuration');
+	public function __construct($params) {
+		if (!isset($params['host'])) {
+			throw new \Exception('Invalid configuration, no host provided');
 		}
+
+		if (isset($params['auth'])) {
+			$auth = $params['auth'];
+		} else if (isset($params['user']) && isset($params['password']) && isset($params['share'])) {
+			list($workgroup, $user) = $this->splitUser($params['user']);
+			$auth = new BasicAuth($user, $workgroup, $params['password']);
+		} else {
+			throw new \Exception('Invalid configuration, no credentials provided');
+		}
+
+		if (isset($params['logger'])) {
+			$this->logger = $params['logger'];
+		} else {
+			$this->logger = \OC::$server->getLogger();
+		}
+
+		$serverFactory = new ServerFactory();
+		$this->server = $serverFactory->createServer($params['host'], $auth);
+		$this->share = $this->server->getShare(trim($params['share'], '/'));
+
+		$this->root = $params['root'] ?? '/';
+		$this->root = '/' . ltrim($this->root, '/');
+		$this->root = rtrim($this->root, '/') . '/';
+
 		$this->statCache = new CappedMemoryCache();
 		parent::__construct($params);
 	}
 
+	private function splitUser($user) {
+		if (strpos($user, '/')) {
+			return explode('/', $user, 2);
+		} elseif (strpos($user, '\\')) {
+			return explode('\\', $user);
+		} else {
+			return [null, $user];
+		}
+	}
+
 	/**
 	 * @return string
 	 */
@@ -108,7 +132,7 @@ class SMB extends Common implements INotifyStorage {
 		// FIXME: double slash to keep compatible with the old storage ids,
 		// failure to do so will lead to creation of a new storage id and
 		// loss of shares from the storage
-		return 'smb::' . $this->server->getUser() . '@' . $this->server->getHost() . '//' . $this->share->getName() . '/' . $this->root;
+		return 'smb::' . $this->server->getAuth()->getUsername() . '@' . $this->server->getHost() . '//' . $this->share->getName() . '/' . $this->root;
 	}
 
 	/**
@@ -132,7 +156,7 @@ class SMB extends Common implements INotifyStorage {
 	/**
 	 * @param string $path
 	 * @return \Icewind\SMB\IFileInfo
-	 * @throws StorageNotAvailableException
+	 * @throws StorageAuthException
 	 */
 	protected function getFileInfo($path) {
 		try {
@@ -142,11 +166,26 @@ class SMB extends Common implements INotifyStorage {
 			}
 			return $this->statCache[$path];
 		} catch (ConnectException $e) {
-			\OC::$server->getLogger()->logException($e, ['message' => 'Error while getting file info']);
-			throw new StorageNotAvailableException($e->getMessage(), $e->getCode(), $e);
+			$this->throwUnavailable($e);
+		} catch (ForbiddenException $e) {
+			// with php-smbclient, this exceptions is thrown when the provided password is invalid.
+			// Possible is also ForbiddenException with a different error code, so we check it.
+			if($e->getCode() === 1) {
+				$this->throwUnavailable($e);
+			}
+			throw $e;
 		}
 	}
 
+	/**
+	 * @param \Exception $e
+	 * @throws StorageAuthException
+	 */
+	protected function throwUnavailable(\Exception $e) {
+		$this->logger->logException($e, ['message' => 'Error while getting file info']);
+		throw new StorageAuthException($e->getMessage(), $e);
+	}
+
 	/**
 	 * @param string $path
 	 * @return \Icewind\SMB\IFileInfo[]
@@ -162,19 +201,19 @@ class SMB extends Common implements INotifyStorage {
 			return array_filter($files, function (IFileInfo $file) {
 				try {
 					if ($file->isHidden()) {
-						\OC::$server->getLogger()->debug('hiding hidden file ' . $file->getName());
+						$this->logger->debug('hiding hidden file ' . $file->getName());
 					}
 					return !$file->isHidden();
 				} catch (ForbiddenException $e) {
-					\OC::$server->getLogger()->logException($e, ['level' => Util::DEBUG, 'message' => 'Hiding forbidden entry ' . $file->getName()]);
+					$this->logger->logException($e, ['level' => ILogger::DEBUG, 'message' => 'Hiding forbidden entry ' . $file->getName()]);
 					return false;
 				} catch (NotFoundException $e) {
-					\OC::$server->getLogger()->logException($e, ['level' => Util::DEBUG, 'message' => 'Hiding not found entry ' . $file->getName()]);
+					$this->logger->logException($e, ['level' => ILogger::DEBUG, 'message' => 'Hiding not found entry ' . $file->getName()]);
 					return false;
 				}
 			});
 		} catch (ConnectException $e) {
-			\OC::$server->getLogger()->logException($e, ['message' => 'Error while getting folder content']);
+			$this->logger->logException($e, ['message' => 'Error while getting folder content']);
 			throw new StorageNotAvailableException($e->getMessage(), $e->getCode(), $e);
 		}
 	}
@@ -217,7 +256,7 @@ class SMB extends Common implements INotifyStorage {
 				$this->remove($target);
 				$result = $this->share->rename($absoluteSource, $absoluteTarget, false);
 			} else {
-				\OC::$server->getLogger()->logException($e, ['level' => ILogger::WARN]);
+				$this->logger->logException($e, ['level' => ILogger::WARN]);
 				return false;
 			}
 		} catch (InvalidArgumentException $e) {
@@ -225,11 +264,11 @@ class SMB extends Common implements INotifyStorage {
 				$this->remove($target);
 				$result = $this->share->rename($absoluteSource, $absoluteTarget, false);
 			} else {
-				\OC::$server->getLogger()->logException($e, ['level' => ILogger::WARN]);
+				$this->logger->logException($e, ['level' => ILogger::WARN]);
 				return false;
 			}
 		} catch (\Exception $e) {
-			\OC::$server->getLogger()->logException($e, ['level' => Util::WARN]);
+			$this->logger->logException($e, ['level' => ILogger::WARN]);
 			return false;
 		}
 		unset($this->statCache[$absoluteSource], $this->statCache[$absoluteTarget]);
@@ -318,7 +357,7 @@ class SMB extends Common implements INotifyStorage {
 		} catch (ForbiddenException $e) {
 			return false;
 		} catch (ConnectException $e) {
-			\OC::$server->getLogger()->logException($e, ['message' => 'Error while deleting file']);
+			$this->logger->logException($e, ['message' => 'Error while deleting file']);
 			throw new StorageNotAvailableException($e->getMessage(), $e->getCode(), $e);
 		}
 	}
@@ -387,7 +426,7 @@ class SMB extends Common implements INotifyStorage {
 						if (!$this->isCreatable(dirname($path))) {
 							return false;
 						}
-						$tmpFile = \OCP\Files::tmpFile($ext);
+						$tmpFile = \OC::$server->getTempManager()->getTemporaryFile($ext);
 					}
 					$source = fopen($tmpFile, $mode);
 					$share = $this->share;
@@ -403,7 +442,7 @@ class SMB extends Common implements INotifyStorage {
 		} catch (ForbiddenException $e) {
 			return false;
 		} catch (ConnectException $e) {
-			\OC::$server->getLogger()->logException($e, ['message' => 'Error while opening file']);
+			$this->logger->logException($e, ['message' => 'Error while opening file']);
 			throw new StorageNotAvailableException($e->getMessage(), $e->getCode(), $e);
 		}
 	}
@@ -430,7 +469,7 @@ class SMB extends Common implements INotifyStorage {
 		} catch (ForbiddenException $e) {
 			return false;
 		} catch (ConnectException $e) {
-			\OC::$server->getLogger()->logException($e, ['message' => 'Error while removing folder']);
+			$this->logger->logException($e, ['message' => 'Error while removing folder']);
 			throw new StorageNotAvailableException($e->getMessage(), $e->getCode(), $e);
 		}
 	}
@@ -444,7 +483,7 @@ class SMB extends Common implements INotifyStorage {
 			}
 			return false;
 		} catch (ConnectException $e) {
-			\OC::$server->getLogger()->logException($e, ['message' => 'Error while creating file']);
+			$this->logger->logException($e, ['message' => 'Error while creating file']);
 			throw new StorageNotAvailableException($e->getMessage(), $e->getCode(), $e);
 		}
 	}
@@ -480,7 +519,7 @@ class SMB extends Common implements INotifyStorage {
 			$this->share->mkdir($path);
 			return true;
 		} catch (ConnectException $e) {
-			\OC::$server->getLogger()->logException($e, ['message' => 'Error while creating folder']);
+			$this->logger->logException($e, ['message' => 'Error while creating folder']);
 			throw new StorageNotAvailableException($e->getMessage(), $e->getCode(), $e);
 		} catch (Exception $e) {
 			return false;
@@ -541,7 +580,7 @@ class SMB extends Common implements INotifyStorage {
 	public static function checkDependencies() {
 		return (
 			(bool)\OC_Helper::findBinaryPath('smbclient')
-			|| Server::NativeAvailable()
+			|| NativeServer::available(new System())
 		) ? true : ['smbclient'];
 	}
 
@@ -554,7 +593,7 @@ class SMB extends Common implements INotifyStorage {
 		try {
 			return parent::test();
 		} catch (Exception $e) {
-			\OC::$server->getLogger()->logException($e);
+			$this->logger->logException($e);
 			return false;
 		}
 	}
