@@ -4,8 +4,6 @@ OC.L10N.register(
     "Could not revert: %s" : "Klarte ikkje å tilbakestilla: %s",
     "Versions" : "Utgåver",
     "Failed to revert {file} to revision {timestamp}." : "Klarte ikkje å tilbakestilla {file} til utgåva {timestamp}.",
-    "Restore" : "Gjenopprett",
-    "More versions..." : "Fleire utgåver …",
-    "No other versions available" : "Ingen andre utgåver tilgjengeleg"
+    "Restore" : "Gjenopprett"
 },
 "nplurals=2; plural=(n != 1);");
