@@ -8,13 +8,13 @@
  * @author Björn Schießle <bjoern@schiessle.org>
  * @author Carlos Damken <carlos@damken.com>
  * @author Felix Moeller <mail@felixmoeller.de>
- * @author Georg Ehrke <georg@owncloud.com>
  * @author Joas Schilling <coding@schilljs.com>
  * @author Jörn Friedrich Dreyer <jfd@butonic.de>
  * @author Lukas Reschke <lukas@statuscode.ch>
  * @author Morris Jobke <hey@morrisjobke.de>
  * @author Robin Appelman <robin@icewind.nl>
  * @author Robin McCorkell <robin@mccorkell.me.uk>
+ * @author Roeland Jago Douma <roeland@famdouma.nl>
  * @author Thomas Müller <thomas.mueller@tmit.eu>
  * @author Victor Dubiniuk <dubiniuk@owncloud.com>
  * @author Vincent Petry <pvince81@owncloud.com>
@@ -47,6 +47,8 @@ use OC\Files\Filesystem;
 use OC\Files\View;
 use OCA\Files_Versions\AppInfo\Application;
 use OCA\Files_Versions\Command\Expire;
+use OCA\Files_Versions\Events\CreateVersionEvent;
+use OCA\Files_Versions\Versions\IVersionManager;
 use OCP\Files\NotFoundException;
 use OCP\Lock\ILockingProvider;
 use OCP\User;
@@ -102,7 +104,7 @@ class Storage {
 			$uid = User::getUser();
 		}
 		Filesystem::initMountPoints($uid);
-		if ( $uid != User::getUser() ) {
+		if ( $uid !== User::getUser() ) {
 			$info = Filesystem::getFileInfo($filename);
 			$ownerView = new View('/'.$uid.'/files');
 			try {
@@ -161,41 +163,46 @@ class Storage {
 	 * store a new version of a file.
 	 */
 	public static function store($filename) {
-		if(\OCP\Config::getSystemValue('files_versions', Storage::DEFAULTENABLED)=='true') {
 
-			// if the file gets streamed we need to remove the .part extension
-			// to get the right target
-			$ext = pathinfo($filename, PATHINFO_EXTENSION);
-			if ($ext === 'part') {
-				$filename = substr($filename, 0, strlen($filename) - 5);
-			}
-
-			// we only handle existing files
-			if (! Filesystem::file_exists($filename) || Filesystem::is_dir($filename)) {
-				return false;
-			}
-
-			list($uid, $filename) = self::getUidAndFilename($filename);
-
-			$files_view = new View('/'.$uid .'/files');
-			$users_view = new View('/'.$uid);
-
-			// no use making versions for empty files
-			if ($files_view->filesize($filename) === 0) {
-				return false;
-			}
-
-			// create all parent folders
-			self::createMissingDirectories($filename, $users_view);
-
-			self::scheduleExpire($uid, $filename);
-
-			// store a new version of a file
-			$mtime = $users_view->filemtime('files/' . $filename);
-			$users_view->copy('files/' . $filename, 'files_versions/' . $filename . '.v' . $mtime);
-			// call getFileInfo to enforce a file cache entry for the new version
-			$users_view->getFileInfo('files_versions/' . $filename . '.v' . $mtime);
+		// if the file gets streamed we need to remove the .part extension
+		// to get the right target
+		$ext = pathinfo($filename, PATHINFO_EXTENSION);
+		if ($ext === 'part') {
+			$filename = substr($filename, 0, -5);
 		}
+
+		// we only handle existing files
+		if (! Filesystem::file_exists($filename) || Filesystem::is_dir($filename)) {
+			return false;
+		}
+
+		list($uid, $filename) = self::getUidAndFilename($filename);
+
+		$files_view = new View('/'.$uid .'/files');
+
+		$eventDispatcher = \OC::$server->getEventDispatcher();
+		$fileInfo = $files_view->getFileInfo($filename);
+		$id = $fileInfo->getId();
+		$nodes = \OC::$server->getRootFolder()->getUserFolder($uid)->getById($id);
+		foreach ($nodes as $node) {
+			$event = new CreateVersionEvent($node);
+			$eventDispatcher->dispatch('OCA\Files_Versions::createVersion', $event);
+			if ($event->shouldCreateVersion() === false) {
+				return false;
+			}
+		}
+
+		// no use making versions for empty files
+		if ($fileInfo->getSize() === 0) {
+			return false;
+		}
+
+		/** @var IVersionManager $versionManager */
+		$versionManager = \OC::$server->query(IVersionManager::class);
+		$userManager = \OC::$server->getUserManager();
+		$user = $userManager->get($uid);
+
+		$versionManager->createVersion($user, $fileInfo);
 	}
 
 
@@ -320,61 +327,68 @@ class Storage {
 	 */
 	public static function rollback($file, $revision) {
 
-		if(\OCP\Config::getSystemValue('files_versions', Storage::DEFAULTENABLED)=='true') {
-			// add expected leading slash
-			$file = '/' . ltrim($file, '/');
-			list($uid, $filename) = self::getUidAndFilename($file);
-			if ($uid === null || trim($filename, '/') === '') {
-				return false;
-			}
-
-			$users_view = new View('/'.$uid);
-			$files_view = new View('/'. User::getUser().'/files');
-
-			$versionCreated = false;
-
-			$fileInfo = $files_view->getFileInfo($file);
-
-			// check if user has the permissions to revert a version
-			if (!$fileInfo->isUpdateable()) {
-				return false;
-			}
-
-			//first create a new version
-			$version = 'files_versions'.$filename.'.v'.$users_view->filemtime('files'.$filename);
-			if (!$users_view->file_exists($version)) {
-				$users_view->copy('files'.$filename, 'files_versions'.$filename.'.v'.$users_view->filemtime('files'.$filename));
-				$versionCreated = true;
-			}
-
-			$fileToRestore =  'files_versions' . $filename . '.v' . $revision;
-
-			// Restore encrypted version of the old file for the newly restored file
-			// This has to happen manually here since the file is manually copied below
-			$oldVersion = $users_view->getFileInfo($fileToRestore)->getEncryptedVersion();
-			$oldFileInfo = $users_view->getFileInfo($fileToRestore);
-			$cache = $fileInfo->getStorage()->getCache();
-			$cache->update(
-				$fileInfo->getId(), [
-					'encrypted' => $oldVersion,
-					'encryptedVersion' => $oldVersion,
-					'size' => $oldFileInfo->getSize()
-				]
-			);
-
-			// rollback
-			if (self::copyFileContents($users_view, $fileToRestore, 'files' . $filename)) {
-				$files_view->touch($file, $revision);
-				Storage::scheduleExpire($uid, $file);
-				\OC_Hook::emit('\OCP\Versions', 'rollback', array(
-					'path' => $filename,
-					'revision' => $revision,
-				));
-				return true;
-			} else if ($versionCreated) {
-				self::deleteVersion($users_view, $version);
-			}
+		// add expected leading slash
+		$file = '/' . ltrim($file, '/');
+		list($uid, $filename) = self::getUidAndFilename($file);
+		if ($uid === null || trim($filename, '/') === '') {
+			return false;
 		}
+
+		// Fetch the userfolder to trigger view hooks
+		$userFolder = \OC::$server->getUserFolder($uid);
+
+		$users_view = new View('/'.$uid);
+		$files_view = new View('/'. User::getUser().'/files');
+
+		$versionCreated = false;
+
+		$fileInfo = $files_view->getFileInfo($file);
+
+		// check if user has the permissions to revert a version
+		if (!$fileInfo->isUpdateable()) {
+			return false;
+		}
+
+		//first create a new version
+		$version = 'files_versions'.$filename.'.v'.$users_view->filemtime('files'.$filename);
+		if (!$users_view->file_exists($version)) {
+			$users_view->copy('files'.$filename, 'files_versions'.$filename.'.v'.$users_view->filemtime('files'.$filename));
+			$versionCreated = true;
+		}
+
+		$fileToRestore =  'files_versions' . $filename . '.v' . $revision;
+
+		// Restore encrypted version of the old file for the newly restored file
+		// This has to happen manually here since the file is manually copied below
+		$oldVersion = $users_view->getFileInfo($fileToRestore)->getEncryptedVersion();
+		$oldFileInfo = $users_view->getFileInfo($fileToRestore);
+		$cache = $fileInfo->getStorage()->getCache();
+		$cache->update(
+			$fileInfo->getId(), [
+				'encrypted' => $oldVersion,
+				'encryptedVersion' => $oldVersion,
+				'size' => $oldFileInfo->getSize()
+			]
+		);
+
+		// rollback
+		if (self::copyFileContents($users_view, $fileToRestore, 'files' . $filename)) {
+			$files_view->touch($file, $revision);
+			Storage::scheduleExpire($uid, $file);
+
+			$node = $userFolder->get($file);
+
+			// TODO: move away from those legacy hooks!
+			\OC_Hook::emit('\OCP\Versions', 'rollback', array(
+				'path' => $filename,
+				'revision' => $revision,
+				'node' => $node,
+			));
+			return true;
+		} else if ($versionCreated) {
+			self::deleteVersion($users_view, $version);
+		}
+
 		return false;
 
 	}
@@ -462,11 +476,12 @@ class Storage {
 						if (empty($userFullPath)) {
 							$versions[$key]['preview'] = '';
 						} else {
-							$versions[$key]['preview'] = \OCP\Util::linkToRoute('core_ajax_versions_preview', array('file' => $userFullPath, 'version' => $timestamp));
+							$versions[$key]['preview'] = \OC::$server->getURLGenerator('files_version.Preview.getPreview', ['file' => $userFullPath, 'version' => $timestamp]);
 						}
 						$versions[$key]['path'] = Filesystem::normalizePath($pathinfo['dirname'] . '/' . $filename);
 						$versions[$key]['name'] = $versionedFile;
 						$versions[$key]['size'] = $view->filesize($dir . '/' . $entryName);
+						$versions[$key]['mimetype'] = \OC::$server->getMimeTypeDetector()->detectPath($versionedFile);
 					}
 				}
 			}
@@ -493,7 +508,7 @@ class Storage {
 
 		$toDelete = [];
 		foreach (array_reverse($versions['all']) as $key => $version) {
-			if (intval($version['version'])<$threshold) {
+			if ((int)$version['version'] <$threshold) {
 				$toDelete[$key] = $version;
 			} else {
 				//Versions are sorted by time - nothing mo to iterate.
@@ -556,7 +571,7 @@ class Storage {
 				$fileData = $file->getData();
 				$filePath = $dir . '/' . $fileData['name'];
 				if ($file['type'] === 'dir') {
-					array_push($dirs, $filePath);
+					$dirs[] = $filePath;
 				} else {
 					$versionsBegin = strrpos($filePath, '.v');
 					$relPathStart = strlen(self::VERSIONS_ROOT);
@@ -628,7 +643,7 @@ class Storage {
 
 		$interval = 1;
 		$step = Storage::$max_versions_per_interval[$interval]['step'];
-		if (Storage::$max_versions_per_interval[$interval]['intervalEndsAfter'] == -1) {
+		if (Storage::$max_versions_per_interval[$interval]['intervalEndsAfter'] === -1) {
 			$nextInterval = -1;
 		} else {
 			$nextInterval = $time - Storage::$max_versions_per_interval[$interval]['intervalEndsAfter'];
@@ -643,12 +658,12 @@ class Storage {
 		foreach ($versions as $key => $version) {
 			$newInterval = true;
 			while ($newInterval) {
-				if ($nextInterval == -1 || $prevTimestamp > $nextInterval) {
+				if ($nextInterval === -1 || $prevTimestamp > $nextInterval) {
 					if ($version['version'] > $nextVersion) {
 						//distance between two version too small, mark to delete
 						$toDelete[$key] = $version['path'] . '.v' . $version['version'];
 						$size += $version['size'];
-						\OCP\Util::writeLog('files_versions', 'Mark to expire '. $version['path'] .' next version should be ' . $nextVersion . " or smaller. (prevTimestamp: " . $prevTimestamp . "; step: " . $step, \OCP\Util::INFO);
+						\OC::$server->getLogger()->info('Mark to expire '. $version['path'] .' next version should be ' . $nextVersion . " or smaller. (prevTimestamp: " . $prevTimestamp . "; step: " . $step, ['app' => 'files_versions']);
 					} else {
 						$nextVersion = $version['version'] - $step;
 						$prevTimestamp = $version['version'];
@@ -658,7 +673,7 @@ class Storage {
 					$interval++;
 					$step = Storage::$max_versions_per_interval[$interval]['step'];
 					$nextVersion = $prevTimestamp - $step;
-					if (Storage::$max_versions_per_interval[$interval]['intervalEndsAfter'] == -1) {
+					if (Storage::$max_versions_per_interval[$interval]['intervalEndsAfter'] === -1) {
 						$nextInterval = -1;
 					} else {
 						$nextInterval = $time - Storage::$max_versions_per_interval[$interval]['intervalEndsAfter'];
@@ -677,7 +692,7 @@ class Storage {
 	 * @param string $uid owner of the file
 	 * @param string $fileName file/folder for which to schedule expiration
 	 */
-	private static function scheduleExpire($uid, $fileName) {
+	public static function scheduleExpire($uid, $fileName) {
 		// let the admin disable auto expire
 		$expiration = self::getExpiration();
 		if ($expiration->isEnabled()) {
@@ -687,30 +702,38 @@ class Storage {
 	}
 
 	/**
-	 * Expire versions which exceed the quota
+	 * Expire versions which exceed the quota.
 	 *
-	 * @param string $filename
+	 * This will setup the filesystem for the given user but will not
+	 * tear it down afterwards.
+	 *
+	 * @param string $filename path to file to expire
+	 * @param string $uid user for which to expire the version
 	 * @return bool|int|null
 	 */
-	public static function expire($filename) {
-		$config = \OC::$server->getConfig();
+	public static function expire($filename, $uid) {
 		$expiration = self::getExpiration();
 
-		if($config->getSystemValue('files_versions', Storage::DEFAULTENABLED)=='true' && $expiration->isEnabled()) {
+		if ($expiration->isEnabled()) {
+			// get available disk space for user
+			$user = \OC::$server->getUserManager()->get($uid);
+			if (is_null($user)) {
+				\OC::$server->getLogger()->error('Backends provided no user object for ' . $uid, ['app' => 'files_versions']);
+				throw new \OC\User\NoUserException('Backends provided no user object for ' . $uid);
+			}
+
+			\OC_Util::setupFS($uid);
 
 			if (!Filesystem::file_exists($filename)) {
 				return false;
 			}
 
-			list($uid, $filename) = self::getUidAndFilename($filename);
 			if (empty($filename)) {
 				// file maybe renamed or deleted
 				return false;
 			}
 			$versionsFileview = new View('/'.$uid.'/files_versions');
 
-			// get available disk space for user
-			$user = \OC::$server->getUserManager()->get($uid);
 			$softQuota = true;
 			$quota = $user->getQuota();
 			if ( $quota === null || $quota === 'none' ) {
@@ -764,12 +787,13 @@ class Storage {
 				$versionsSize = $versionsSize - $sizeOfDeletedVersions;
 			}
 
+			$logger = \OC::$server->getLogger();
 			foreach($toDelete as $key => $path) {
 				\OC_Hook::emit('\OCP\Versions', 'preDelete', array('path' => $path, 'trigger' => self::DELETE_TRIGGER_QUOTA_EXCEEDED));
 				self::deleteVersion($versionsFileview, $path);
 				\OC_Hook::emit('\OCP\Versions', 'delete', array('path' => $path, 'trigger' => self::DELETE_TRIGGER_QUOTA_EXCEEDED));
 				unset($allVersions[$key]); // update array with the versions we keep
-				\OCP\Util::writeLog('files_versions', "Expire: " . $path, \OCP\Util::INFO);
+				$logger->info('Expire: ' . $path, ['app' => 'files_versions']);
 			}
 
 			// Check if enough space is available after versions are rearranged.
@@ -785,7 +809,7 @@ class Storage {
 				\OC_Hook::emit('\OCP\Versions', 'preDelete', array('path' => $version['path'].'.v'.$version['version'], 'trigger' => self::DELETE_TRIGGER_QUOTA_EXCEEDED));
 				self::deleteVersion($versionsFileview, $version['path'] . '.v' . $version['version']);
 				\OC_Hook::emit('\OCP\Versions', 'delete', array('path' => $version['path'].'.v'.$version['version'], 'trigger' => self::DELETE_TRIGGER_QUOTA_EXCEEDED));
-				\OCP\Util::writeLog('files_versions', 'running out of space! Delete oldest version: ' . $version['path'].'.v'.$version['version'] , \OCP\Util::INFO);
+				\OC::$server->getLogger()->info('running out of space! Delete oldest version: ' . $version['path'].'.v'.$version['version'], ['app' => 'files_versions']);
 				$versionsSize -= $version['size'];
 				$availableSpace += $version['size'];
 				next($allVersions);
@@ -806,7 +830,7 @@ class Storage {
 	 * "files" folder
 	 * @param View $view view on data/user/
 	 */
-	private static function createMissingDirectories($filename, $view) {
+	public static function createMissingDirectories($filename, $view) {
 		$dirname = Filesystem::normalizePath(dirname($filename));
 		$dirParts = explode('/', $dirname);
 		$dir = "/files_versions";
@@ -826,7 +850,7 @@ class Storage {
 		if (is_null(self::$application)) {
 			self::$application = new Application();
 		}
-		return self::$application->getContainer()->query('Expiration');
+		return self::$application->getContainer()->query(Expiration::class);
 	}
 
 }
