@@ -8,8 +8,10 @@
  * @author Arthur Schiwon <blizzz@arthur-schiwon.de>
  * @author Bart Visscher <bartv@thisnet.nl>
  * @author Benjamin Diele <benjamin@diele.be>
+ * @author bline <scottbeck@gmail.com>
  * @author Christopher Schäpers <kondou@ts.unde.re>
  * @author Joas Schilling <coding@schilljs.com>
+ * @author Juan Pablo Villafáñez <jvillafanez@solidgear.es>
  * @author Jörn Friedrich Dreyer <jfd@butonic.de>
  * @author Lorenzo M. Catucci <lorenzo@sancho.ccd.uniroma2.it>
  * @author Lukas Reschke <lukas@statuscode.ch>
@@ -18,9 +20,10 @@
  * @author Morris Jobke <hey@morrisjobke.de>
  * @author Nicolas Grekas <nicolas.grekas@gmail.com>
  * @author Ralph Krimmel <rkrimme1@gwdg.de>
- * @author Renaud Fortier <Renaud.Fortier@fsaa.ulaval.ca>
  * @author Robin McCorkell <robin@mccorkell.me.uk>
  * @author Roger Szabo <roger.szabo@web.de>
+ * @author root <root@localhost.localdomain>
+ * @author Victor Dubiniuk <dubiniuk@owncloud.com>
  *
  * @license AGPL-3.0
  *
@@ -41,22 +44,24 @@
 namespace OCA\User_LDAP;
 
 use OC\HintException;
+use OC\Hooks\PublicEmitter;
 use OCA\User_LDAP\Exceptions\ConstraintViolationException;
-use OCA\User_LDAP\User\IUserTools;
 use OCA\User_LDAP\User\Manager;
 use OCA\User_LDAP\User\OfflineUser;
 use OCA\User_LDAP\Mapping\AbstractMapping;
-
 use OC\ServerNotAvailableException;
+use OCP\IConfig;
+use OCP\ILogger;
+use OCP\IUserManager;
 
 /**
  * Class Access
  * @package OCA\User_LDAP
  */
-class Access extends LDAPUtility implements IUserTools {
-	/**
-	 * @var \OCA\User_LDAP\Connection
-	 */
+class Access extends LDAPUtility {
+	const UUID_ATTRIBUTES = ['entryuuid', 'nsuniqueid', 'objectguid', 'guid', 'ipauniqueid'];
+
+	/** @var \OCA\User_LDAP\Connection */
 	public $connection;
 	/** @var Manager */
 	public $userManager;
@@ -83,19 +88,31 @@ class Access extends LDAPUtility implements IUserTools {
 	* @var AbstractMapping $userMapper
 	*/
 	protected $groupMapper;
-	
+
 	/**
 	 * @var \OCA\User_LDAP\Helper
 	 */
 	private $helper;
+	/** @var IConfig */
+	private $config;
+	/** @var IUserManager */
+	private $ncUserManager;
 
-	public function __construct(Connection $connection, ILDAPWrapper $ldap,
-		Manager $userManager, Helper $helper) {
+	public function __construct(
+		Connection $connection,
+		ILDAPWrapper $ldap,
+		Manager $userManager,
+		Helper $helper,
+		IConfig $config,
+		IUserManager $ncUserManager
+	) {
 		parent::__construct($ldap);
 		$this->connection = $connection;
 		$this->userManager = $userManager;
 		$this->userManager->setLdapAccess($this);
 		$this->helper = $helper;
+		$this->config = $config;
+		$this->ncUserManager = $ncUserManager;
 	}
 
 	/**
@@ -168,13 +185,13 @@ class Access extends LDAPUtility implements IUserTools {
 		if(!$this->checkConnection()) {
 			\OCP\Util::writeLog('user_ldap',
 				'No LDAP Connector assigned, access impossible for readAttribute.',
-				\OCP\Util::WARN);
+				ILogger::WARN);
 			return false;
 		}
 		$cr = $this->connection->getConnectionResource();
 		if(!$this->ldap->isResource($cr)) {
 			//LDAP not available
-			\OCP\Util::writeLog('user_ldap', 'LDAP resource not available.', \OCP\Util::DEBUG);
+			\OCP\Util::writeLog('user_ldap', 'LDAP resource not available.', ILogger::DEBUG);
 			return false;
 		}
 		//Cancel possibly running Paged Results operation, otherwise we run in
@@ -182,7 +199,7 @@ class Access extends LDAPUtility implements IUserTools {
 		$this->abandonPagedSearch();
 		// openLDAP requires that we init a new Paged Search. Not needed by AD,
 		// but does not hurt either.
-		$pagingSize = intval($this->connection->ldapPagingSize);
+		$pagingSize = (int)$this->connection->ldapPagingSize;
 		// 0 won't result in replies, small numbers may leave out groups
 		// (cf. #12306), 500 is default for paging and should work everywhere.
 		$maxResults = $pagingSize > 20 ? $pagingSize : 500;
@@ -229,7 +246,7 @@ class Access extends LDAPUtility implements IUserTools {
 			}
 		} while($isRangeRequest);
 
-		\OCP\Util::writeLog('user_ldap', 'Requested attribute '.$attr.' not found for '.$dn, \OCP\Util::DEBUG);
+		\OCP\Util::writeLog('user_ldap', 'Requested attribute '.$attr.' not found for '.$dn, ILogger::DEBUG);
 		return false;
 	}
 
@@ -253,13 +270,13 @@ class Access extends LDAPUtility implements IUserTools {
 		if (!$this->ldap->isResource($rr)) {
 			if ($attribute !== '') {
 				//do not throw this message on userExists check, irritates
-				\OCP\Util::writeLog('user_ldap', 'readAttribute failed for DN ' . $dn, \OCP\Util::DEBUG);
+				\OCP\Util::writeLog('user_ldap', 'readAttribute failed for DN ' . $dn, ILogger::DEBUG);
 			}
 			//in case an error occurs , e.g. object does not exist
 			return false;
 		}
 		if ($attribute === '' && ($filter === 'objectclass=*' || $this->invokeLDAPMethod('countEntries', $cr, $rr) === 1)) {
-			\OCP\Util::writeLog('user_ldap', 'readAttribute: ' . $dn . ' found', \OCP\Util::DEBUG);
+			\OCP\Util::writeLog('user_ldap', 'readAttribute: ' . $dn . ' found', ILogger::DEBUG);
 			return true;
 		}
 		$er = $this->invokeLDAPMethod('firstEntry', $cr, $rr);
@@ -339,13 +356,13 @@ class Access extends LDAPUtility implements IUserTools {
 	 * @throws \Exception
 	 */
 	public function setPassword($userDN, $password) {
-		if(intval($this->connection->turnOnPasswordChange) !== 1) {
+		if((int)$this->connection->turnOnPasswordChange !== 1) {
 			throw new \Exception('LDAP password changes are disabled.');
 		}
 		$cr = $this->connection->getConnectionResource();
 		if(!$this->ldap->isResource($cr)) {
 			//LDAP not available
-			\OCP\Util::writeLog('user_ldap', 'LDAP resource not available.', \OCP\Util::DEBUG);
+			\OCP\Util::writeLog('user_ldap', 'LDAP resource not available.', ILogger::DEBUG);
 			return false;
 		}
 		try {
@@ -406,8 +423,7 @@ class Access extends LDAPUtility implements IUserTools {
 				$domainParts[] = $part;
 			}
 		}
-		$domainDN = implode(',', $domainParts);
-		return $domainDN;
+		return implode(',', $domainParts);
 	}
 
 	/**
@@ -512,51 +528,58 @@ class Access extends LDAPUtility implements IUserTools {
 
 	/**
 	 * returns an internal Nextcloud name for the given LDAP DN, false on DN outside of search DN
-	 * @param string $dn the dn of the user object
-	 * @param string $ldapName optional, the display name of the object
+	 *
+	 * @param string $fdn the dn of the user object
+	 * @param string|null $ldapName optional, the display name of the object
 	 * @param bool $isUser optional, whether it is a user object (otherwise group assumed)
-	 * @return string|false with with the name to use in Nextcloud
+	 * @param bool|null $newlyMapped
+	 * @param array|null $record
+	 * @return false|string with with the name to use in Nextcloud
+	 * @throws \Exception
 	 */
-	public function dn2ocname($fdn, $ldapName = null, $isUser = true) {
+	public function dn2ocname($fdn, $ldapName = null, $isUser = true, &$newlyMapped = null, array $record = null) {
+		$newlyMapped = false;
 		if($isUser) {
 			$mapper = $this->getUserMapper();
 			$nameAttribute = $this->connection->ldapUserDisplayName;
+			$filter = $this->connection->ldapUserFilter;
 		} else {
 			$mapper = $this->getGroupMapper();
 			$nameAttribute = $this->connection->ldapGroupDisplayName;
+			$filter = $this->connection->ldapGroupFilter;
 		}
 
 		//let's try to retrieve the Nextcloud name from the mappings table
-		$ocName = $mapper->getNameByDN($fdn);
-		if(is_string($ocName)) {
-			return $ocName;
+		$ncName = $mapper->getNameByDN($fdn);
+		if(is_string($ncName)) {
+			return $ncName;
 		}
 
 		//second try: get the UUID and check if it is known. Then, update the DN and return the name.
-		$uuid = $this->getUUID($fdn, $isUser);
+		$uuid = $this->getUUID($fdn, $isUser, $record);
 		if(is_string($uuid)) {
-			$ocName = $mapper->getNameByUUID($uuid);
-			if(is_string($ocName)) {
+			$ncName = $mapper->getNameByUUID($uuid);
+			if(is_string($ncName)) {
 				$mapper->setDNbyUUID($fdn, $uuid);
-				return $ocName;
+				return $ncName;
 			}
 		} else {
 			//If the UUID can't be detected something is foul.
-			\OCP\Util::writeLog('user_ldap', 'Cannot determine UUID for '.$fdn.'. Skipping.', \OCP\Util::INFO);
+			\OCP\Util::writeLog('user_ldap', 'Cannot determine UUID for '.$fdn.'. Skipping.', ILogger::INFO);
 			return false;
 		}
 
 		if(is_null($ldapName)) {
-			$ldapName = $this->readAttribute($fdn, $nameAttribute);
+			$ldapName = $this->readAttribute($fdn, $nameAttribute, $filter);
 			if(!isset($ldapName[0]) && empty($ldapName[0])) {
-				\OCP\Util::writeLog('user_ldap', 'No or empty name for '.$fdn.'.', \OCP\Util::INFO);
+				\OCP\Util::writeLog('user_ldap', 'No or empty name for '.$fdn.' with filter '.$filter.'.', ILogger::INFO);
 				return false;
 			}
 			$ldapName = $ldapName[0];
 		}
 
 		if($isUser) {
-			$usernameAttribute = strval($this->connection->ldapExpertUsernameAttr);
+			$usernameAttribute = (string)$this->connection->ldapExpertUsernameAttr;
 			if ($usernameAttribute !== '') {
 				$username = $this->readAttribute($fdn, $usernameAttribute);
 				$username = $username[0];
@@ -568,7 +591,7 @@ class Access extends LDAPUtility implements IUserTools {
 			} catch (\InvalidArgumentException $e) {
 				\OC::$server->getLogger()->logException($e, [
 					'app' => 'user_ldap',
-					'level' => Util::WARN,
+					'level' => ILogger::WARN,
 				]);
 				// we don't attempt to set a username here. We can go for
 				// for an alternative 4 digit random number as we would append
@@ -585,23 +608,47 @@ class Access extends LDAPUtility implements IUserTools {
 		//NOTE: mind, disabling cache affects only this instance! Using it
 		// outside of core user management will still cache the user as non-existing.
 		$originalTTL = $this->connection->ldapCacheTTL;
-		$this->connection->setConfiguration(array('ldapCacheTTL' => 0));
-		if(($isUser && !\OCP\User::userExists($intName))
-			|| (!$isUser && !\OC::$server->getGroupManager()->groupExists($intName))) {
-			if($mapper->map($fdn, $intName, $uuid)) {
-				$this->connection->setConfiguration(array('ldapCacheTTL' => $originalTTL));
+		$this->connection->setConfiguration(['ldapCacheTTL' => 0]);
+		if( $intName !== ''
+			&& (($isUser && !$this->ncUserManager->userExists($intName))
+				|| (!$isUser && !\OC::$server->getGroupManager()->groupExists($intName))
+			)
+		) {
+			$this->connection->setConfiguration(['ldapCacheTTL' => $originalTTL]);
+			$newlyMapped = $this->mapAndAnnounceIfApplicable($mapper, $fdn, $intName, $uuid, $isUser);
+			if($newlyMapped) {
 				return $intName;
 			}
 		}
-		$this->connection->setConfiguration(array('ldapCacheTTL' => $originalTTL));
 
+		$this->connection->setConfiguration(['ldapCacheTTL' => $originalTTL]);
 		$altName = $this->createAltInternalOwnCloudName($intName, $isUser);
-		if(is_string($altName) && $mapper->map($fdn, $altName, $uuid)) {
-			return $altName;
+		if (is_string($altName)) {
+			if($this->mapAndAnnounceIfApplicable($mapper, $fdn, $altName, $uuid, $isUser)) {
+				$newlyMapped = true;
+				return $altName;
+			}
 		}
 
 		//if everything else did not help..
-		\OCP\Util::writeLog('user_ldap', 'Could not create unique name for '.$fdn.'.', \OCP\Util::INFO);
+		\OCP\Util::writeLog('user_ldap', 'Could not create unique name for '.$fdn.'.', ILogger::INFO);
+		return false;
+	}
+
+	public function mapAndAnnounceIfApplicable(
+		AbstractMapping $mapper,
+		string $fdn,
+		string $name,
+		string $uuid,
+		bool $isUser
+	) :bool {
+		if($mapper->map($fdn, $name, $uuid)) {
+			if ($this->ncUserManager instanceof PublicEmitter && $isUser) {
+				$this->cacheUserExists($name);
+				$this->ncUserManager->emit('\OC\User', 'assignedUserId', [$name]);
+			}
+			return true;
+		}
 		return false;
 	}
 
@@ -631,6 +678,7 @@ class Access extends LDAPUtility implements IUserTools {
 	 * @param array $ldapObjects as returned by fetchList()
 	 * @param bool $isUsers
 	 * @return array
+	 * @throws \Exception
 	 */
 	private function ldap2NextcloudNames($ldapObjects, $isUsers) {
 		if($isUsers) {
@@ -639,7 +687,7 @@ class Access extends LDAPUtility implements IUserTools {
 		} else {
 			$nameAttribute = $this->connection->ldapGroupDisplayName;
 		}
-		$nextcloudNames = array();
+		$nextcloudNames = [];
 
 		foreach($ldapObjects as $ldapObject) {
 			$nameByLDAP = null;
@@ -655,6 +703,7 @@ class Access extends LDAPUtility implements IUserTools {
 			if($ncName) {
 				$nextcloudNames[] = $ncName;
 				if($isUsers) {
+					$this->updateUserState($ncName);
 					//cache the user names so it does not need to be retrieved
 					//again later (e.g. sharing dialogue).
 					if(is_null($nameByLDAP)) {
@@ -669,6 +718,19 @@ class Access extends LDAPUtility implements IUserTools {
 		return $nextcloudNames;
 	}
 
+	/**
+	 * removes the deleted-flag of a user if it was set
+	 *
+	 * @param string $ncname
+	 * @throws \Exception
+	 */
+	public function updateUserState($ncname) {
+		$user = $this->userManager->get($ncname);
+		if($user instanceof OfflineUser) {
+			$user->unmark();
+		}
+	}
+
 	/**
 	 * caches the user display name
 	 * @param string $ocName the internal Nextcloud username
@@ -717,7 +779,7 @@ class Access extends LDAPUtility implements IUserTools {
 		//20 attempts, something else is very wrong. Avoids infinite loop.
 		while($attempts < 20){
 			$altName = $name . '_' . rand(1000,9999);
-			if(!\OCP\User::userExists($altName)) {
+			if(!$this->ncUserManager->userExists($altName)) {
 				return $altName;
 			}
 			$attempts++;
@@ -739,14 +801,14 @@ class Access extends LDAPUtility implements IUserTools {
 	 */
 	private function _createAltInternalOwnCloudNameForGroups($name) {
 		$usedNames = $this->groupMapper->getNamesBySearch($name, "", '_%');
-		if(!($usedNames) || count($usedNames) === 0) {
+		if(!$usedNames || count($usedNames) === 0) {
 			$lastNo = 1; //will become name_2
 		} else {
 			natsort($usedNames);
 			$lastName = array_pop($usedNames);
-			$lastNo = intval(substr($lastName, strrpos($lastName, '_') + 1));
+			$lastNo = (int)substr($lastName, strrpos($lastName, '_') + 1);
 		}
-		$altName = $name.'_'.strval($lastNo+1);
+		$altName = $name.'_'. (string)($lastNo+1);
 		unset($usedNames);
 
 		$attempts = 1;
@@ -793,8 +855,7 @@ class Access extends LDAPUtility implements IUserTools {
 	public function fetchUsersByLoginName($loginName, $attributes = array('dn')) {
 		$loginName = $this->escapeFilterPart($loginName);
 		$filter = str_replace('%uid', $loginName, $this->connection->ldapLoginFilter);
-		$users = $this->fetchListOfUsers($filter, $attributes);
-		return $users;
+		return $this->fetchListOfUsers($filter, $attributes);
 	}
 
 	/**
@@ -802,13 +863,12 @@ class Access extends LDAPUtility implements IUserTools {
 	 * utilizing the login filter.
 	 *
 	 * @param string $loginName
-	 * @return array
+	 * @return int
 	 */
 	public function countUsersByLoginName($loginName) {
 		$loginName = $this->escapeFilterPart($loginName);
 		$filter = str_replace('%uid', $loginName, $this->connection->ldapLoginFilter);
-		$users = $this->countUsers($filter);
-		return $users;
+		return $this->countUsers($filter);
 	}
 
 	/**
@@ -816,19 +876,35 @@ class Access extends LDAPUtility implements IUserTools {
 	 * @param string|string[] $attr
 	 * @param int $limit
 	 * @param int $offset
+	 * @param bool $forceApplyAttributes
 	 * @return array
 	 */
-	public function fetchListOfUsers($filter, $attr, $limit = null, $offset = null) {
+	public function fetchListOfUsers($filter, $attr, $limit = null, $offset = null, $forceApplyAttributes = false) {
 		$ldapRecords = $this->searchUsers($filter, $attr, $limit, $offset);
-		$this->batchApplyUserAttributes($ldapRecords);
-		return $this->fetchList($ldapRecords, (count($attr) > 1));
+		$recordsToUpdate = $ldapRecords;
+		if(!$forceApplyAttributes) {
+			$isBackgroundJobModeAjax = $this->config
+					->getAppValue('core', 'backgroundjobs_mode', 'ajax') === 'ajax';
+			$recordsToUpdate = array_filter($ldapRecords, function($record) use ($isBackgroundJobModeAjax) {
+				$newlyMapped = false;
+				$uid = $this->dn2ocname($record['dn'][0], null, true, $newlyMapped, $record);
+				if(is_string($uid)) {
+					$this->cacheUserExists($uid);
+				}
+				return ($uid !== false) && ($newlyMapped || $isBackgroundJobModeAjax);
+			});
+		}
+		$this->batchApplyUserAttributes($recordsToUpdate);
+		return $this->fetchList($ldapRecords, $this->manyAttributes($attr));
 	}
 
 	/**
 	 * provided with an array of LDAP user records the method will fetch the
 	 * user object and requests it to process the freshly fetched attributes and
 	 * and their values
+	 *
 	 * @param array $ldapRecords
+	 * @throws \Exception
 	 */
 	public function batchApplyUserAttributes(array $ldapRecords){
 		$displayNameAttribute = strtolower($this->connection->ldapUserDisplayName);
@@ -837,16 +913,12 @@ class Access extends LDAPUtility implements IUserTools {
 				// displayName is obligatory
 				continue;
 			}
-			$ocName  = $this->dn2ocname($userRecord['dn'][0]);
+			$ocName  = $this->dn2ocname($userRecord['dn'][0], null, true);
 			if($ocName === false) {
 				continue;
 			}
-			$this->cacheUserExists($ocName);
+			$this->updateUserState($ocName);
 			$user = $this->userManager->get($ocName);
-			if($user instanceof OfflineUser) {
-				$user->unmark();
-				$user = $this->userManager->get($ocName);
-			}
 			if ($user !== null) {
 				$user->processAttributes($userRecord);
 			} else {
@@ -866,7 +938,7 @@ class Access extends LDAPUtility implements IUserTools {
 	 * @return array
 	 */
 	public function fetchListOfGroups($filter, $attr, $limit = null, $offset = null) {
-		return $this->fetchList($this->searchGroups($filter, $attr, $limit, $offset), (count($attr) > 1));
+		return $this->fetchList($this->searchGroups($filter, $attr, $limit, $offset), $this->manyAttributes($attr));
 	}
 
 	/**
@@ -903,7 +975,11 @@ class Access extends LDAPUtility implements IUserTools {
 	 * Executes an LDAP search
 	 */
 	public function searchUsers($filter, $attr = null, $limit = null, $offset = null) {
-		return $this->search($filter, $this->connection->ldapBaseUsers, $attr, $limit, $offset);
+		$result = [];
+		foreach($this->connection->ldapBaseUsers as $base) {
+			$result = array_merge($result, $this->search($filter, [$base], $attr, $limit, $offset));
+		}
+		return $result;
 	}
 
 	/**
@@ -914,7 +990,12 @@ class Access extends LDAPUtility implements IUserTools {
 	 * @return false|int
 	 */
 	public function countUsers($filter, $attr = array('dn'), $limit = null, $offset = null) {
-		return $this->count($filter, $this->connection->ldapBaseUsers, $attr, $limit, $offset);
+		$result = false;
+		foreach($this->connection->ldapBaseUsers as $base) {
+			$count = $this->count($filter, [$base], $attr, $limit, $offset);
+			$result = is_int($count) ? (int)$result + $count : $result;
+		}
+		return $result;
 	}
 
 	/**
@@ -928,7 +1009,11 @@ class Access extends LDAPUtility implements IUserTools {
 	 * Executes an LDAP search
 	 */
 	public function searchGroups($filter, $attr = null, $limit = null, $offset = null) {
-		return $this->search($filter, $this->connection->ldapBaseGroups, $attr, $limit, $offset);
+		$result = [];
+		foreach($this->connection->ldapBaseGroups as $base) {
+			$result = array_merge($result, $this->search($filter, [$base], $attr, $limit, $offset));
+		}
+		return $result;
 	}
 
 	/**
@@ -940,7 +1025,12 @@ class Access extends LDAPUtility implements IUserTools {
 	 * @return int|bool
 	 */
 	public function countGroups($filter, $attr = array('dn'), $limit = null, $offset = null) {
-		return $this->count($filter, $this->connection->ldapBaseGroups, $attr, $limit, $offset);
+		$result = false;
+		foreach($this->connection->ldapBaseGroups as $base) {
+			$count = $this->count($filter, [$base], $attr, $limit, $offset);
+			$result = is_int($count) ? (int)$result + $count : $result;
+		}
+		return $result;
 	}
 
 	/**
@@ -951,7 +1041,12 @@ class Access extends LDAPUtility implements IUserTools {
 	 * @return int|bool
 	 */
 	public function countObjects($limit = null, $offset = null) {
-		return $this->count('objectclass=*', $this->connection->ldapBase, array('dn'), $limit, $offset);
+		$result = false;
+		foreach($this->connection->ldapBase as $base) {
+			$count = $this->count('objectclass=*', [$base], ['dn'], $limit, $offset);
+			$result = is_int($count) ? (int)$result + $count : $result;
+		}
+		return $result;
 	}
 
 	/**
@@ -988,13 +1083,13 @@ class Access extends LDAPUtility implements IUserTools {
 			 * Maybe implement exponential backoff?
 			 * This was enough to get solr indexer working which has large delays between LDAP fetches.
 			 */
-			\OCP\Util::writeLog('user_ldap', "Connection lost on $command, attempting to reestablish.", \OCP\Util::DEBUG);
+			\OCP\Util::writeLog('user_ldap', "Connection lost on $command, attempting to reestablish.", ILogger::DEBUG);
 			$this->connection->resetConnectionResource();
 			$cr = $this->connection->getConnectionResource();
 
 			if(!$this->ldap->isResource($cr)) {
 				// Seems like we didn't find any resource.
-				\OCP\Util::writeLog('user_ldap', "Could not $command, because resource is missing.", \OCP\Util::DEBUG);
+				\OCP\Util::writeLog('user_ldap', "Could not $command, because resource is missing.", ILogger::DEBUG);
 				throw $e;
 			}
 
@@ -1006,11 +1101,15 @@ class Access extends LDAPUtility implements IUserTools {
 
 	/**
 	 * retrieved. Results will according to the order in the array.
+	 *
+	 * @param $filter
+	 * @param $base
+	 * @param string[]|string|null $attr
 	 * @param int $limit optional, maximum results to be counted
 	 * @param int $offset optional, a starting point
 	 * @return array|false array with the search result as first value and pagedSearchOK as
 	 * second | false if not successful
-	 * @throws \OC\ServerNotAvailableException
+	 * @throws ServerNotAvailableException
 	 */
 	private function executeSearch($filter, $base, &$attr = null, $limit = null, $offset = null) {
 		if(!is_null($attr) && !is_array($attr)) {
@@ -1022,19 +1121,19 @@ class Access extends LDAPUtility implements IUserTools {
 		if(!$this->ldap->isResource($cr)) {
 			// Seems like we didn't find any resource.
 			// Return an empty array just like before.
-			\OCP\Util::writeLog('user_ldap', 'Could not search, because resource is missing.', \OCP\Util::DEBUG);
+			\OCP\Util::writeLog('user_ldap', 'Could not search, because resource is missing.', ILogger::DEBUG);
 			return false;
 		}
 
 		//check whether paged search should be attempted
-		$pagedSearchOK = $this->initPagedSearch($filter, $base, $attr, intval($limit), $offset);
+		$pagedSearchOK = $this->initPagedSearch($filter, $base, $attr, (int)$limit, $offset);
 
 		$linkResources = array_pad(array(), count($base), $cr);
 		$sr = $this->invokeLDAPMethod('search', $linkResources, $base, $filter, $attr);
 		// cannot use $cr anymore, might have changed in the previous call!
 		$error = $this->ldap->errno($this->connection->getConnectionResource());
 		if(!is_array($sr) || $error !== 0) {
-			\OCP\Util::writeLog('user_ldap', 'Attempt for Paging?  '.print_r($pagedSearchOK, true), \OCP\Util::ERROR);
+			\OCP\Util::writeLog('user_ldap', 'Attempt for Paging?  '.print_r($pagedSearchOK, true), ILogger::ERROR);
 			return false;
 		}
 
@@ -1075,7 +1174,7 @@ class Access extends LDAPUtility implements IUserTools {
 				$this->pagedSearchedSuccessful = true;
 			}
 		} else {
-			if(!is_null($limit) && intval($this->connection->ldapPagingSize) !== 0) {
+			if(!is_null($limit) && (int)$this->connection->ldapPagingSize !== 0) {
 				\OC::$server->getLogger()->debug(
 					'Paged search was not available',
 					[ 'app' => 'user_ldap' ]
@@ -1092,6 +1191,7 @@ class Access extends LDAPUtility implements IUserTools {
 
 	/**
 	 * executes an LDAP search, but counts the results only
+	 *
 	 * @param string $filter the LDAP filter for the search
 	 * @param array $base an array containing the LDAP subtree(s) that shall be searched
 	 * @param string|string[] $attr optional, array, one or more attributes that shall be
@@ -1101,12 +1201,12 @@ class Access extends LDAPUtility implements IUserTools {
 	 * @param bool $skipHandling indicates whether the pages search operation is
 	 * completed
 	 * @return int|false Integer or false if the search could not be initialized
-	 *
+	 * @throws ServerNotAvailableException
 	 */
 	private function count($filter, $base, $attr = null, $limit = null, $offset = null, $skipHandling = false) {
-		\OCP\Util::writeLog('user_ldap', 'Count filter:  '.print_r($filter, true), \OCP\Util::DEBUG);
+		\OCP\Util::writeLog('user_ldap', 'Count filter:  '.print_r($filter, true), ILogger::DEBUG);
 
-		$limitPerPage = intval($this->connection->ldapPagingSize);
+		$limitPerPage = (int)$this->connection->ldapPagingSize;
 		if(!is_null($limit) && $limit < $limitPerPage && $limit > 0) {
 			$limitPerPage = $limit;
 		}
@@ -1116,8 +1216,7 @@ class Access extends LDAPUtility implements IUserTools {
 		$this->connection->getConnectionResource();
 
 		do {
-			$search = $this->executeSearch($filter, $base, $attr,
-										   $limitPerPage, $offset);
+			$search = $this->executeSearch($filter, $base, $attr, $limitPerPage, $offset);
 			if($search === false) {
 				return $counter > 0 ? $counter : false;
 			}
@@ -1150,7 +1249,7 @@ class Access extends LDAPUtility implements IUserTools {
 		$counter = 0;
 
 		foreach($searchResults as $res) {
-			$count = intval($this->invokeLDAPMethod('countEntries', $this->connection->getConnectionResource(), $res));
+			$count = (int)$this->invokeLDAPMethod('countEntries', $this->connection->getConnectionResource(), $res);
 			$counter += $count;
 		}
 
@@ -1159,6 +1258,7 @@ class Access extends LDAPUtility implements IUserTools {
 
 	/**
 	 * Executes an LDAP search
+	 *
 	 * @param string $filter the LDAP filter for the search
 	 * @param array $base an array containing the LDAP subtree(s) that shall be searched
 	 * @param string|string[] $attr optional, array, one or more attributes that shall be
@@ -1166,9 +1266,10 @@ class Access extends LDAPUtility implements IUserTools {
 	 * @param int $offset
 	 * @param bool $skipHandling
 	 * @return array with the search result
+	 * @throws ServerNotAvailableException
 	 */
 	public function search($filter, $base, $attr = null, $limit = null, $offset = null, $skipHandling = false) {
-		$limitPerPage = intval($this->connection->ldapPagingSize);
+		$limitPerPage = (int)$this->connection->ldapPagingSize;
 		if(!is_null($limit) && $limit < $limitPerPage && $limit > 0) {
 			$limitPerPage = $limit;
 		}
@@ -1179,12 +1280,12 @@ class Access extends LDAPUtility implements IUserTools {
 		 * loops through until we get $continue equals true and
 		 * $findings['count'] < $limit
 		 */
-		$findings = array();
+		$findings = [];
 		$savedoffset = $offset;
 		do {
 			$search = $this->executeSearch($filter, $base, $attr, $limitPerPage, $offset);
 			if($search === false) {
-				return array();
+				return [];
 			}
 			list($sr, $pagedSearchOK) = $search;
 			$cr = $this->connection->getConnectionResource();
@@ -1221,7 +1322,7 @@ class Access extends LDAPUtility implements IUserTools {
 		}
 
 		if(!is_null($attr)) {
-			$selection = array();
+			$selection = [];
 			$i = 0;
 			foreach($findings as $item) {
 				if(!is_array($item)) {
@@ -1229,15 +1330,18 @@ class Access extends LDAPUtility implements IUserTools {
 				}
 				$item = \OCP\Util::mb_array_change_key_case($item, MB_CASE_LOWER, 'UTF-8');
 				foreach($attr as $key) {
-					$key = mb_strtolower($key, 'UTF-8');
 					if(isset($item[$key])) {
 						if(is_array($item[$key]) && isset($item[$key]['count'])) {
 							unset($item[$key]['count']);
 						}
 						if($key !== 'dn') {
-							$selection[$i][$key] = $this->resemblesDN($key) ?
-								$this->helper->sanitizeDN($item[$key])
-								: $item[$key];
+							if($this->resemblesDN($key)) {
+								$selection[$i][$key] = $this->helper->sanitizeDN($item[$key]);
+							} else if($key === 'objectguid' || $key === 'guid') {
+								$selection[$i][$key] = [$this->convertObjectGUID2Str($item[$key][0])];
+							} else {
+								$selection[$i][$key] = $item[$key];
+							}
 						} else {
 							$selection[$i][$key] = [$this->helper->sanitizeDN($item[$key])];
 						}
@@ -1258,7 +1362,7 @@ class Access extends LDAPUtility implements IUserTools {
 				&& !is_null($limit)
 			)
 		) {
-			$findings = array_slice($findings, intval($offset), $limit);
+			$findings = array_slice($findings, (int)$offset, $limit);
 		}
 		return $findings;
 	}
@@ -1416,7 +1520,7 @@ class Access extends LDAPUtility implements IUserTools {
 				\OCP\Util::writeLog(
 					'user_ldap',
 					'Creating advanced filter for search failed, falling back to simple method.',
-					\OCP\Util::INFO
+					ILogger::INFO
 				);
 			}
 		}
@@ -1539,12 +1643,14 @@ class Access extends LDAPUtility implements IUserTools {
 
 	/**
 	 * auto-detects the directory's UUID attribute
+	 *
 	 * @param string $dn a known DN used to check against
 	 * @param bool $isUser
 	 * @param bool $force the detection should be run, even if it is not set to auto
+	 * @param array|null $ldapRecord
 	 * @return bool true on success, false otherwise
 	 */
-	private function detectUuidAttribute($dn, $isUser = true, $force = false) {
+	private function detectUuidAttribute($dn, $isUser = true, $force = false, array $ldapRecord = null) {
 		if($isUser) {
 			$uuidAttr     = 'ldapUuidUserAttribute';
 			$uuidOverride = $this->connection->ldapExpertUUIDUserAttr;
@@ -1562,22 +1668,33 @@ class Access extends LDAPUtility implements IUserTools {
 			return true;
 		}
 
-		// for now, supported attributes are entryUUID, nsuniqueid, objectGUID, ipaUniqueID
-		$testAttributes = array('entryuuid', 'nsuniqueid', 'objectguid', 'guid', 'ipauniqueid');
+		foreach(self::UUID_ATTRIBUTES as $attribute) {
+			if($ldapRecord !== null) {
+				// we have the info from LDAP already, we don't need to talk to the server again
+				if(isset($ldapRecord[$attribute])) {
+					$this->connection->$uuidAttr = $attribute;
+					return true;
+				} else {
+					continue;
+				}
+			}
 
-		foreach($testAttributes as $attribute) {
 			$value = $this->readAttribute($dn, $attribute);
 			if(is_array($value) && isset($value[0]) && !empty($value[0])) {
-				\OCP\Util::writeLog('user_ldap',
-									'Setting '.$attribute.' as '.$uuidAttr,
-									\OCP\Util::DEBUG);
+				\OCP\Util::writeLog(
+					'user_ldap',
+					'Setting '.$attribute.' as '.$uuidAttr,
+					ILogger::DEBUG
+				);
 				$this->connection->$uuidAttr = $attribute;
 				return true;
 			}
 		}
-		\OCP\Util::writeLog('user_ldap',
-							'Could not autodetect the UUID attribute',
-							\OCP\Util::ERROR);
+		\OCP\Util::writeLog(
+			'user_ldap',
+			'Could not autodetect the UUID attribute',
+			ILogger::ERROR
+		);
 
 		return false;
 	}
@@ -1585,9 +1702,10 @@ class Access extends LDAPUtility implements IUserTools {
 	/**
 	 * @param string $dn
 	 * @param bool $isUser
-	 * @return string|bool
+	 * @param null $ldapRecord
+	 * @return bool|string
 	 */
-	public function getUUID($dn, $isUser = true) {
+	public function getUUID($dn, $isUser = true, $ldapRecord = null) {
 		if($isUser) {
 			$uuidAttr     = 'ldapUuidUserAttribute';
 			$uuidOverride = $this->connection->ldapExpertUUIDUserAttr;
@@ -1597,13 +1715,16 @@ class Access extends LDAPUtility implements IUserTools {
 		}
 
 		$uuid = false;
-		if($this->detectUuidAttribute($dn, $isUser)) {
-			$uuid = $this->readAttribute($dn, $this->connection->$uuidAttr);
+		if($this->detectUuidAttribute($dn, $isUser, false, $ldapRecord)) {
+			$attr = $this->connection->$uuidAttr;
+			$uuid = isset($ldapRecord[$attr]) ? $ldapRecord[$attr] : $this->readAttribute($dn, $attr);
 			if( !is_array($uuid)
 				&& $uuidOverride !== ''
-				&& $this->detectUuidAttribute($dn, $isUser, true)) {
-					$uuid = $this->readAttribute($dn,
-												 $this->connection->$uuidAttr);
+				&& $this->detectUuidAttribute($dn, $isUser, true, $ldapRecord))
+			{
+				$uuid = isset($ldapRecord[$this->connection->$uuidAttr])
+					? $ldapRecord[$this->connection->$uuidAttr]
+					: $this->readAttribute($dn, $this->connection->$uuidAttr);
 			}
 			if(is_array($uuid) && isset($uuid[0]) && !empty($uuid[0])) {
 				$uuid = $uuid[0];
@@ -1772,15 +1893,15 @@ class Access extends LDAPUtility implements IUserTools {
 
 	/**
 	 * resets a running Paged Search operation
+	 *
+	 * @throws ServerNotAvailableException
 	 */
 	private function abandonPagedSearch() {
-		if($this->connection->hasPagedResultSupport) {
-			$cr = $this->connection->getConnectionResource();
-			$this->invokeLDAPMethod('controlPagedResult', $cr, 0, false, $this->lastCookie);
-			$this->getPagedSearchResultState();
-			$this->lastCookie = '';
-			$this->cookies = array();
-		}
+		$cr = $this->connection->getConnectionResource();
+		$this->invokeLDAPMethod('controlPagedResult', $cr, 0, false, $this->lastCookie);
+		$this->getPagedSearchResultState();
+		$this->lastCookie = '';
+		$this->cookies = [];
 	}
 
 	/**
@@ -1797,7 +1918,7 @@ class Access extends LDAPUtility implements IUserTools {
 		}
 		$offset -= $limit;
 		//we work with cache here
-		$cacheKey = 'lc' . crc32($base) . '-' . crc32($filter) . '-' . intval($limit) . '-' . intval($offset);
+		$cacheKey = 'lc' . crc32($base) . '-' . crc32($filter) . '-' . (int)$limit . '-' . (int)$offset;
 		$cookie = '';
 		if(isset($this->cookies[$cacheKey])) {
 			$cookie = $this->cookies[$cacheKey];
@@ -1819,10 +1940,6 @@ class Access extends LDAPUtility implements IUserTools {
 	 * @return bool
 	 */
 	public function hasMoreResults() {
-		if(!$this->connection->hasPagedResultSupport) {
-			return false;
-		}
-
 		if(empty($this->lastCookie) && $this->lastCookie !== '0') {
 			// as in RFC 2696, when all results are returned, the cookie will
 			// be empty.
@@ -1844,7 +1961,7 @@ class Access extends LDAPUtility implements IUserTools {
 	private function setPagedResultCookie($base, $filter, $limit, $offset, $cookie) {
 		// allow '0' for 389ds
 		if(!empty($cookie) || $cookie === '0') {
-			$cacheKey = 'lc' . crc32($base) . '-' . crc32($filter) . '-' .intval($limit) . '-' . intval($offset);
+			$cacheKey = 'lc' . crc32($base) . '-' . crc32($filter) . '-' . (int)$limit . '-' . (int)$offset;
 			$this->cookies[$cacheKey] = $cookie;
 			$this->lastCookie = $cookie;
 		}
@@ -1871,31 +1988,27 @@ class Access extends LDAPUtility implements IUserTools {
 	 */
 	private function initPagedSearch($filter, $bases, $attr, $limit, $offset) {
 		$pagedSearchOK = false;
-		if($this->connection->hasPagedResultSupport && ($limit !== 0)) {
-			$offset = intval($offset); //can be null
+		if ($limit !== 0) {
+			$offset = (int)$offset; //can be null
 			\OCP\Util::writeLog('user_ldap',
 				'initializing paged search for  Filter '.$filter.' base '.print_r($bases, true)
 				.' attr '.print_r($attr, true). ' limit ' .$limit.' offset '.$offset,
-				\OCP\Util::DEBUG);
+				ILogger::DEBUG);
 			//get the cookie from the search for the previous search, required by LDAP
 			foreach($bases as $base) {
 
 				$cookie = $this->getPagedResultCookie($base, $filter, $limit, $offset);
 				if(empty($cookie) && $cookie !== "0" && ($offset > 0)) {
-					// no cookie known, although the offset is not 0. Maybe cache run out. We need
-					// to start all over *sigh* (btw, Dear Reader, did you know LDAP paged
-					// searching was designed by MSFT?)
-					// 		Lukas: No, but thanks to reading that source I finally know!
-					// '0' is valid, because 389ds
+					// no cookie known from a potential previous search. We need
+					// to start from 0 to come to the desired page. cookie value
+					// of '0' is valid, because 389ds
 					$reOffset = ($offset - $limit) < 0 ? 0 : $offset - $limit;
-					//a bit recursive, $offset of 0 is the exit
-					\OCP\Util::writeLog('user_ldap', 'Looking for cookie L/O '.$limit.'/'.$reOffset, \OCP\Util::INFO);
 					$this->search($filter, array($base), $attr, $limit, $reOffset, true);
 					$cookie = $this->getPagedResultCookie($base, $filter, $limit, $offset);
 					//still no cookie? obviously, the server does not like us. Let's skip paging efforts.
+					// '0' is valid, because 389ds
 					//TODO: remember this, probably does not change in the next request...
 					if(empty($cookie) && $cookie !== '0') {
-						// '0' is valid, because 389ds
 						$cookie = null;
 					}
 				}
@@ -1908,11 +2021,10 @@ class Access extends LDAPUtility implements IUserTools {
 					if(!$pagedSearchOK) {
 						return false;
 					}
-					\OCP\Util::writeLog('user_ldap', 'Ready for a paged search', \OCP\Util::DEBUG);
+					\OCP\Util::writeLog('user_ldap', 'Ready for a paged search', ILogger::DEBUG);
 				} else {
-					\OCP\Util::writeLog('user_ldap',
-						'No paged search for us, Cpt., Limit '.$limit.' Offset '.$offset,
-						\OCP\Util::INFO);
+					$e = new \Exception('No paged search possible, Limit '.$limit.' Offset '.$offset);
+					\OC::$server->getLogger()->logException($e, ['level' => ILogger::DEBUG]);
 				}
 
 			}
@@ -1922,14 +2034,14 @@ class Access extends LDAPUtility implements IUserTools {
 		 * So we added "&& !empty($this->lastCookie)" to this test to ignore pagination
 		 * if we don't have a previous paged search.
 		 */
-		} else if($this->connection->hasPagedResultSupport && $limit === 0 && !empty($this->lastCookie)) {
+		} else if ($limit === 0 && !empty($this->lastCookie)) {
 			// a search without limit was requested. However, if we do use
 			// Paged Search once, we always must do it. This requires us to
 			// initialize it with the configured page size.
 			$this->abandonPagedSearch();
 			// in case someone set it to 0 … use 500, otherwise no results will
 			// be returned.
-			$pageSize = intval($this->connection->ldapPagingSize) > 0 ? intval($this->connection->ldapPagingSize) : 500;
+			$pageSize = (int)$this->connection->ldapPagingSize > 0 ? (int)$this->connection->ldapPagingSize : 500;
 			$pagedSearchOK = $this->invokeLDAPMethod('controlPagedResult',
 				$this->connection->getConnectionResource(),
 				$pageSize, false, '');
@@ -1938,4 +2050,17 @@ class Access extends LDAPUtility implements IUserTools {
 		return $pagedSearchOK;
 	}
 
+	/**
+	 * Is more than one $attr used for search?
+	 *
+	 * @param string|string[]|null $attr
+	 * @return bool
+	 */
+	private function manyAttributes($attr): bool {
+		if (\is_array($attr)) {
+			return \count($attr) > 1;
+		}
+		return false;
+	}
+
 }
