@@ -245,7 +245,7 @@ class ConvertType extends Command implements CompletionAwareInterface {
 				$currentMigration = $fromMS->getMigration('current');
 				if ($currentMigration !== '0') {
 					$toMS = new MigrationService($app, $toDB);
-					$toMS->migrate($currentMigration);
+					$toMS->migrate($currentMigration, true);
 				}
 			}
 		}
@@ -301,7 +301,7 @@ class ConvertType extends Command implements CompletionAwareInterface {
 
 		$query = $fromDB->getQueryBuilder();
 		$query->automaticTablePrefix(false);
-		$query->selectAlias($query->createFunction('COUNT(*)'), 'num_entries')
+		$query->select($query->func()->count('*', 'num_entries'))
 			->from($table->getName());
 		$result = $query->execute();
 		$count = $result->fetchColumn();
@@ -383,6 +383,9 @@ class ConvertType extends Command implements CompletionAwareInterface {
 			case Type::TEXT:
 				$this->columnTypes[$tableName][$columnName] = IQueryBuilder::PARAM_LOB;
 				break;
+			case Type::BOOLEAN:
+				$this->columnTypes[$tableName][$columnName] = IQueryBuilder::PARAM_BOOL;
+				break;
 			default:
 				$this->columnTypes[$tableName][$columnName] = false;
 		}
