@@ -5,63 +5,73 @@ OC.L10N.register(
     "This can usually be fixed by giving the webserver write access to the config directory" : "Tai, dažniausiai, gali būti ištaisyta suteikiant saityno serveriui rašymo prieigą prie konfigūracijos katalogo",
     "See %s" : "Žiūrėkite %s",
     "This can usually be fixed by giving the webserver write access to the config directory. See %s" : "Tai, dažniausiai, gali būti pataisyta, suteikiant saityno serveriui rašymo prieigą prie konfigūracijos katalogo. Žiūrėkite %s",
-    "The files of the app %$1s were not replaced correctly. Make sure it is a version compatible with the server." : "Programėlės %$1s failai buvo pakeisti neteisingai. Įsitikinkite, kad versija yra suderinama su serveriu.",
     "Sample configuration detected" : "Aptiktas konfigūracijos pavyzdys",
     "It has been detected that the sample configuration has been copied. This can break your installation and is unsupported. Please read the documentation before performing changes on config.php" : "Pastebėta, kad nukopijuota pavyzdinė konfigūracija. Tai gali pažeisti jūsų diegimą ir yra nepalaikoma. Prieš atliekant pakeitimus config.php faile, prašome perskaityti dokumentaciją.",
     "%1$s and %2$s" : "%1$s ir %2$s",
     "%1$s, %2$s and %3$s" : "%1$s, %2$s ir %3$s",
     "%1$s, %2$s, %3$s and %4$s" : "%1$s, %2$s, %3$s ir %4$s",
     "%1$s, %2$s, %3$s, %4$s and %5$s" : "%1$s, %2$s, %3$s, %4$s ir %5$s",
+    "Groupware bundle" : "Grupinio darbo įrangos rinkinys",
     "PHP %s or higher is required." : "Reikalinga PHP %s arba aukštesnė.",
     "PHP with a version lower than %s is required." : "Reikalinga žemesnė nei %s PHP versija. ",
     "Following databases are supported: %s" : "Yra palaikomos šios duomenų bazės: %s",
     "The command line tool %s could not be found" : "Nepavyko rasti komandų eilutės įrankio %s",
     "The library %s is not available." : "Biblioteka %s nėra prieinama.",
-    "Library %s with a version higher than %s is required - available version %s." : "Bibliotekos %s versija turi būti aukštesnė nei %s - turima versija %s.",
-    "Library %s with a version lower than %s is required - available version %s." : "Bibliotekos %s versija turi būti žemesnė nei %s - turima versija %s.",
     "Following platforms are supported: %s" : "Yra palaikomos šios platformos: %s",
     "Server version %s or higher is required." : "Reikalinga %s arba aukštesnė serverio versija ",
     "Server version %s or lower is required." : "Reikalinga %s arba žemesnė serverio versija. ",
+    "Logged in user must be an admin" : "Prisijungęs naudotojas privalo būti administratoriumi",
+    "Wiping of device %s has started" : "Įrenginio %s duomenų ištrynimas pradėtas",
+    "Wiping of device »%s« has started" : "Įrenginio »%s« duomenų ištrynimas pradėtas",
+    "»%s« started remote wipe" : "»%s« pradėjo nuotolinių duomenų ištrynimą",
+    "Device or application »%s« has started the remote wipe process. You will receive another email once the process has finished" : "Įrenginys ar programa »%s« pradėjo nuotolinių duomenų ištrynimo procesą. Procesui užsibaigus, gausite dar vieną el. laišką",
+    "Wiping of device %s has finished" : "Įrenginio %s duomenų ištrynimas užbaigtas",
+    "Wiping of device »%s« has finished" : "Įrenginio »%s« duomenų ištrynimas užbaigtas",
+    "»%s« finished remote wipe" : "»%s« užbaigė nuotolinių duomenų ištrynimą",
+    "Device or application »%s« has finished the remote wipe process." : "Įrenginys ar programa »%s« užbaigė nuotolinių duomenų ištrynimo procesą.",
+    "Remote wipe started" : "Nuotolinių duomenų ištrynimas pradėtas",
+    "A remote wipe was started on device %s" : "Nuotolinių duomenų ištrynimas buvo pradėtas įrenginyje %s",
+    "Remote wipe finished" : "Nuotolinių duomenų ištrynimas užbaigtas",
+    "The remote wipe on %s has finished" : "Nuotolinių duomenų ištrynimas ties %s yra užbaigtas",
+    "Authentication" : "Tapatybės nustatymas",
     "Unknown filetype" : "Nežinomas failo tipas",
     "Invalid image" : "Neteisingas paveikslas",
+    "Avatar image is not square" : "Avataro paveikslas nėra kvadratinis",
     "today" : "šiandien",
+    "tomorrow" : "rytoj",
     "yesterday" : "vakar",
-    "_%n day ago_::_%n days ago_" : ["prieš %n dieną","prieš %n dienas","prieš %n dienų"],
+    "_%n day ago_::_%n days ago_" : ["prieš %n dieną","prieš %n dienas","prieš %n dienų","prieš %n dienų"],
     "last month" : "praeitą mėnesį",
-    "_%n month ago_::_%n months ago_" : ["prieš %n mėnesį","prieš %n mėnesius","prieš %n mėnesių"],
+    "_%n month ago_::_%n months ago_" : ["prieš %n mėnesį","prieš %n mėnesius","prieš %n mėnesių","prieš %n mėnesių"],
     "last year" : "praeitais metais",
-    "_%n year ago_::_%n years ago_" : ["prieš %n metus","prieš %n metus","prieš %n metų"],
-    "_%n hour ago_::_%n hours ago_" : ["prieš %n valandą","prieš %n valandas","prieš %n valandų"],
-    "_%n minute ago_::_%n minutes ago_" : ["prieš %n minutę","prieš % minutes","prieš %n minučių"],
+    "_%n year ago_::_%n years ago_" : ["prieš %n metus","prieš %n metus","prieš %n metų","prieš %n metų"],
+    "_%n hour ago_::_%n hours ago_" : ["prieš %n valandą","prieš %n valandas","prieš %n valandų","prieš %n valandų"],
+    "_%n minute ago_::_%n minutes ago_" : ["prieš %n minutę","prieš % minutes","prieš %n minučių","prieš %n minučių"],
     "seconds ago" : "prieš keletą sekundžių",
-    "Module with ID: %s does not exist. Please enable it in your apps settings or contact your administrator." : "Modulio, kurio id: %s, nėra. Prašome jį įjungti savo programėlių nustatymuose arba susisiekti su savo administratoriumi.",
+    "Empty file" : "Tuščias failas",
+    "Module with ID: %s does not exist. Please enable it in your apps settings or contact your administrator." : "Modulio, kurio ID: %s, nėra. Prašome jį įjungti savo programėlių nustatymuose arba susisiekti su savo administratoriumi.",
     "File name is a reserved word" : "Failo pavadinimas negalimas, žodis rezervuotas",
     "File name contains at least one invalid character" : "Failo vardas sudarytas iš neleistinų simbolių",
     "File name is too long" : "Failo pavadinimas per ilgas",
     "Empty filename is not allowed" : "Tuščias failo pavadinimas nėra leidžiamas",
     "App \"%s\" cannot be installed because appinfo file cannot be read." : "Programėlė \"%s\" negali būti įdiegta, kadangi negalima perskaityti appinfo failo.",
     "App \"%s\" cannot be installed because it is not compatible with this version of the server." : "Programėlė \"%s\" negali būti įdiegta, kadangi ji nėra suderinama su serverio versija.",
+    "__language_name__" : "Lietuvių",
     "This is an automatically sent email, please do not reply." : "Tai yra automatinis pranešimas, prašome neatsakyti.",
     "Help" : "Pagalba",
     "Apps" : "Programėlės",
     "Settings" : "Nustatymai",
     "Log out" : "Atsijungti",
     "Users" : "Naudotojai",
-    "APCu" : "APCu",
-    "Redis" : "Redis",
+    "Unknown user" : "Nežinomas naudotojas",
+    "Overview" : "Apžvalga",
     "Basic settings" : "Pagrindiniai nustatymai",
-    "Sharing" : "Dalijimasis",
+    "Sharing" : "Bendrinimas",
     "Security" : "Saugumas",
-    "Encryption" : "Šifravimas",
+    "Groupware" : "Grupinio darbo įranga",
     "Additional settings" : "Papildomi nustatymai",
-    "Tips & tricks" : "Patarimai ir gudrybės",
     "Personal info" : "Asmeninė informacija",
-    "Sync clients" : "Sinchronizavimo klientas",
-    "Unlimited" : "Neribota",
-    "__language_name__" : "Lietuvių",
-    "Verifying" : "Tikrinimas",
-    "Verifying …" : "Tikrinama...",
-    "Verify" : "Patikrinti",
+    "Mobile & desktop" : "Mobilieji ir darbalaukiai",
     "%s enter the database username and name." : "%s įrašykite duomenų bazės naudotojo vardą ir pavadinimą.",
     "%s enter the database username." : "%s įrašykite duomenų bazės naudotojo vardą.",
     "%s enter the database name." : "%s įrašykite duomenų bazės pavadinimą.",
@@ -71,40 +81,21 @@ OC.L10N.register(
     "PostgreSQL username and/or password not valid" : "Neteisingas PostgreSQL naudotojo vardas ir/arba slaptažodis",
     "You need to enter details of an existing account." : "Jūs turite suvesti egzistuojančios paskyros duomenis.",
     "Mac OS X is not supported and %s will not work properly on this platform. Use it at your own risk! " : "Mac OS X nėra palaikomas, %s neveiks tinkamai šioje platformoje. Naudodami prisiimate visą riziką !",
+    "For the best results, please consider using a GNU/Linux server instead." : "Geriausiems rezultatams, apsvarstykite galimybę, vietoj šio, naudoti GNU/Linux serverį",
+    "Please remove the open_basedir setting within your php.ini or switch to 64-bit PHP." : "Pašalinkite savo php.ini faile open_basedir nustatymą arba persijunkite į 64-bitų PHP.",
     "Set an admin username." : "Nustatyti administratoriaus naudotojo vardą.",
     "Set an admin password." : "Nustatyti administratoriaus slaptažodį.",
     "Can't create or write into the data directory %s" : "Negalima nuskaityti arba rašyti į duomenų katalogą. %s",
-    "Invalid Federated Cloud ID" : "Netinkamas Centralizuoto Serverio ID",
-    "Sharing %s failed, because the backend does not allow shares from type %i" : "%s dalinimasis nepavyko, nes sistema nepalaiko šio duomenų tipo %i",
-    "Sharing %s failed, because the file does not exist" : "%s dalinimasis nepavyko, nes failas neegzistuoja.     ",
+    "Invalid Federated Cloud ID" : "Neteisingas federacinės debesijos ID",
+    "Open »%s«" : "Atverti \"%s\"",
+    "%1$s via %2$s" : "%1$s per %2$s",
     "You are not allowed to share %s" : "Jums neleidžiama bendrinti %s",
-    "Sharing %s failed, because you can not share with yourself" : "%s bendrinimas nepavyko, jūs negalite bendrinti su savimi pačiu.",
-    "Sharing %s failed, because the user %s does not exist" : "%s bendrinimas nepavyko, nes naudotojas %s neegzistuoja",
-    "Sharing %s failed, because the user %s is not a member of any groups that %s is a member of" : "%s bendrinimas nepavyko, nes naudotojas %s nėra tos pačios grupės, kaip %s, narys.",
-    "Sharing %s failed, because this item is already shared with %s" : "%s bendrinimas nepavyko, kadangi šis elementas jau yra bendrinamas su %s",
-    "Sharing %s failed, because this item is already shared with user %s" : "%s bendrinimas nepavyko, kadangi šis elementas jau yra bendrinamas su naudotoju %s",
-    "Sharing %s failed, because the group %s does not exist" : "%s bendrinimas nepavyko, nes grupė %s neegzistuoja",
-    "Sharing %s failed, because %s is not a member of the group %s" : " %s bendrinimas nepavyko, nes %s nėra %s grupės narys.",
-    "You need to provide a password to create a public link, only protected links are allowed" : "Viešoms nuorodoms būtinas slaptažodis, leidžiamos tik apsaugotos nuorodos.",
-    "Sharing %s failed, because sharing with links is not allowed" : "Bendrinimas %s nepavyko, kadangi bendrinimas su nuorodomis yra neleidžiamas.",
-    "Not allowed to create a federated share with the same user" : "Negalima dalintis su identišku naudotoju kitame serveryje",
-    "Sharing %s failed, could not find %s, maybe the server is currently unreachable." : "%s pasidalinimas nepavyko, neįmanoma rasti %s, tikėtina, kad serveris šiuo metu nepasiekiamas",
-    "Share type %s is not valid for %s" : "Bendrinimo tipas %s netinka %s",
-    "Cannot set expiration date. Expiration date is in the past" : "Nepavyko nustatyti galiojimo datos. Galiojimo data yra praėjęs laikas.",
-    "Sharing failed, because the user %s is the original sharer" : "Bendrinimas nepavyko, nes naudotojas %s yra bendrintojas.",
-    "Sharing %s failed, because the permissions exceed permissions granted to %s" : "%s bendrinimas nepavyko, kadangi teisės  viršijo %s suteiktas teises.",
-    "Sharing %s failed, because resharing is not allowed" : "%s bendrinimas nepavyko, nes perskirstymas yra neleidžiamas.",
-    "Sharing %s failed, because the file could not be found in the file cache" : "%s dalinimasis nepavyko, nes failo nepavyko rasti podelyje.   ",
     "Can’t increase permissions of %s" : "Negalima pridėti papildomų %s leidimų",
     "Files can’t be shared with delete permissions" : "Failai negali būti bendrinami su trynimo leidimu.",
     "Files can’t be shared with create permissions" : "Failai negali būti bendrinami su sukūrimo leidimu.",
     "Expiration date is in the past" : "Bendrinimo pabaigos data yra praėjęs laikas",
     "Can’t set expiration date more than %s days in the future" : "Negalima nustatyti galiojimo laiko ilgesnio nei %s dienos.",
-    "%s shared »%s« with you" : "%s pasidalino »%s« su jumis",
-    "%s shared »%s« with you." : "%s pasidalino »%s« su Jumis.",
     "Click the button below to open it." : "Norėdami atverti failą, spustelėkite mygtuką žemiau.",
-    "Open »%s«" : "Atverti \"%s\"",
-    "%s via %s" : "%s per %s",
     "The requested share does not exist anymore" : "Pageidaujamas bendrinimas daugiau neegzistuoja.",
     "Could not find category \"%s\"" : "Nepavyko rasti kategorijos „%s“",
     "Sunday" : "Sekmadienis",
@@ -161,16 +152,13 @@ OC.L10N.register(
     "Could not create user" : "Nepavyko sukurti naudotojo",
     "User disabled" : "Naudotojas išjungtas",
     "Login canceled by app" : "Programėlė nutraukė prisijungimo procesą",
-    "No app name specified" : "Nenurodytas programėlės pavadinimas",
-    "App '%s' could not be installed!" : "Nepavyko įdiegti '%s' programėlės!",
-    "App \"%s\" cannot be installed because the following dependencies are not fulfilled: %s" : "Programa \"%s\" negali būti įdiegta, nes nėra įvykdytos: %s",
+    "App \"%1$s\" cannot be installed because the following dependencies are not fulfilled: %2$s" : "Programėlė \"%1$s\" negali būti įdiegta, nes nėra patenkinamos šios priklausomybės: %2$s",
     "a safe home for all your data" : "saugūs namai visiems jūsų duomenims",
     "File is currently busy, please try again later" : "Failas šiuo metu yra užimtas, prašome vėliau pabandyti dar kartą",
     "Can't read file" : "Nepavyksta perskaityti failo",
     "Application is not enabled" : "Programa neįjungta",
     "Authentication error" : "Tapatybės nustatymo klaida",
     "Token expired. Please reload page." : "Pasibaigė prieigos rakto galiojimas. Prašome įkelti puslapį iš naujo.",
-    "Unknown user" : "Nežinomas naudotojas",
     "No database drivers (sqlite, mysql, or postgresql) installed." : "Nėra įdiegtos duomenų bazių tvarkyklės (sqlite, mysql, or postgresql)",
     "Cannot write into \"config\" directory" : "Nepavyksta rašyti į \"config\" katalogą!",
     "Cannot write into \"apps\" directory" : "Nepavyksta įrašyti į \"apps\" katalogą",
@@ -192,37 +180,43 @@ OC.L10N.register(
     "Please ask your server administrator to restart the web server." : "Kreipkitės į savo sistemos administratorių, kad jis perkrautų žiniatinklio serverį.",
     "PostgreSQL >= 9 required" : "Reikalinga PostgreSQL >= 9",
     "Please upgrade your database version" : "Atnaujinkite duomenų bazės versiją.",
+    "Please change the permissions to 0770 so that the directory cannot be listed by other users." : "Pakeiskite leidimus į 0770, kad šis katalogas negalėtų būti išvardytas kitiems naudotojams.",
     "Your data directory is readable by other users" : "Duomenų katalogą gali skaityti kiti naudotojai",
+    "Your data directory must be an absolute path" : "Jūsų duomenų katalogas privalo būti absoliutusis kelias",
     "Check the value of \"datadirectory\" in your configuration" : "Patikrinkite \"duomenų direktorijos\" reikšmę konfigūracijoje.",
     "Your data directory is invalid" : "Neteisingas duomenų katalogas",
     "Ensure there is a file called \".ocdata\" in the root of the data directory." : "Įsitikinkite, kad šakniniame duomenų kataloge yra  yra \".ocdata\" failas.",
+    "Action \"%s\" not supported or implemented." : "Veiksmas \"%s\" nepalaikomas ar neįgyvendintas.",
+    "Could not obtain lock type %d on \"%s\"." : "Nepavyko gauti užrakto tipo %d ties \"%s\".",
     "Storage unauthorized. %s" : "Saugykla nesankcionuota. %s",
     "Storage incomplete configuration. %s" : "Nepilna saugyklos konfigūracija. %s",
     "Storage connection error. %s" : "Saugyklos sujungimo ryšio klaida. %s",
     "Storage is temporarily not available" : "Saugykla yra laikinai neprieinama",
     "Storage connection timeout. %s" : "Sujungimo su saugykla laikas baigėsi. %s",
-    "This can usually be fixed by %sgiving the webserver write access to the config directory%s." : "Tai, dažniausiai, gali būti ištaisyta %ssuteikiant saityno serveriui rašymo prieigą prie konfigūracijos katalogo%s.",
-    "Module with id: %s does not exist. Please enable it in your apps settings or contact your administrator." : "Modulio, kurio id: %s, nėra. Prašome jį įjungti savo programėlių nustatymuose arba susisiekti su savo administratoriumi.",
-    "Server settings" : "Serverio nustatymai",
-    "DB Error: \"%s\"" : "DB klaida: \"%s\"",
-    "Offending command was: \"%s\"" : "Vykdyta komanda buvo: \"%s\"",
-    "You need to enter either an existing account or the administrator." : "Turite prisijungti su egzistuojančia paskyra arba su administratoriumi.",
-    "Offending command was: \"%s\", name: %s, password: %s" : "Vykdyta komanda buvo: \"%s\", name: %s, password: %s",
-    "Setting permissions for %s failed, because the permissions exceed permissions granted to %s" : "%s bendrinimas nepavyko, kadangi teisės  viršijo %s suteiktas teises.",
-    "Setting permissions for %s failed, because the item was not found" : "Teisių nustatymas %s nepavyko, nes elementas nerastas.",
-    "Cannot clear expiration date. Shares are required to have an expiration date." : "Negalima pašalinti galiojimo datos. Bendrinamai privalo turėti galiojimo datą",
-    "Cannot increase permissions of %s" : "Negalima pridėti papildomų %s leidimų",
-    "Files can't be shared with delete permissions" : "Failai negali būti bendrinami su trynimo leidimu.",
-    "Files can't be shared with create permissions" : "Failai negali būti bendrinami su sukūrimo leidimu.",
-    "Cannot set expiration date more than %s days in the future" : "Negalima nustatyti galiojimo laiko ilgesnio nei %s dienos.",
-    "Personal" : "Asmeniniai",
-    "Admin" : "Administravimas",
-    "This can usually be fixed by %sgiving the webserver write access to the apps directory%s or disabling the appstore in the config file." : "Tai, dažniausiai, gali būti ištaisyta, %s suteikiant saityno serveriui rašymo prieigą prie programų katalogo%s arba uždraudžiant appstore konfigūraciniame kataloge. ",
-    "Cannot create \"data\" directory (%s)" : "Nepavyksta sukurti katalogo \"data\" (%s)",
-    "This can usually be fixed by <a href=\"%s\" target=\"_blank\" rel=\"noreferrer\">giving the webserver write access to the root directory</a>." : "<a href=\"%s\" target=\"_blank\" rel=\"noreferrer\">Tai, dažniausiai, gali būti ištaisyta, suteikiant saityno serveriui rašymo prieigą prie šakninio katalogo</a>.",
-    "Permissions can usually be fixed by %sgiving the webserver write access to the root directory%s." : "Teisės gali būti pataisytos, %s suteikiant saityno serveriui rašymo prieigą prie šakninio katalogo%s. ",
-    "Data directory (%s) is readable by other users" : "Duomenų katalogą (%s) skaito kiti naudotojai",
-    "Data directory (%s) is invalid" : "Duomenų katalogas (%s) netinkamas.",
-    "Please check that the data directory contains a file \".ocdata\" in its root." : "Prašau pasitikrinkite, ar duomenų šakninė direktorija turi \".ocdata\" failą."
+    "Create" : "Sukurti",
+    "Change" : "Keisti",
+    "Delete" : "Ištrinti",
+    "Share" : "Bendrinti",
+    "Unlimited" : "Neribota",
+    "Verifying" : "Tikrinimas",
+    "Verifying …" : "Tikrinama...",
+    "Verify" : "Patikrinti",
+    "Sharing %s failed, because the backend does not allow shares from type %i" : "%s dalinimasis nepavyko, nes sistema nepalaiko šio duomenų tipo %i",
+    "Sharing %s failed, because the file does not exist" : "%s dalinimasis nepavyko, nes failas neegzistuoja.     ",
+    "Sharing %s failed, because you can not share with yourself" : "%s bendrinimas nepavyko, jūs negalite bendrinti su savimi pačiu.",
+    "Sharing %1$s failed, because the user %2$s does not exist" : "Nepavyko bendrinti %1$s, nes naudotojo %2$s nėra",
+    "Sharing %1$s failed, because this item is already shared with %2$s" : "Nepavyko bendrinti %1$s, nes šis elementas jau yra bendrinamas su %2$s",
+    "Sharing %1$s failed, because this item is already shared with user %2$s" : "Nepavyko bendrinti %1$s, nes šis elementas jau yra bendrinamas su naudotoju %2$s",
+    "Sharing %1$s failed, because the group %2$s does not exist" : "Nepavyko bendrinti %1$s, nes grupės %2$s nėra",
+    "Sharing %1$s failed, because %2$s is not a member of the group %3$s" : "Nepavyko bendrinti %1$s, nes %2$s nėra grupės %3$s narys",
+    "You need to provide a password to create a public link, only protected links are allowed" : "Viešoms nuorodoms būtinas slaptažodis, leidžiamos tik apsaugotos nuorodos.",
+    "Sharing %s failed, because sharing with links is not allowed" : "Bendrinimas %s nepavyko, kadangi bendrinimas su nuorodomis yra neleidžiamas.",
+    "Not allowed to create a federated share with the same user" : "Negalima sukurti federacinį viešinį su tuo pačiu naudotoju",
+    "Sharing %1$s failed, could not find %2$s, maybe the server is currently unreachable." : "Nepavyko bendrinti %1$s, nepavyko rasti %2$s, galbūt, šiuo metu serveris yra nepasiekiamas.",
+    "Cannot set expiration date. Shares cannot expire later than %s after they have been shared" : "Negalima nustatyti galiojimo pabaigos datos. Bendrinimas negali pasibaigti vėliau nei %s po jo pasidalinimo",
+    "Cannot set expiration date. Expiration date is in the past" : "Nepavyko nustatyti galiojimo datos. Galiojimo data yra praėjęs laikas.",
+    "Sharing failed, because the user %s is the original sharer" : "Bendrinimas nepavyko, nes naudotojas %s yra bendrintojas.",
+    "Sharing %s failed, because resharing is not allowed" : "%s bendrinimas nepavyko, nes perskirstymas yra neleidžiamas.",
+    "Sharing %s failed, because the file could not be found in the file cache" : "%s dalinimasis nepavyko, nes failo nepavyko rasti podelyje.   "
 },
-"nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n%10>=2 && (n%100<10 || n%100>=20) ? 1 : 2);");
+"nplurals=4; plural=(n % 10 == 1 && (n % 100 > 19 || n % 100 < 11) ? 0 : (n % 10 >= 2 && n % 10 <=9) && (n % 100 > 19 || n % 100 < 11) ? 1 : n % 1 != 0 ? 2: 3);");
