@@ -2,8 +2,8 @@
 /**
  * @copyright Copyright (c) 2016, ownCloud, Inc.
  *
- * @author Arthur Schiwon <blizzz@arthur-schiwon.de>
- * @author Georg Ehrke <georg@owncloud.com>
+ * @author Bjoern Schiessle <bjoern@schiessle.org>
+ * @author Georg Ehrke <oc.list@georgehrke.com>
  * @author Joas Schilling <coding@schilljs.com>
  * @author Lukas Reschke <lukas@statuscode.ch>
  * @author Morris Jobke <hey@morrisjobke.de>
@@ -30,41 +30,45 @@
 
 namespace OC;
 
-use OC\Repair\AssetCache;
-use OC\Repair\AvatarPermissions;
-use OC\Repair\CleanTags;
-use OC\Repair\Collation;
-use OC\Repair\DropOldJobs;
-use OC\Repair\MoveUpdaterStepFile;
-use OC\Repair\NC11\CleanPreviews;
-use OC\Repair\NC11\FixMountStorages;
-use OC\Repair\NC11\MoveAvatars;
-use OC\Repair\OldGroupMembershipShares;
-use OC\Repair\RemoveGetETagEntries;
-use OC\Repair\RemoveOldShares;
-use OC\Repair\RemoveRootShares;
-use OC\Repair\SharePropagation;
-use OC\Repair\SqliteAutoincrement;
-use OC\Repair\DropOldTables;
-use OC\Repair\FillETags;
-use OC\Repair\InnoDB;
-use OC\Repair\RepairLegacyStorages;
-use OC\Repair\RepairMimeTypes;
-use OC\Repair\SearchLuceneTables;
-use OC\Repair\UpdateOutdatedOcsIds;
-use OC\Repair\RepairInvalidShares;
-use OC\Repair\RepairUnmergedShares;
 use OCP\AppFramework\QueryException;
 use OCP\Migration\IOutput;
 use OCP\Migration\IRepairStep;
+use OC\AvatarManager;
+use OC\Repair\AddCleanupUpdaterBackupsJob;
+use OC\Repair\CleanTags;
+use OC\Repair\ClearGeneratedAvatarCache;
+use OC\Repair\ClearFrontendCaches;
+use OC\Repair\Collation;
+use OC\Repair\MoveUpdaterStepFile;
+use OC\Repair\NC11\FixMountStorages;
+use OC\Repair\NC13\AddLogRotateJob;
+use OC\Repair\NC13\RepairInvalidPaths;
+use OC\Repair\NC14\AddPreviewBackgroundCleanupJob;
+use OC\Repair\NC14\RepairPendingCronJobs;
+use OC\Repair\NC15\SetVcardDatabaseUID;
+use OC\Repair\OldGroupMembershipShares;
+use OC\Repair\Owncloud\DropAccountTermsTable;
+use OC\Repair\Owncloud\SaveAccountsTableData;
+use OC\Repair\RemoveLinkShares;
+use OC\Repair\RemoveRootShares;
+use OC\Repair\RepairInvalidShares;
+use OC\Repair\RepairMimeTypes;
+use OC\Repair\SqliteAutoincrement;
+use OC\Template\JSCombiner;
+use OC\Template\SCSSCacher;
 use Symfony\Component\EventDispatcher\EventDispatcher;
+use OCP\AppFramework\Utility\ITimeFactory;
+use Symfony\Component\EventDispatcher\EventDispatcherInterface;
 use Symfony\Component\EventDispatcher\GenericEvent;
 
-class Repair implements IOutput{
-	/* @var IRepairStep[] */
+class Repair implements IOutput {
+
+	/** @var IRepairStep[] */
 	private $repairSteps;
+
 	/** @var EventDispatcher */
 	private $dispatcher;
+
 	/** @var string */
 	private $currentStep;
 
@@ -76,7 +80,7 @@ class Repair implements IOutput{
 	 */
 	public function __construct($repairSteps = [], EventDispatcher $dispatcher = null) {
 		$this->repairSteps = $repairSteps;
-		$this->dispatcher = $dispatcher;
+		$this->dispatcher  = $dispatcher;
 	}
 
 	/**
@@ -85,6 +89,7 @@ class Repair implements IOutput{
 	public function run() {
 		if (count($this->repairSteps) === 0) {
 			$this->emit('\OC\Repair', 'info', array('No repair steps available'));
+
 			return;
 		}
 		// run each repair step
@@ -133,36 +138,20 @@ class Repair implements IOutput{
 		return [
 			new Collation(\OC::$server->getConfig(), \OC::$server->getLogger(), \OC::$server->getDatabaseConnection(), false),
 			new RepairMimeTypes(\OC::$server->getConfig()),
-			new RepairLegacyStorages(\OC::$server->getConfig(), \OC::$server->getDatabaseConnection()),
-			new AssetCache(),
-			new FillETags(\OC::$server->getDatabaseConnection()),
 			new CleanTags(\OC::$server->getDatabaseConnection(), \OC::$server->getUserManager()),
-			new DropOldTables(\OC::$server->getDatabaseConnection()),
-			new DropOldJobs(\OC::$server->getJobList()),
-			new RemoveGetETagEntries(\OC::$server->getDatabaseConnection()),
-			new UpdateOutdatedOcsIds(\OC::$server->getConfig()),
 			new RepairInvalidShares(\OC::$server->getConfig(), \OC::$server->getDatabaseConnection()),
-			new SharePropagation(\OC::$server->getConfig()),
-			new RemoveOldShares(\OC::$server->getDatabaseConnection()),
-			new AvatarPermissions(\OC::$server->getDatabaseConnection()),
 			new RemoveRootShares(\OC::$server->getDatabaseConnection(), \OC::$server->getUserManager(), \OC::$server->getLazyRootFolder()),
-			new RepairUnmergedShares(
-				\OC::$server->getConfig(),
-				\OC::$server->getDatabaseConnection(),
-				\OC::$server->getUserManager(),
-				\OC::$server->getGroupManager()
-			),
 			new MoveUpdaterStepFile(\OC::$server->getConfig()),
-			new MoveAvatars(
-				\OC::$server->getJobList(),
-				\OC::$server->getConfig()
-			),
-			new CleanPreviews(
-				\OC::$server->getJobList(),
-				\OC::$server->getUserManager(),
-				\OC::$server->getConfig()
-			),
 			new FixMountStorages(\OC::$server->getDatabaseConnection()),
+			new RepairInvalidPaths(\OC::$server->getDatabaseConnection(), \OC::$server->getConfig()),
+			new AddLogRotateJob(\OC::$server->getJobList()),
+			new ClearFrontendCaches(\OC::$server->getMemCacheFactory(), \OC::$server->query(SCSSCacher::class), \OC::$server->query(JSCombiner::class)),
+			new ClearGeneratedAvatarCache(\OC::$server->getConfig(), \OC::$server->query(AvatarManager::class)),
+			new AddPreviewBackgroundCleanupJob(\OC::$server->getJobList()),
+			new AddCleanupUpdaterBackupsJob(\OC::$server->getJobList()),
+			new RepairPendingCronJobs(\OC::$server->getDatabaseConnection(), \OC::$server->getConfig()),
+			new SetVcardDatabaseUID(\OC::$server->getDatabaseConnection(), \OC::$server->getConfig(), \OC::$server->getLogger()),
+			new RemoveLinkShares(\OC::$server->getDatabaseConnection(), \OC::$server->getConfig(), \OC::$server->getGroupManager(), \OC::$server->getNotificationManager(), \OC::$server->query(ITimeFactory::class)),
 		];
 	}
 
@@ -174,7 +163,7 @@ class Repair implements IOutput{
 	 */
 	public static function getExpensiveRepairSteps() {
 		return [
-			new OldGroupMembershipShares(\OC::$server->getDatabaseConnection(), \OC::$server->getGroupManager()),
+			new OldGroupMembershipShares(\OC::$server->getDatabaseConnection(), \OC::$server->getGroupManager())
 		];
 	}
 
@@ -186,21 +175,14 @@ class Repair implements IOutput{
 	 */
 	public static function getBeforeUpgradeRepairSteps() {
 		$connection = \OC::$server->getDatabaseConnection();
-		$steps = [
-			new InnoDB(),
+		$config     = \OC::$server->getConfig();
+		$steps      = [
 			new Collation(\OC::$server->getConfig(), \OC::$server->getLogger(), $connection, true),
 			new SqliteAutoincrement($connection),
-			new SearchLuceneTables(),
+			new SaveAccountsTableData($connection, $config),
+			new DropAccountTermsTable($connection)
 		];
 
-		//There is no need to delete all previews on every single update
-		//only 7.0.0 through 7.0.2 generated broken previews
-		$currentVersion = \OC::$server->getConfig()->getSystemValue('version');
-		if (version_compare($currentVersion, '7.0.0.0', '>=') &&
-			version_compare($currentVersion, '7.0.3.4', '<=')) {
-			$steps[] = new \OC\Repair\Preview();
-		}
-
 		return $steps;
 	}
 
