@@ -1,13 +1,16 @@
 <?php
 /**
  * @copyright Copyright (c) 2016, ownCloud, Inc.
+ * @copyright Copyright (c) 2016, Lukas Reschke <lukas@statuscode.ch>
  *
  * @author Arthur Schiwon <blizzz@arthur-schiwon.de>
+ * @author Bjoern Schiessle <bjoern@schiessle.org>
  * @author Frank Karlitschek <frank@karlitschek.de>
  * @author Joas Schilling <coding@schilljs.com>
  * @author Lukas Reschke <lukas@statuscode.ch>
  * @author Morris Jobke <hey@morrisjobke.de>
  * @author Robin Appelman <robin@icewind.nl>
+ * @author Roeland Jago Douma <roeland@famdouma.nl>
  * @author Steffen Lindner <mail@steffen-lindner.de>
  * @author Thomas Müller <thomas.mueller@tmit.eu>
  * @author Victor Dubiniuk <dubiniuk@owncloud.com>
@@ -31,12 +34,13 @@
 
 namespace OC;
 
+use OC\DB\MigrationService;
 use OC\Hooks\BasicEmitter;
 use OC\IntegrityCheck\Checker;
 use OC_App;
 use OCP\IConfig;
-use OC\Setup;
 use OCP\ILogger;
+use OCP\Util;
 use Symfony\Component\EventDispatcher\GenericEvent;
 
 /**
@@ -52,21 +56,15 @@ class Updater extends BasicEmitter {
 
 	/** @var ILogger $log */
 	private $log;
-	
+
 	/** @var IConfig */
 	private $config;
 
 	/** @var Checker */
 	private $checker;
 
-	/** @var bool */
-	private $simulateStepEnabled;
-
-	/** @var bool */
-	private $updateStepEnabled;
-
-	/** @var bool */
-	private $skip3rdPartyAppsDisable;
+	/** @var Installer */
+	private $installer;
 
 	private $logLevelNames = [
 		0 => 'Debug',
@@ -80,46 +78,16 @@ class Updater extends BasicEmitter {
 	 * @param IConfig $config
 	 * @param Checker $checker
 	 * @param ILogger $log
+	 * @param Installer $installer
 	 */
 	public function __construct(IConfig $config,
 								Checker $checker,
-								ILogger $log = null) {
+								ILogger $log = null,
+								Installer $installer) {
 		$this->log = $log;
 		$this->config = $config;
 		$this->checker = $checker;
-		$this->simulateStepEnabled = true;
-		$this->updateStepEnabled = true;
-	}
-
-	/**
-	 * Sets whether the database migration simulation must
-	 * be enabled.
-	 * This can be set to false to skip this test.
-	 *
-	 * @param bool $flag true to enable simulation, false otherwise
-	 */
-	public function setSimulateStepEnabled($flag) {
-		$this->simulateStepEnabled = $flag;
-	}
-
-	/**
-	 * Sets whether the update must be performed.
-	 * This can be set to false to skip the actual update.
-	 *
-	 * @param bool $flag true to enable update, false otherwise
-	 */
-	public function setUpdateStepEnabled($flag) {
-		$this->updateStepEnabled = $flag;
-	}
-
-	/**
-	 * Sets whether the update disables 3rd party apps.
-	 * This can be set to true to skip the disable.
-	 *
-	 * @param bool $flag false to not disable, true otherwise
-	 */
-	public function setSkip3rdPartyAppsDisable($flag) {
-		$this->skip3rdPartyAppsDisable = $flag;
+		$this->installer = $installer;
 	}
 
 	/**
@@ -130,10 +98,11 @@ class Updater extends BasicEmitter {
 	 */
 	public function upgrade() {
 		$this->emitRepairEvents();
+		$this->logAllEvents();
 
-		$logLevel = $this->config->getSystemValue('loglevel', \OCP\Util::WARN);
+		$logLevel = $this->config->getSystemValue('loglevel', ILogger::WARN);
 		$this->emit('\OC\Updater', 'setDebugLogLevel', [ $logLevel, $this->logLevelNames[$logLevel] ]);
-		$this->config->setSystemValue('loglevel', \OCP\Util::DEBUG);
+		$this->config->setSystemValue('loglevel', ILogger::DEBUG);
 
 		$wasMaintenanceModeEnabled = $this->config->getSystemValue('maintenance', false);
 
@@ -142,8 +111,16 @@ class Updater extends BasicEmitter {
 			$this->emit('\OC\Updater', 'maintenanceEnabled');
 		}
 
+		// Clear CAN_INSTALL file if not on git
+		if (\OC_Util::getChannel() !== 'git' && is_file(\OC::$configDir.'/CAN_INSTALL')) {
+			if (!unlink(\OC::$configDir . '/CAN_INSTALL')) {
+				$this->log->error('Could not cleanup CAN_INSTALL from your config folder. Please remove this file manually.');
+			}
+		}
+
 		$installedVersion = $this->config->getSystemValue('version', '0.0.0');
 		$currentVersion = implode('.', \OCP\Util::getVersion());
+
 		$this->log->debug('starting upgrade from ' . $installedVersion . ' to ' . $currentVersion, array('app' => 'core'));
 
 		$success = true;
@@ -178,13 +155,13 @@ class Updater extends BasicEmitter {
 	/**
 	 * Return version from which this version is allowed to upgrade from
 	 *
-	 * @return string allowed previous version
+	 * @return array allowed previous versions per vendor
 	 */
-	private function getAllowedPreviousVersion() {
+	private function getAllowedPreviousVersions() {
 		// this should really be a JSON file
 		require \OC::$SERVERROOT . '/version.php';
 		/** @var array $OC_VersionCanBeUpgradedFrom */
-		return implode('.', $OC_VersionCanBeUpgradedFrom);
+		return $OC_VersionCanBeUpgradedFrom;
 	}
 
 	/**
@@ -203,26 +180,29 @@ class Updater extends BasicEmitter {
 	 * Whether an upgrade to a specified version is possible
 	 * @param string $oldVersion
 	 * @param string $newVersion
-	 * @param string $allowedPreviousVersion
+	 * @param array $allowedPreviousVersions
 	 * @return bool
 	 */
-	public function isUpgradePossible($oldVersion, $newVersion, $allowedPreviousVersion) {
-		$allowedUpgrade = (version_compare($allowedPreviousVersion, $oldVersion, '<=')
-			&& (version_compare($oldVersion, $newVersion, '<=') || $this->config->getSystemValue('debug', false)));
+	public function isUpgradePossible($oldVersion, $newVersion, array $allowedPreviousVersions) {
+		$version = explode('.', $oldVersion);
+		$majorMinor = $version[0] . '.' . $version[1];
 
-		if ($allowedUpgrade) {
-			return $allowedUpgrade;
+		$currentVendor = $this->config->getAppValue('core', 'vendor', '');
+
+		// Vendor was not set correctly on install, so we have to white-list known versions
+		if ($currentVendor === '' && isset($allowedPreviousVersions['owncloud'][$oldVersion])) {
+			$currentVendor = 'owncloud';
 		}
 
-		// Upgrade not allowed, someone switching vendor?
-		if ($this->getVendor() !== $this->config->getAppValue('core', 'vendor', '')) {
-			$oldVersion = explode('.', $oldVersion);
-			$newVersion = explode('.', $newVersion);
-
-			return $oldVersion[0] === $newVersion[0] && $oldVersion[1] === $newVersion[1];
+		if ($currentVendor === 'nextcloud') {
+			return isset($allowedPreviousVersions[$currentVendor][$majorMinor])
+				&& (version_compare($oldVersion, $newVersion, '<=') ||
+					$this->config->getSystemValue('debug', false));
 		}
 
-		return false;
+		// Check if the instance can be migrated
+		return isset($allowedPreviousVersions[$currentVendor][$majorMinor]) ||
+			isset($allowedPreviousVersions[$currentVendor][$oldVersion]);
 	}
 
 	/**
@@ -236,8 +216,8 @@ class Updater extends BasicEmitter {
 	 */
 	private function doUpgrade($currentVersion, $installedVersion) {
 		// Stop update if the update is over several major versions
-		$allowedPreviousVersion = $this->getAllowedPreviousVersion();
-		if (!self::isUpgradePossible($installedVersion, $currentVersion, $allowedPreviousVersion)) {
+		$allowedPreviousVersions = $this->getAllowedPreviousVersions();
+		if (!$this->isUpgradePossible($installedVersion, $currentVersion, $allowedPreviousVersions)) {
 			throw new \Exception('Updates between multiple major versions and downgrades are unsupported.');
 		}
 
@@ -258,75 +238,60 @@ class Updater extends BasicEmitter {
 		$repair = new Repair(Repair::getBeforeUpgradeRepairSteps(), \OC::$server->getEventDispatcher());
 		$repair->run();
 
-		// simulate DB upgrade
-		if ($this->simulateStepEnabled) {
-			$this->checkCoreUpgrade();
-
-			// simulate apps DB upgrade
-			$this->checkAppUpgrade($currentVersion);
+		$this->doCoreUpgrade();
 
+		try {
+			// TODO: replace with the new repair step mechanism https://github.com/owncloud/core/pull/24378
+			Setup::installBackgroundJobs();
+		} catch (\Exception $e) {
+			throw new \Exception($e->getMessage());
 		}
 
-		if ($this->updateStepEnabled) {
-			$this->doCoreUpgrade();
+		// update all shipped apps
+		$this->checkAppsRequirements();
+		$this->doAppUpgrade();
 
-			try {
-				// TODO: replace with the new repair step mechanism https://github.com/owncloud/core/pull/24378
-				Setup::installBackgroundJobs();
-			} catch (\Exception $e) {
-				throw new \Exception($e->getMessage());
-			}
+		// Update the appfetchers version so it downloads the correct list from the appstore
+		\OC::$server->getAppFetcher()->setVersion($currentVersion);
 
-			// update all shipped apps
-			$disabledApps = $this->checkAppsRequirements();
-			$this->doAppUpgrade();
+		// upgrade appstore apps
+		$this->upgradeAppStoreApps(\OC::$server->getAppManager()->getInstalledApps());
+		$this->upgradeAppStoreApps(\OC_App::$autoDisabledApps, true);
 
-			// upgrade appstore apps
-			$this->upgradeAppStoreApps($disabledApps);
-
-			// install new shipped apps on upgrade
-			OC_App::loadApps('authentication');
-			$errors = Installer::installShippedApps(true);
-			foreach ($errors as $appId => $exception) {
-				/** @var \Exception $exception */
-				$this->log->logException($exception, ['app' => $appId]);
-				$this->emit('\OC\Updater', 'failure', [$appId . ': ' . $exception->getMessage()]);
-			}
-
-			// post-upgrade repairs
-			$repair = new Repair(Repair::getRepairSteps(), \OC::$server->getEventDispatcher());
-			$repair->run();
-
-			//Invalidate update feed
-			$this->config->setAppValue('core', 'lastupdatedat', 0);
-
-			// Check for code integrity if not disabled
-			if(\OC::$server->getIntegrityCodeChecker()->isCodeCheckEnforced()) {
-				$this->emit('\OC\Updater', 'startCheckCodeIntegrity');
-				$this->checker->runInstanceVerification();
-				$this->emit('\OC\Updater', 'finishedCheckCodeIntegrity');
-			}
-
-			// only set the final version if everything went well
-			$this->config->setSystemValue('version', implode('.', \OCP\Util::getVersion()));
-			$this->config->setAppValue('core', 'vendor', $this->getVendor());
+		// install new shipped apps on upgrade
+		OC_App::loadApps(['authentication']);
+		$errors = Installer::installShippedApps(true);
+		foreach ($errors as $appId => $exception) {
+			/** @var \Exception $exception */
+			$this->log->logException($exception, ['app' => $appId]);
+			$this->emit('\OC\Updater', 'failure', [$appId . ': ' . $exception->getMessage()]);
 		}
-	}
 
-	protected function checkCoreUpgrade() {
-		$this->emit('\OC\Updater', 'dbSimulateUpgradeBefore');
+		// post-upgrade repairs
+		$repair = new Repair(Repair::getRepairSteps(), \OC::$server->getEventDispatcher());
+		$repair->run();
 
-		// simulate core DB upgrade
-		\OC_DB::simulateUpdateDbFromStructure(\OC::$SERVERROOT . '/db_structure.xml');
+		//Invalidate update feed
+		$this->config->setAppValue('core', 'lastupdatedat', 0);
 
-		$this->emit('\OC\Updater', 'dbSimulateUpgrade');
+		// Check for code integrity if not disabled
+		if(\OC::$server->getIntegrityCodeChecker()->isCodeCheckEnforced()) {
+			$this->emit('\OC\Updater', 'startCheckCodeIntegrity');
+			$this->checker->runInstanceVerification();
+			$this->emit('\OC\Updater', 'finishedCheckCodeIntegrity');
+		}
+
+		// only set the final version if everything went well
+		$this->config->setSystemValue('version', implode('.', Util::getVersion()));
+		$this->config->setAppValue('core', 'vendor', $this->getVendor());
 	}
 
 	protected function doCoreUpgrade() {
 		$this->emit('\OC\Updater', 'dbUpgradeBefore');
 
-		// do the real upgrade
-		\OC_DB::updateDbFromStructure(\OC::$SERVERROOT . '/db_structure.xml');
+		// execute core migrations
+		$ms = new MigrationService('core', \OC::$server->getDatabaseConnection());
+		$ms->migrate();
 
 		$this->emit('\OC\Updater', 'dbUpgrade');
 	}
@@ -338,10 +303,11 @@ class Updater extends BasicEmitter {
 		$apps = \OC_App::getEnabledApps();
 		$this->emit('\OC\Updater', 'appUpgradeCheckBefore');
 
+		$appManager = \OC::$server->getAppManager();
 		foreach ($apps as $appId) {
 			$info = \OC_App::getAppInfo($appId);
 			$compatible = \OC_App::isAppCompatible($version, $info);
-			$isShipped = \OC_App::isShipped($appId);
+			$isShipped = $appManager->isShipped($appId);
 
 			if ($compatible && $isShipped && \OC_App::shouldUpgrade($appId)) {
 				/**
@@ -389,7 +355,7 @@ class Updater extends BasicEmitter {
 				if(!isset($stacks[$type])) {
 					$stacks[$type] = array();
 				}
-				if (\OC_App::isType($appId, $type)) {
+				if (\OC_App::isType($appId, [$type])) {
 					$stacks[$type][] = $appId;
 					$priorityType = true;
 					break;
@@ -410,7 +376,7 @@ class Updater extends BasicEmitter {
 					// load authentication, filesystem and logging apps after
 					// upgrading them. Other apps my need to rely on modifying
 					// user and/or filesystem aspects.
-					\OC_App::loadApp($appId, false);
+					\OC_App::loadApp($appId);
 				}
 			}
 		}
@@ -428,13 +394,17 @@ class Updater extends BasicEmitter {
 	private function checkAppsRequirements() {
 		$isCoreUpgrade = $this->isCodeUpgrade();
 		$apps = OC_App::getEnabledApps();
-		$version = \OCP\Util::getVersion();
+		$version = implode('.', Util::getVersion());
 		$disabledApps = [];
+		$appManager = \OC::$server->getAppManager();
 		foreach ($apps as $app) {
 			// check if the app is compatible with this version of ownCloud
 			$info = OC_App::getAppInfo($app);
-			if(!OC_App::isAppCompatible($version, $info)) {
-				OC_App::disable($app);
+			if($info === null || !OC_App::isAppCompatible($version, $info)) {
+				if ($appManager->isShipped($app)) {
+					throw new \UnexpectedValueException('The files of the app "' . $app . '" were not correctly replaced before running the update');
+				}
+				\OC::$server->getAppManager()->disableApp($app);
 				$this->emit('\OC\Updater', 'incompatibleAppDisabled', array($app));
 			}
 			// no need to disable any app in case this is a non-core upgrade
@@ -442,20 +412,13 @@ class Updater extends BasicEmitter {
 				continue;
 			}
 			// shipped apps will remain enabled
-			if (OC_App::isShipped($app)) {
+			if ($appManager->isShipped($app)) {
 				continue;
 			}
 			// authentication and session apps will remain enabled as well
 			if (OC_App::isType($app, ['session', 'authentication'])) {
 				continue;
 			}
-
-			// disable any other 3rd party apps if not overriden
-			if(!$this->skip3rdPartyAppsDisable) {
-				\OC_App::disable($app);
-				$disabledApps[]= $app;
-				$this->emit('\OC\Updater', 'thirdPartyAppDisabled', array($app));
-			};
 		}
 		return $disabledApps;
 	}
@@ -465,7 +428,7 @@ class Updater extends BasicEmitter {
 	 */
 	private function isCodeUpgrade() {
 		$installedVersion = $this->config->getSystemValue('version', '0.0.0');
-		$currentVersion = implode('.', \OCP\Util::getVersion());
+		$currentVersion = implode('.', Util::getVersion());
 		if (version_compare($currentVersion, $installedVersion, '>')) {
 			return true;
 		}
@@ -474,16 +437,22 @@ class Updater extends BasicEmitter {
 
 	/**
 	 * @param array $disabledApps
+	 * @param bool $reenable
 	 * @throws \Exception
 	 */
-	private function upgradeAppStoreApps(array $disabledApps) {
+	private function upgradeAppStoreApps(array $disabledApps, $reenable = false) {
 		foreach($disabledApps as $app) {
 			try {
-				if (Installer::isUpdateAvailable($app)) {
-					$ocsId = \OC::$server->getConfig()->getAppValue($app, 'ocsid', '');
+				$this->emit('\OC\Updater', 'checkAppStoreAppBefore', [$app]);
+				if ($this->installer->isUpdateAvailable($app)) {
+					$this->emit('\OC\Updater', 'upgradeAppStoreApp', [$app]);
+					$this->installer->updateAppstoreApp($app);
+				}
+				$this->emit('\OC\Updater', 'checkAppStoreApp', [$app]);
 
-					$this->emit('\OC\Updater', 'upgradeAppStoreApp', array($app));
-					Installer::updateAppByOCSId($ocsId);
+				if ($reenable) {
+					$ocApp = new \OC_App();
+					$ocApp->enable($app);
 				}
 			} catch (\Exception $ex) {
 				$this->log->logException($ex, ['app' => 'core']);
@@ -518,5 +487,138 @@ class Updater extends BasicEmitter {
 		});
 	}
 
+	private function logAllEvents() {
+		$log = $this->log;
+
+		$dispatcher = \OC::$server->getEventDispatcher();
+		$dispatcher->addListener('\OC\DB\Migrator::executeSql', function($event) use ($log) {
+			if (!$event instanceof GenericEvent) {
+				return;
+			}
+			$log->info('\OC\DB\Migrator::executeSql: ' . $event->getSubject() . ' (' . $event->getArgument(0) . ' of ' . $event->getArgument(1) . ')', ['app' => 'updater']);
+		});
+		$dispatcher->addListener('\OC\DB\Migrator::checkTable', function($event) use ($log) {
+			if (!$event instanceof GenericEvent) {
+				return;
+			}
+			$log->info('\OC\DB\Migrator::checkTable: ' . $event->getSubject() . ' (' . $event->getArgument(0) . ' of ' . $event->getArgument(1) . ')', ['app' => 'updater']);
+		});
+
+		$repairListener = function($event) use ($log) {
+			if (!$event instanceof GenericEvent) {
+				return;
+			}
+			switch ($event->getSubject()) {
+				case '\OC\Repair::startProgress':
+					$log->info('\OC\Repair::startProgress: Starting ... ' . $event->getArgument(1) .  ' (' . $event->getArgument(0) . ')', ['app' => 'updater']);
+					break;
+				case '\OC\Repair::advance':
+					$desc = $event->getArgument(1);
+					if (empty($desc)) {
+						$desc = '';
+					}
+					$log->info('\OC\Repair::advance: ' . $desc . ' (' . $event->getArgument(0) . ')', ['app' => 'updater']);
+
+					break;
+				case '\OC\Repair::finishProgress':
+					$log->info('\OC\Repair::finishProgress', ['app' => 'updater']);
+					break;
+				case '\OC\Repair::step':
+					$log->info('\OC\Repair::step: Repair step: ' . $event->getArgument(0), ['app' => 'updater']);
+					break;
+				case '\OC\Repair::info':
+					$log->info('\OC\Repair::info: Repair info: ' . $event->getArgument(0), ['app' => 'updater']);
+					break;
+				case '\OC\Repair::warning':
+					$log->warning('\OC\Repair::warning: Repair warning: ' . $event->getArgument(0), ['app' => 'updater']);
+					break;
+				case '\OC\Repair::error':
+					$log->error('\OC\Repair::error: Repair error: ' . $event->getArgument(0), ['app' => 'updater']);
+					break;
+			}
+		};
+
+		$dispatcher->addListener('\OC\Repair::startProgress', $repairListener);
+		$dispatcher->addListener('\OC\Repair::advance', $repairListener);
+		$dispatcher->addListener('\OC\Repair::finishProgress', $repairListener);
+		$dispatcher->addListener('\OC\Repair::step', $repairListener);
+		$dispatcher->addListener('\OC\Repair::info', $repairListener);
+		$dispatcher->addListener('\OC\Repair::warning', $repairListener);
+		$dispatcher->addListener('\OC\Repair::error', $repairListener);
+
+
+		$this->listen('\OC\Updater', 'maintenanceEnabled', function () use($log) {
+			$log->info('\OC\Updater::maintenanceEnabled: Turned on maintenance mode', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'maintenanceDisabled', function () use($log) {
+			$log->info('\OC\Updater::maintenanceDisabled: Turned off maintenance mode', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'maintenanceActive', function () use($log) {
+			$log->info('\OC\Updater::maintenanceActive: Maintenance mode is kept active', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'updateEnd', function ($success) use($log) {
+			if ($success) {
+				$log->info('\OC\Updater::updateEnd: Update successful', ['app' => 'updater']);
+			} else {
+				$log->error('\OC\Updater::updateEnd: Update failed', ['app' => 'updater']);
+			}
+		});
+		$this->listen('\OC\Updater', 'dbUpgradeBefore', function () use($log) {
+			$log->info('\OC\Updater::dbUpgradeBefore: Updating database schema', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'dbUpgrade', function () use($log) {
+			$log->info('\OC\Updater::dbUpgrade: Updated database', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'dbSimulateUpgradeBefore', function () use($log) {
+			$log->info('\OC\Updater::dbSimulateUpgradeBefore: Checking whether the database schema can be updated (this can take a long time depending on the database size)', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'dbSimulateUpgrade', function () use($log) {
+			$log->info('\OC\Updater::dbSimulateUpgrade: Checked database schema update', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'incompatibleAppDisabled', function ($app) use($log) {
+			$log->info('\OC\Updater::incompatibleAppDisabled: Disabled incompatible app: ' . $app, ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'checkAppStoreAppBefore', function ($app) use($log) {
+			$log->info('\OC\Updater::checkAppStoreAppBefore: Checking for update of app "' . $app . '" in appstore', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'upgradeAppStoreApp', function ($app) use($log) {
+			$log->info('\OC\Updater::upgradeAppStoreApp: Update app "' . $app . '" from appstore', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'checkAppStoreApp', function ($app) use($log) {
+			$log->info('\OC\Updater::checkAppStoreApp: Checked for update of app "' . $app . '" in appstore', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'appUpgradeCheckBefore', function () use ($log) {
+			$log->info('\OC\Updater::appUpgradeCheckBefore: Checking updates of apps', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'appSimulateUpdate', function ($app) use ($log) {
+			$log->info('\OC\Updater::appSimulateUpdate: Checking whether the database schema for <' . $app . '> can be updated (this can take a long time depending on the database size)', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'appUpgradeCheck', function () use ($log) {
+			$log->info('\OC\Updater::appUpgradeCheck: Checked database schema update for apps', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'appUpgradeStarted', function ($app) use ($log) {
+			$log->info('\OC\Updater::appUpgradeStarted: Updating <' . $app . '> ...', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'appUpgrade', function ($app, $version) use ($log) {
+			$log->info('\OC\Updater::appUpgrade: Updated <' . $app . '> to ' . $version, ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'failure', function ($message) use($log) {
+			$log->error('\OC\Updater::failure: ' . $message, ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'setDebugLogLevel', function () use($log) {
+			$log->info('\OC\Updater::setDebugLogLevel: Set log level to debug', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'resetLogLevel', function ($logLevel, $logLevelName) use($log) {
+			$log->info('\OC\Updater::resetLogLevel: Reset log level to ' . $logLevelName . '(' . $logLevel . ')', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'startCheckCodeIntegrity', function () use($log) {
+			$log->info('\OC\Updater::startCheckCodeIntegrity: Starting code integrity check...', ['app' => 'updater']);
+		});
+		$this->listen('\OC\Updater', 'finishedCheckCodeIntegrity', function () use($log) {
+			$log->info('\OC\Updater::finishedCheckCodeIntegrity: Finished code integrity check', ['app' => 'updater']);
+		});
+
+	}
+
 }
 
