@@ -1,4 +1,5 @@
 <?php
+declare(strict_types=1);
 /**
  * @copyright Copyright (c) 2016, ownCloud, Inc.
  * @copyright Copyright (c) 2016, Lukas Reschke <lukas@statuscode.ch>
@@ -50,12 +51,13 @@
  *
  */
 use OC\App\DependencyAnalyzer;
-use OC\App\InfoParser;
 use OC\App\Platform;
 use OC\DB\MigrationService;
 use OC\Installer;
 use OC\Repair;
+use OC\ServerNotAvailableException;
 use OCP\App\ManagerEvent;
+use OCP\ILogger;
 
 /**
  * This class manages the apps. It allows them to register and integrate in the
@@ -63,23 +65,22 @@ use OCP\App\ManagerEvent;
  * upgrading and removing apps.
  */
 class OC_App {
-	static private $appVersion = [];
-	static private $adminForms = array();
-	static private $personalForms = array();
-	static private $appInfo = array();
-	static private $appTypes = array();
-	static private $loadedApps = array();
-	static private $altLogin = array();
+	static private $adminForms = [];
+	static private $personalForms = [];
+	static private $appTypes = [];
+	static private $loadedApps = [];
+	static private $altLogin = [];
 	static private $alreadyRegistered = [];
+	static public $autoDisabledApps = [];
 	const officialApp = 200;
 
 	/**
 	 * clean the appId
 	 *
-	 * @param string|boolean $app AppId that needs to be cleaned
+	 * @param string $app AppId that needs to be cleaned
 	 * @return string
 	 */
-	public static function cleanAppId($app) {
+	public static function cleanAppId(string $app): string {
 		return str_replace(array('\0', '/', '\\', '..'), '', $app);
 	}
 
@@ -89,23 +90,23 @@ class OC_App {
 	 * @param string $app
 	 * @return bool
 	 */
-	public static function isAppLoaded($app) {
+	public static function isAppLoaded(string $app): bool {
 		return in_array($app, self::$loadedApps, true);
 	}
 
 	/**
 	 * loads all apps
 	 *
-	 * @param string[] | string | null $types
+	 * @param string[] $types
 	 * @return bool
 	 *
 	 * This function walks through the ownCloud directory and loads all apps
 	 * it can find. A directory contains an app if the file /appinfo/info.xml
 	 * exists.
 	 *
-	 * if $types is set, only apps of those types will be loaded
+	 * if $types is set to non-empty array, only apps of those types will be loaded
 	 */
-	public static function loadApps($types = null) {
+	public static function loadApps(array $types = []): bool {
 		if (\OC::$server->getSystemConfig()->getValue('maintenance', false)) {
 			return false;
 		}
@@ -123,7 +124,7 @@ class OC_App {
 		// prevent app.php from printing output
 		ob_start();
 		foreach ($apps as $app) {
-			if ((is_null($types) or self::isType($app, $types)) && !in_array($app, self::$loadedApps)) {
+			if (($types === [] or self::isType($app, $types)) && !in_array($app, self::$loadedApps)) {
 				self::loadApp($app);
 			}
 		}
@@ -138,7 +139,7 @@ class OC_App {
 	 * @param string $app
 	 * @throws Exception
 	 */
-	public static function loadApp($app) {
+	public static function loadApp(string $app) {
 		self::$loadedApps[] = $app;
 		$appPath = self::getAppPath($app);
 		if($appPath === false) {
@@ -152,20 +153,17 @@ class OC_App {
 			\OC::$server->getEventLogger()->start('load_app_' . $app, 'Load app: ' . $app);
 			try {
 				self::requireAppFile($app);
-			} catch (Error $ex) {
+			} catch (Throwable $ex) {
+				if($ex instanceof ServerNotAvailableException) {
+					throw $ex;
+				}
 				\OC::$server->getLogger()->logException($ex);
 				if (!\OC::$server->getAppManager()->isShipped($app)) {
 					// Only disable apps which are not shipped
-					self::disable($app);
+					\OC::$server->getAppManager()->disableApp($app);
+					self::$autoDisabledApps[] = $app;
 				}
 			}
-			if (self::isType($app, array('authentication'))) {
-				// since authentication apps affect the "is app enabled for group" check,
-				// the enabled apps cache needs to be cleared to make sure that the
-				// next time getEnableApps() is called it will also include apps that were
-				// enabled for groups
-				self::$enabledAppsCache = array();
-			}
 			\OC::$server->getEventLogger()->end('load_app_' . $app);
 		}
 
@@ -185,6 +183,28 @@ class OC_App {
 				\OC::$server->getActivityManager()->registerProvider($provider);
 			}
 		}
+
+		if (!empty($info['settings']['admin'])) {
+			foreach ($info['settings']['admin'] as $setting) {
+				\OC::$server->getSettingsManager()->registerSetting('admin', $setting);
+			}
+		}
+		if (!empty($info['settings']['admin-section'])) {
+			foreach ($info['settings']['admin-section'] as $section) {
+				\OC::$server->getSettingsManager()->registerSection('admin', $section);
+			}
+		}
+		if (!empty($info['settings']['personal'])) {
+			foreach ($info['settings']['personal'] as $setting) {
+				\OC::$server->getSettingsManager()->registerSetting('personal', $setting);
+			}
+		}
+		if (!empty($info['settings']['personal-section'])) {
+			foreach ($info['settings']['personal-section'] as $section) {
+				\OC::$server->getSettingsManager()->registerSection('personal', $section);
+			}
+		}
+
 		if (!empty($info['collaboration']['plugins'])) {
 			// deal with one or many plugin entries
 			$plugins = isset($info['collaboration']['plugins']['plugin']['@value']) ?
@@ -208,7 +228,7 @@ class OC_App {
 	 * @param string $app
 	 * @param string $path
 	 */
-	public static function registerAutoloading($app, $path) {
+	public static function registerAutoloading(string $app, string $path) {
 		$key = $app . '-' . $path;
 		if(isset(self::$alreadyRegistered[$key])) {
 			return;
@@ -240,7 +260,7 @@ class OC_App {
 	 * @param string $app app name
 	 * @throws Error
 	 */
-	private static function requireAppFile($app) {
+	private static function requireAppFile(string $app) {
 		// encapsulated here to avoid variable scope conflicts
 		require_once $app . '/appinfo/app.php';
 	}
@@ -249,13 +269,10 @@ class OC_App {
 	 * check if an app is of a specific type
 	 *
 	 * @param string $app
-	 * @param string|array $types
+	 * @param array $types
 	 * @return bool
 	 */
-	public static function isType($app, $types) {
-		if (is_string($types)) {
-			$types = array($types);
-		}
+	public static function isType(string $app, array $types): bool {
 		$appTypes = self::getAppTypes($app);
 		foreach ($types as $type) {
 			if (array_search($type, $appTypes) !== false) {
@@ -271,7 +288,7 @@ class OC_App {
 	 * @param string $app
 	 * @return array
 	 */
-	private static function getAppTypes($app) {
+	private static function getAppTypes(string $app): array {
 		//load the cache
 		if (count(self::$appTypes) == 0) {
 			self::$appTypes = \OC::$server->getAppConfig()->getValues(false, 'types');
@@ -279,16 +296,17 @@ class OC_App {
 
 		if (isset(self::$appTypes[$app])) {
 			return explode(',', self::$appTypes[$app]);
-		} else {
-			return array();
 		}
+
+		return [];
 	}
 
 	/**
 	 * read app types from info.xml and cache them in the database
 	 */
-	public static function setAppTypes($app) {
-		$appData = self::getAppInfo($app);
+	public static function setAppTypes(string $app) {
+		$appManager = \OC::$server->getAppManager();
+		$appData = $appManager->getAppInfo($app);
 		if(!is_array($appData)) {
 			return;
 		}
@@ -300,21 +318,17 @@ class OC_App {
 			$appData['types'] = [];
 		}
 
-		\OC::$server->getAppConfig()->setValue($app, 'types', $appTypes);
+		$config = \OC::$server->getConfig();
+		$config->setAppValue($app, 'types', $appTypes);
 
-		if (\OC::$server->getAppManager()->hasProtectedAppType($appData['types'])) {
-			$enabled = \OC::$server->getAppConfig()->getValue($app, 'enabled', 'yes');
+		if ($appManager->hasProtectedAppType($appData['types'])) {
+			$enabled = $config->getAppValue($app, 'enabled', 'yes');
 			if ($enabled !== 'yes' && $enabled !== 'no') {
-				\OC::$server->getAppConfig()->setValue($app, 'enabled', 'yes');
+				$config->setAppValue($app, 'enabled', 'yes');
 			}
 		}
 	}
 
-	/**
-	 * get all enabled apps
-	 */
-	protected static $enabledAppsCache = array();
-
 	/**
 	 * Returns apps enabled for the current user.
 	 *
@@ -323,9 +337,9 @@ class OC_App {
 	 * currently logged in one
 	 * @return string[]
 	 */
-	public static function getEnabledApps($forceRefresh = false, $all = false) {
+	public static function getEnabledApps(bool $forceRefresh = false, bool $all = false): array {
 		if (!\OC::$server->getSystemConfig()->getValue('installed', false)) {
-			return array();
+			return [];
 		}
 		// in incognito mode or when logged out, $user will be false,
 		// which is also the case during an upgrade
@@ -358,7 +372,7 @@ class OC_App {
 	 *
 	 * This function checks whether or not an app is enabled.
 	 */
-	public static function isEnabled($app) {
+	public static function isEnabled(string $app): bool {
 		return \OC::$server->getAppManager()->isEnabledForUser($app);
 	}
 
@@ -372,11 +386,11 @@ class OC_App {
 	 *
 	 * This function set an app as enabled in appconfig.
 	 */
-	public function enable($appId,
-						   $groups = null) {
-		self::$enabledAppsCache = []; // flush
+	public function enable(string $appId,
+						   array $groups = []) {
 
 		// Check if app is already downloaded
+		/** @var Installer $installer */
 		$installer = \OC::$server->query(Installer::class);
 		$isDownloaded = $installer->isDownloaded($appId);
 
@@ -387,7 +401,7 @@ class OC_App {
 		$installer->installApp($appId);
 
 		$appManager = \OC::$server->getAppManager();
-		if (!is_null($groups)) {
+		if ($groups !== []) {
 			$groupManager = \OC::$server->getGroupManager();
 			$groupsList = [];
 			foreach ($groups as $group) {
@@ -402,68 +416,6 @@ class OC_App {
 		}
 	}
 
-	/**
-	 * @param string $app
-	 * @return bool
-	 */
-	public static function removeApp($app) {
-		if (\OC::$server->getAppManager()->isShipped($app)) {
-			return false;
-		}
-
-		$installer = \OC::$server->query(Installer::class);
-		return $installer->removeApp($app);
-	}
-
-	/**
-	 * This function set an app as disabled in appconfig.
-	 *
-	 * @param string $app app
-	 * @throws Exception
-	 */
-	public static function disable($app) {
-		// flush
-		self::$enabledAppsCache = array();
-
-		// run uninstall steps
-		$appData = OC_App::getAppInfo($app);
-		if (!is_null($appData)) {
-			OC_App::executeRepairSteps($app, $appData['repair-steps']['uninstall']);
-		}
-
-		// emit disable hook - needed anymore ?
-		\OC_Hook::emit('OC_App', 'pre_disable', array('app' => $app));
-
-		// finally disable it
-		$appManager = \OC::$server->getAppManager();
-		$appManager->disableApp($app);
-	}
-
-	// This is private as well. It simply works, so don't ask for more details
-	private static function proceedNavigation($list) {
-		usort($list, function($a, $b) {
-			if (isset($a['order']) && isset($b['order'])) {
-				return ($a['order'] < $b['order']) ? -1 : 1;
-			} else if (isset($a['order']) || isset($b['order'])) {
-				return isset($a['order']) ? -1 : 1;
-			} else {
-				return ($a['name'] < $b['name']) ? -1 : 1;
-			}
-		});
-
-		$activeApp = OC::$server->getNavigationManager()->getActiveEntry();
-		foreach ($list as $index => &$navEntry) {
-			if ($navEntry['id'] == $activeApp) {
-				$navEntry['active'] = true;
-			} else {
-				$navEntry['active'] = false;
-			}
-		}
-		unset($navEntry);
-
-		return $list;
-	}
-
 	/**
 	 * Get the path where to install apps
 	 *
@@ -480,7 +432,7 @@ class OC_App {
 			}
 		}
 
-		\OCP\Util::writeLog('core', 'No application directories are marked as writable.', \OCP\Util::ERROR);
+		\OCP\Util::writeLog('core', 'No application directories are marked as writable.', ILogger::ERROR);
 		return null;
 	}
 
@@ -491,18 +443,18 @@ class OC_App {
 	 * @param string $appId
 	 * @return false|string
 	 */
-	public static function findAppInDirectories($appId) {
+	public static function findAppInDirectories(string $appId) {
 		$sanitizedAppId = self::cleanAppId($appId);
 		if($sanitizedAppId !== $appId) {
 			return false;
 		}
-		static $app_dir = array();
+		static $app_dir = [];
 
 		if (isset($app_dir[$appId])) {
 			return $app_dir[$appId];
 		}
 
-		$possibleApps = array();
+		$possibleApps = [];
 		foreach (OC::$APPSROOTS as $dir) {
 			if (file_exists($dir['path'] . '/' . $appId)) {
 				$possibleApps[] = $dir;
@@ -516,7 +468,7 @@ class OC_App {
 			$app_dir[$appId] = $dir;
 			return $dir;
 		} else {
-			$versionToLoad = array();
+			$versionToLoad = [];
 			foreach ($possibleApps as $possibleApp) {
 				$version = self::getAppVersionByPath($possibleApp['path'] . '/' . $appId);
 				if (empty($versionToLoad) || version_compare($version, $versionToLoad['version'], '>')) {
@@ -539,7 +491,7 @@ class OC_App {
 	 * @param string $appId
 	 * @return string|false
 	 */
-	public static function getAppPath($appId) {
+	public static function getAppPath(string $appId) {
 		if ($appId === null || trim($appId) === '') {
 			return false;
 		}
@@ -557,7 +509,7 @@ class OC_App {
 	 * @param string $appId
 	 * @return string|false
 	 */
-	public static function getAppWebPath($appId) {
+	public static function getAppWebPath(string $appId) {
 		if (($dir = self::findAppInDirectories($appId)) != false) {
 			return OC::$WEBROOT . $dir['url'] . '/' . $appId;
 		}
@@ -570,15 +522,10 @@ class OC_App {
 	 * @param string $appId
 	 * @param bool $useCache
 	 * @return string
+	 * @deprecated 14.0.0 use \OC::$server->getAppManager()->getAppVersion()
 	 */
-	public static function getAppVersion($appId, $useCache = true) {
-		if($useCache && isset(self::$appVersion[$appId])) {
-			return self::$appVersion[$appId];
-		}
-
-		$file = self::getAppPath($appId);
-		self::$appVersion[$appId] = ($file !== false) ? self::getAppVersionByPath($file) : '0';
-		return self::$appVersion[$appId];
+	public static function getAppVersion(string $appId, bool $useCache = true): string {
+		return \OC::$server->getAppManager()->getAppVersion($appId, $useCache);
 	}
 
 	/**
@@ -587,9 +534,9 @@ class OC_App {
 	 * @param string $path
 	 * @return string
 	 */
-	public static function getAppVersionByPath($path) {
+	public static function getAppVersionByPath(string $path): string {
 		$infoFile = $path . '/appinfo/info.xml';
-		$appData = self::getAppInfo($infoFile, true);
+		$appData = \OC::$server->getAppManager()->getAppInfo($infoFile, true);
 		return isset($appData['version']) ? $appData['version'] : '';
 	}
 
@@ -602,67 +549,38 @@ class OC_App {
 	 * @param string $lang
 	 * @return array|null
 	 * @note all data is read from info.xml, not just pre-defined fields
+	 * @deprecated 14.0.0 use \OC::$server->getAppManager()->getAppInfo()
 	 */
-	public static function getAppInfo($appId, $path = false, $lang = null) {
-		if ($path) {
-			$file = $appId;
-		} else {
-			if ($lang === null && isset(self::$appInfo[$appId])) {
-				return self::$appInfo[$appId];
-			}
-			$appPath = self::getAppPath($appId);
-			if($appPath === false) {
-				return null;
-			}
-			$file = $appPath . '/appinfo/info.xml';
-		}
-
-		$parser = new InfoParser(\OC::$server->getMemCacheFactory()->createLocal('core.appinfo'));
-		$data = $parser->parse($file);
-
-		if (is_array($data)) {
-			$data = OC_App::parseAppInfo($data, $lang);
-		}
-		if(isset($data['ocsid'])) {
-			$storedId = \OC::$server->getConfig()->getAppValue($appId, 'ocsid');
-			if($storedId !== '' && $storedId !== $data['ocsid']) {
-				$data['ocsid'] = $storedId;
-			}
-		}
-
-		if ($lang === null) {
-			self::$appInfo[$appId] = $data;
-		}
-
-		return $data;
+	public static function getAppInfo(string $appId, bool $path = false, string $lang = null) {
+		return \OC::$server->getAppManager()->getAppInfo($appId, $path, $lang);
 	}
 
 	/**
 	 * Returns the navigation
 	 *
 	 * @return array
+	 * @deprecated 14.0.0 use \OC::$server->getNavigationManager()->getAll()
 	 *
 	 * This function returns an array containing all entries added. The
 	 * entries are sorted by the key 'order' ascending. Additional to the keys
 	 * given for each app the following keys exist:
 	 *   - active: boolean, signals if the user is on this navigation entry
 	 */
-	public static function getNavigation() {
-		$entries = OC::$server->getNavigationManager()->getAll();
-		return self::proceedNavigation($entries);
+	public static function getNavigation(): array {
+		return OC::$server->getNavigationManager()->getAll();
 	}
 
 	/**
 	 * Returns the Settings Navigation
 	 *
 	 * @return string[]
+	 * @deprecated 14.0.0 use \OC::$server->getNavigationManager()->getAll('settings')
 	 *
 	 * This function returns an array containing all settings pages added. The
 	 * entries are sorted by the key 'order' ascending.
 	 */
-	public static function getSettingsNavigation() {
-		$entries = OC::$server->getNavigationManager()->getAll('settings');
-		return self::proceedNavigation($entries);
+	public static function getSettingsNavigation(): array {
+		return OC::$server->getNavigationManager()->getAll('settings');
 	}
 
 	/**
@@ -670,10 +588,10 @@ class OC_App {
 	 *
 	 * @return string
 	 */
-	public static function getCurrentApp() {
+	public static function getCurrentApp(): string {
 		$request = \OC::$server->getRequest();
 		$script = substr($request->getScriptName(), strlen(OC::$WEBROOT) + 1);
-		$topFolder = substr($script, 0, strpos($script, '/'));
+		$topFolder = substr($script, 0, strpos($script, '/') ?: 0);
 		if (empty($topFolder)) {
 			$path_info = $request->getPathInfo();
 			if ($path_info) {
@@ -682,7 +600,7 @@ class OC_App {
 		}
 		if ($topFolder == 'apps') {
 			$length = strlen($topFolder);
-			return substr($script, $length + 1, strpos($script, '/', $length + 1) - $length - 1);
+			return substr($script, $length + 1, strpos($script, '/', $length + 1) - $length - 1) ?: '';
 		} else {
 			return $topFolder;
 		}
@@ -692,8 +610,8 @@ class OC_App {
 	 * @param string $type
 	 * @return array
 	 */
-	public static function getForms($type) {
-		$forms = array();
+	public static function getForms(string $type): array {
+		$forms = [];
 		switch ($type) {
 			case 'admin':
 				$source = self::$adminForms;
@@ -702,7 +620,7 @@ class OC_App {
 				$source = self::$personalForms;
 				break;
 			default:
-				return array();
+				return [];
 		}
 		foreach ($source as $form) {
 			$forms[] = include $form;
@@ -716,7 +634,7 @@ class OC_App {
 	 * @param string $app
 	 * @param string $page
 	 */
-	public static function registerAdmin($app, $page) {
+	public static function registerAdmin(string $app, string $page) {
 		self::$adminForms[] = $app . '/' . $page . '.php';
 	}
 
@@ -725,7 +643,7 @@ class OC_App {
 	 * @param string $app
 	 * @param string $page
 	 */
-	public static function registerPersonal($app, $page) {
+	public static function registerPersonal(string $app, string $page) {
 		self::$personalForms[] = $app . '/' . $page . '.php';
 	}
 
@@ -739,7 +657,7 @@ class OC_App {
 	/**
 	 * @return array
 	 */
-	public static function getAlternativeLogIns() {
+	public static function getAlternativeLogIns(): array {
 		return self::$altLogin;
 	}
 
@@ -749,13 +667,13 @@ class OC_App {
 	 * @return array an array of app names (string IDs)
 	 * @todo: change the name of this method to getInstalledApps, which is more accurate
 	 */
-	public static function getAllApps() {
+	public static function getAllApps(): array {
 
-		$apps = array();
+		$apps = [];
 
 		foreach (OC::$APPSROOTS as $apps_dir) {
 			if (!is_readable($apps_dir['path'])) {
-				\OCP\Util::writeLog('core', 'unable to read app folder : ' . $apps_dir['path'], \OCP\Util::WARN);
+				\OCP\Util::writeLog('core', 'unable to read app folder : ' . $apps_dir['path'], ILogger::WARN);
 				continue;
 			}
 			$dh = opendir($apps_dir['path']);
@@ -781,13 +699,13 @@ class OC_App {
 	 *
 	 * @return array
 	 */
-	public function listAllApps() {
+	public function listAllApps(): array {
 		$installedApps = OC_App::getAllApps();
 
 		$appManager = \OC::$server->getAppManager();
 		//we don't want to show configuration for these
 		$blacklist = $appManager->getAlwaysEnabledApps();
-		$appList = array();
+		$appList = [];
 		$langCode = \OC::$server->getL10N('core')->getLanguageCode();
 		$urlGenerator = \OC::$server->getURLGenerator();
 
@@ -796,16 +714,16 @@ class OC_App {
 
 				$info = OC_App::getAppInfo($app, false, $langCode);
 				if (!is_array($info)) {
-					\OCP\Util::writeLog('core', 'Could not read app info file for app "' . $app . '"', \OCP\Util::ERROR);
+					\OCP\Util::writeLog('core', 'Could not read app info file for app "' . $app . '"', ILogger::ERROR);
 					continue;
 				}
 
 				if (!isset($info['name'])) {
-					\OCP\Util::writeLog('core', 'App id "' . $app . '" has no name in appinfo', \OCP\Util::ERROR);
+					\OCP\Util::writeLog('core', 'App id "' . $app . '" has no name in appinfo', ILogger::ERROR);
 					continue;
 				}
 
-				$enabled = \OC::$server->getAppConfig()->getValue($app, 'enabled', 'no');
+				$enabled = \OC::$server->getConfig()->getAppValue($app, 'enabled', 'no');
 				$info['groups'] = null;
 				if ($enabled === 'yes') {
 					$active = true;
@@ -862,7 +780,7 @@ class OC_App {
 		return $appList;
 	}
 
-	public static function shouldUpgrade($app) {
+	public static function shouldUpgrade(string $app): bool {
 		$versions = self::getAppVersions();
 		$currentVersion = OC_App::getAppVersion($app);
 		if ($currentVersion && isset($versions[$app])) {
@@ -882,7 +800,7 @@ class OC_App {
 	 * @param string $version2 version to take the number of parts from
 	 * @return string shortened $version1
 	 */
-	private static function adjustVersionParts($version1, $version2) {
+	private static function adjustVersionParts(string $version1, string $version2): string {
 		$version1 = explode('.', $version1);
 		$version2 = explode('.', $version2);
 		// reduce $version1 to match the number of parts in $version2
@@ -912,7 +830,7 @@ class OC_App {
 	 *
 	 * @return boolean true if compatible, otherwise false
 	 */
-	public static function isAppCompatible($ocVersion, $appInfo) {
+	public static function isAppCompatible(string $ocVersion, array $appInfo): bool {
 		$requireMin = '';
 		$requireMax = '';
 		if (isset($appInfo['dependencies']['nextcloud']['@attributes']['min-version'])) {
@@ -933,10 +851,6 @@ class OC_App {
 			$requireMax = $appInfo['requiremax'];
 		}
 
-		if (is_array($ocVersion)) {
-			$ocVersion = implode('.', $ocVersion);
-		}
-
 		if (!empty($requireMin)
 			&& version_compare(self::adjustVersionParts($ocVersion, $requireMin), $requireMin, '<')
 		) {
@@ -966,77 +880,20 @@ class OC_App {
 		return $versions;
 	}
 
-	/**
-	 * @param string $app
-	 * @param \OCP\IConfig $config
-	 * @param \OCP\IL10N $l
-	 * @return bool
-	 *
-	 * @throws Exception if app is not compatible with this version of ownCloud
-	 * @throws Exception if no app-name was specified
-	 */
-	public function installApp($app,
-							   \OCP\IConfig $config,
-							   \OCP\IL10N $l) {
-		if ($app !== false) {
-			// check if the app is compatible with this version of ownCloud
-			$info = self::getAppInfo($app);
-			if(!is_array($info)) {
-				throw new \Exception(
-					$l->t('App "%s" cannot be installed because appinfo file cannot be read.',
-						[$info['name']]
-					)
-				);
-			}
-
-			$version = \OCP\Util::getVersion();
-			if (!self::isAppCompatible($version, $info)) {
-				throw new \Exception(
-					$l->t('App "%s" cannot be installed because it is not compatible with this version of the server.',
-						array($info['name'])
-					)
-				);
-			}
-
-			// check for required dependencies
-			self::checkAppDependencies($config, $l, $info);
-
-			$config->setAppValue($app, 'enabled', 'yes');
-			if (isset($appData['id'])) {
-				$config->setAppValue($app, 'ocsid', $appData['id']);
-			}
-
-			if(isset($info['settings']) && is_array($info['settings'])) {
-				$appPath = self::getAppPath($app);
-				self::registerAutoloading($app, $appPath);
-				\OC::$server->getSettingsManager()->setupSettings($info['settings']);
-			}
-
-			\OC_Hook::emit('OC_App', 'post_enable', array('app' => $app));
-		} else {
-			if(empty($appName) ) {
-				throw new \Exception($l->t("No app name specified"));
-			} else {
-				throw new \Exception($l->t("App '%s' could not be installed!", $appName));
-			}
-		}
-
-		return $app;
-	}
-
 	/**
 	 * update the database for the app and call the update script
 	 *
 	 * @param string $appId
 	 * @return bool
 	 */
-	public static function updateApp($appId) {
+	public static function updateApp(string $appId): bool {
 		$appPath = self::getAppPath($appId);
 		if($appPath === false) {
 			return false;
 		}
 		self::registerAutoloading($appId, $appPath);
 
+		\OC::$server->getAppManager()->clearAppsCache();
 		$appData = self::getAppInfo($appId);
 		self::executeRepairSteps($appId, $appData['repair-steps']['pre-migration']);
 
@@ -1049,7 +906,9 @@ class OC_App {
 
 		self::executeRepairSteps($appId, $appData['repair-steps']['post-migration']);
 		self::setupLiveMigrations($appId, $appData['repair-steps']['live-migration']);
-		unset(self::$appVersion[$appId]);
+		// update appversion in app manager
+		\OC::$server->getAppManager()->clearAppsCache();
+		\OC::$server->getAppManager()->getAppVersion($appId, false);
 
 		// run upgrade code
 		if (file_exists($appPath . '/appinfo/update.php')) {
@@ -1057,9 +916,6 @@ class OC_App {
 			include $appPath . '/appinfo/update.php';
 		}
 		self::setupBackgroundJobs($appData['background-jobs']);
-		if(isset($appData['settings']) && is_array($appData['settings'])) {
-			\OC::$server->getSettingsManager()->setupSettings($appData['settings']);
-		}
 
 		//set remote/public handlers
 		if (array_key_exists('ocsid', $appData)) {
@@ -1077,7 +933,7 @@ class OC_App {
 		self::setAppTypes($appId);
 
 		$version = \OC_App::getAppVersion($appId);
-		\OC::$server->getAppConfig()->setValue($appId, 'installed_version', $version);
+		\OC::$server->getConfig()->setAppValue($appId, 'installed_version', $version);
 
 		\OC::$server->getEventDispatcher()->dispatch(ManagerEvent::EVENT_APP_UPDATE, new ManagerEvent(
 			ManagerEvent::EVENT_APP_UPDATE, $appId
@@ -1091,7 +947,7 @@ class OC_App {
 	 * @param string[] $steps
 	 * @throws \OC\NeedsUpdateException
 	 */
-	public static function executeRepairSteps($appId, array $steps) {
+	public static function executeRepairSteps(string $appId, array $steps) {
 		if (empty($steps)) {
 			return;
 		}
@@ -1125,7 +981,7 @@ class OC_App {
 	 * @param string $appId
 	 * @param string[] $steps
 	 */
-	private static function setupLiveMigrations($appId, array $steps) {
+	private static function setupLiveMigrations(string $appId, array $steps) {
 		$queue = \OC::$server->getJobList();
 		foreach ($steps as $step) {
 			$queue->add('OC\Migration\BackgroundRepair', [
@@ -1138,7 +994,7 @@ class OC_App {
 	 * @param string $appId
 	 * @return \OC\Files\View|false
 	 */
-	public static function getStorage($appId) {
+	public static function getStorage(string $appId) {
 		if (\OC::$server->getAppManager()->isEnabledForUser($appId)) { //sanity check
 			if (\OC::$server->getUserSession()->isLoggedIn()) {
 				$view = new \OC\Files\View('/' . OC_User::getUser());
@@ -1147,16 +1003,16 @@ class OC_App {
 				}
 				return new \OC\Files\View('/' . OC_User::getUser() . '/' . $appId);
 			} else {
-				\OCP\Util::writeLog('core', 'Can\'t get app storage, app ' . $appId . ', user not logged in', \OCP\Util::ERROR);
+				\OCP\Util::writeLog('core', 'Can\'t get app storage, app ' . $appId . ', user not logged in', ILogger::ERROR);
 				return false;
 			}
 		} else {
-			\OCP\Util::writeLog('core', 'Can\'t get app storage, app ' . $appId . ' not enabled', \OCP\Util::ERROR);
+			\OCP\Util::writeLog('core', 'Can\'t get app storage, app ' . $appId . ' not enabled', ILogger::ERROR);
 			return false;
 		}
 	}
 
-	protected static function findBestL10NOption($options, $lang) {
+	protected static function findBestL10NOption(array $options, string $lang): string {
 		// only a single option
 		if (isset($options['@value'])) {
 			return $options['@value'];
@@ -1213,7 +1069,7 @@ class OC_App {
 	 * @param string $lang
 	 * @return array improved app data
 	 */
-	public static function parseAppInfo(array $data, $lang = null) {
+	public static function parseAppInfo(array $data, $lang = null): array {
 
 		if ($lang && isset($data['name']) && is_array($data['name'])) {
 			$data['name'] = self::findBestL10NOption($data['name'], $lang);
@@ -1238,13 +1094,13 @@ class OC_App {
 	 * @param array $info
 	 * @throws \Exception
 	 */
-	public static function checkAppDependencies($config, $l, $info) {
+	public static function checkAppDependencies(\OCP\IConfig $config, \OCP\IL10N $l, array $info) {
 		$dependencyAnalyzer = new DependencyAnalyzer(new Platform($config), $l);
 		$missing = $dependencyAnalyzer->analyze($info);
 		if (!empty($missing)) {
 			$missingMsg = implode(PHP_EOL, $missing);
 			throw new \Exception(
-				$l->t('App "%s" cannot be installed because the following dependencies are not fulfilled: %s',
+				$l->t('App "%1$s" cannot be installed because the following dependencies are not fulfilled: %2$s',
 					[$info['name'], $missingMsg]
 				)
 			);
