@@ -43,6 +43,7 @@
  * @author Victor Dubiniuk <dubiniuk@owncloud.com>
  * @author Vincent Petry <pvince81@owncloud.com>
  * @author Volkan Gezer <volkangezer@gmail.com>
+ * @author Robert Dailey <rcdailey@gmail.com>
  *
  * @license AGPL-3.0
  *
@@ -203,7 +204,7 @@ class OC_Util {
 
 		\OC\Files\Filesystem::initMountManager();
 
-		\OC\Files\Filesystem::logWarningWhenAddingStorageWrapper(false);
+		$prevLogging = \OC\Files\Filesystem::logWarningWhenAddingStorageWrapper(false);
 		\OC\Files\Filesystem::addStorageWrapper('mount_options', function ($mountPoint, \OCP\Files\Storage $storage, \OCP\Files\Mount\IMountPoint $mount) {
 			if ($storage->instanceOfStorage('\OC\Files\Storage\Common')) {
 				/** @var \OC\Files\Storage\Common $storage */
@@ -278,7 +279,8 @@ class OC_Util {
 		});
 
 		OC_Hook::emit('OC_Filesystem', 'preSetup', array('user' => $user));
-		\OC\Files\Filesystem::logWarningWhenAddingStorageWrapper(true);
+
+		\OC\Files\Filesystem::logWarningWhenAddingStorageWrapper($prevLogging);
 
 		//check if we are using an object storage
 		$objectStore = \OC::$server->getSystemConfig()->getValue('objectstore', null);
@@ -390,9 +392,10 @@ class OC_Util {
 	/**
 	 * copies the skeleton to the users /files
 	 *
-	 * @param String $userId
+	 * @param string $userId
 	 * @param \OCP\Files\Folder $userDirectory
-	 * @throws \RuntimeException
+	 * @throws \OCP\Files\NotFoundException
+	 * @throws \OCP\Files\NotPermittedException
 	 * @suppress PhanDeprecatedFunction
 	 */
 	public static function copySkeleton($userId, \OCP\Files\Folder $userDirectory) {
@@ -788,13 +791,20 @@ class OC_Util {
 					];
 				}
 			} else if (!is_writable($CONFIG_DATADIRECTORY) or !is_readable($CONFIG_DATADIRECTORY)) {
-				//common hint for all file permissions error messages
-				$permissionsHint = $l->t('Permissions can usually be fixed by giving the webserver write access to the root directory. See %s.',
-					[$urlGenerator->linkToDocs('admin-dir_permissions')]);
-				$errors[] = [
-					'error' => 'Your data directory is not writable',
-					'hint' => $permissionsHint
-				];
+				// is_writable doesn't work for NFS mounts, so try to write a file and check if it exists.
+				$testFile = sprintf('%s/%s.tmp', $CONFIG_DATADIRECTORY, uniqid('data_dir_writability_test_'));
+				$handle = fopen($testFile, 'w');
+				if (!$handle || fwrite($handle, 'Test write operation') === FALSE) {
+					$permissionsHint = $l->t('Permissions can usually be fixed by giving the webserver write access to the root directory. See %s.',
+						[$urlGenerator->linkToDocs('admin-dir_permissions')]);
+					$errors[] = [
+						'error' => 'Your data directory is not writable',
+						'hint' => $permissionsHint
+					];
+				} else {
+					fclose($handle);
+					unlink($testFile);
+				}
 			} else {
 				$errors = array_merge($errors, self::checkDataDirectoryPermissions($CONFIG_DATADIRECTORY));
 			}
@@ -826,7 +836,7 @@ class OC_Util {
 			),
 			'functions' => [
 				'xml_parser_create' => 'libxml',
-				'mb_strcut' => 'mb multibyte',
+				'mb_strcut' => 'mbstring',
 				'ctype_digit' => 'ctype',
 				'json_encode' => 'JSON',
 				'gd_info' => 'GD',
@@ -848,44 +858,36 @@ class OC_Util {
 		$invalidIniSettings = [];
 		$moduleHint = $l->t('Please ask your server administrator to install the module.');
 
-		/**
-		 * FIXME: The dependency check does not work properly on HHVM on the moment
-		 *        and prevents installation. Once HHVM is more compatible with our
-		 *        approach to check for these values we should re-enable those
-		 *        checks.
-		 */
 		$iniWrapper = \OC::$server->getIniWrapper();
-		if (!self::runningOnHhvm()) {
-			foreach ($dependencies['classes'] as $class => $module) {
-				if (!class_exists($class)) {
-					$missingDependencies[] = $module;
+		foreach ($dependencies['classes'] as $class => $module) {
+			if (!class_exists($class)) {
+				$missingDependencies[] = $module;
+			}
+		}
+		foreach ($dependencies['functions'] as $function => $module) {
+			if (!function_exists($function)) {
+				$missingDependencies[] = $module;
+			}
+		}
+		foreach ($dependencies['defined'] as $defined => $module) {
+			if (!defined($defined)) {
+				$missingDependencies[] = $module;
+			}
+		}
+		foreach ($dependencies['ini'] as $setting => $expected) {
+			if (is_bool($expected)) {
+				if ($iniWrapper->getBool($setting) !== $expected) {
+					$invalidIniSettings[] = [$setting, $expected];
 				}
 			}
-			foreach ($dependencies['functions'] as $function => $module) {
-				if (!function_exists($function)) {
-					$missingDependencies[] = $module;
+			if (is_int($expected)) {
+				if ($iniWrapper->getNumeric($setting) !== $expected) {
+					$invalidIniSettings[] = [$setting, $expected];
 				}
 			}
-			foreach ($dependencies['defined'] as $defined => $module) {
-				if (!defined($defined)) {
-					$missingDependencies[] = $module;
-				}
-			}
-			foreach ($dependencies['ini'] as $setting => $expected) {
-				if (is_bool($expected)) {
-					if ($iniWrapper->getBool($setting) !== $expected) {
-						$invalidIniSettings[] = [$setting, $expected];
-					}
-				}
-				if (is_int($expected)) {
-					if ($iniWrapper->getNumeric($setting) !== $expected) {
-						$invalidIniSettings[] = [$setting, $expected];
-					}
-				}
-				if (is_string($expected)) {
-					if (strtolower($iniWrapper->getString($setting)) !== strtolower($expected)) {
-						$invalidIniSettings[] = [$setting, $expected];
-					}
+			if (is_string($expected)) {
+				if (strtolower($iniWrapper->getString($setting)) !== strtolower($expected)) {
+					$invalidIniSettings[] = [$setting, $expected];
 				}
 			}
 		}
@@ -1085,26 +1087,6 @@ class OC_Util {
 		}
 	}
 
-	/**
-	 * Check if the user is a subadmin, redirects to home if not
-	 *
-	 * @return null|boolean $groups where the current user is subadmin
-	 */
-	public static function checkSubAdminUser() {
-		OC_Util::checkLoggedIn();
-		$userObject = \OC::$server->getUserSession()->getUser();
-		$isSubAdmin = false;
-		if($userObject !== null) {
-			$isSubAdmin = \OC::$server->getGroupManager()->getSubAdmin()->isSubAdmin($userObject);
-		}
-
-		if (!$isSubAdmin) {
-			header('Location: ' . \OCP\Util::linkToAbsolute('', 'index.php'));
-			exit();
-		}
-		return true;
-	}
-
 	/**
 	 * Returns the URL of the default page
 	 * based on the system configuration and
@@ -1267,6 +1249,18 @@ class OC_Util {
 			$content = false;
 		}
 
+		if (strpos($url, 'https:') === 0) {
+			$url = 'http:' . substr($url, 6);
+		} else {
+			$url = 'https:' . substr($url, 5);
+		}
+
+		try {
+			$fallbackContent = \OC::$server->getHTTPClientService()->newClient()->get($url)->getBody();
+		} catch (\Exception $e) {
+			$fallbackContent = false;
+		}
+
 		// cleanup
 		@unlink($testFile);
 
@@ -1274,7 +1268,7 @@ class OC_Util {
 		 * If the content is not equal to test content our .htaccess
 		 * is working as required
 		 */
-		return $content !== $testContent;
+		return $content !== $testContent && $fallbackContent !== $testContent;
 	}
 
 	/**
@@ -1332,15 +1326,6 @@ class OC_Util {
 		return (strtoupper(substr(PHP_OS, 0, 6)) === 'DARWIN');
 	}
 
-	/**
-	 * Checks whether server is running on HHVM
-	 *
-	 * @return bool True if running on HHVM, false otherwise
-	 */
-	public static function runningOnHhvm() {
-		return defined('HHVM_VERSION');
-	}
-
 	/**
 	 * Handles the case that there may not be a theme, then check if a "default"
 	 * theme exists and take that one
