@@ -1,6 +1,25 @@
 OC.L10N.register(
     "settings",
     {
+    "Saved" : "Запаметяване",
+    "Email sent" : "Имейлът е изпратен",
+    "Valid until {date}" : "Далидна до {date}",
+    "Delete" : "Изтриване",
+    "Local" : "Локално",
+    "Only visible to local users" : "Видима само за локални потребители",
+    "Only visible to you" : "Видима само за вас",
+    "Contacts" : "Контакти",
+    "Visible to local users and to trusted servers" : "Видима за локални потребители и от доверени сървъри",
+    "Public" : "Публичен",
+    "Unable to change password" : "Неуспешна смяна на паролата.",
+    "Very weak password" : "Много проста парола",
+    "Weak password" : "Проста парола",
+    "So-so password" : "Не особено добра парола",
+    "Good password" : "Добра парола",
+    "Strong password" : "Сложна парола",
+    "Select a profile picture" : "Избор на снимка за профила",
+    "Week starts on {fdow}" : "Първи ден от седмицата е {fdow}",
+    "Groups" : "Групи",
     "{actor} changed your password" : "{actor} промени паролата ви",
     "You changed your password" : "Променихте паролата си",
     "Your password was reset by an administrator" : "Паролата ви е променена от администратора",
@@ -12,10 +31,8 @@ OC.L10N.register(
     "Couldn't remove app." : "Приложението не бе премахнато.",
     "Couldn't update app." : "Приложението не бе актуализирано.",
     "Wrong password" : "Грешна парола",
-    "Saved" : "Запаметяване",
     "No user supplied" : "Липсва потребител",
-    "Unable to change password" : "Неуспешна смяна на паролата.",
-    "Authentication error" : "Възникна проблем с идентификацията",
+    "Authentication error" : "Възникна проблем с удостоверяването",
     "Wrong admin recovery password. Please check the password and try again." : "Грешна администраторска парола за възстановяване. Моля, проверете паролата и опитайте отново.",
     "Invalid SMTP password." : "Невалидна SMTP парола.",
     "A problem occurred while sending the email. Please revise your settings. (Error: %s)" : "Възникна проблем при изпращането на имейла. Моля, провери настройките.  (Грешка: %s)",
@@ -33,55 +50,56 @@ OC.L10N.register(
     "Welcome aboard %s" : "Добре дошли %s",
     "Your username is: %s" : "Потребителско име: %s",
     "Set your password" : "Задайте парола",
-    "Email sent" : "Имейлът е изпратен",
-    "Internet Explorer" : "Internet Explorer",
-    "Edge" : "Edge",
-    "Firefox" : "Firefox",
-    "Google Chrome" : "Google Chrome",
-    "Safari" : "Safari",
-    "Google Chrome for Android" : "Google Chrome за Android",
-    "iPhone iOS" : "iPhone iOS",
-    "iPad iOS" : "iPad iOS",
-    "iOS Client" : "iOS клиент",
-    "Android Client" : "Android клиент",
-    "This session" : "Текущата сесия",
-    "Copy" : "Копиране",
-    "Copied!" : "Копирано!",
-    "Not supported!" : "Не се поддържа!",
-    "Press ⌘-C to copy." : "За копиране натиснете ⌘-C",
-    "Press Ctrl-C to copy." : "За копиране натиснете Ctrl-C",
-    "Valid until {date}" : "Далидна до {date}",
-    "Delete" : "Изтриване",
-    "Local" : "Локално",
-    "Contacts" : "Контакти",
-    "Public" : "Публичен",
-    "Very weak password" : "Много проста парола",
-    "Weak password" : "Проста парола",
-    "So-so password" : "Не особено добра парола",
-    "Good password" : "Добра парола",
-    "Strong password" : "Сложна парола",
-    "Select a profile picture" : "Избор на снимка за профила",
-    "Week starts on {fdow}" : "Първи ден от седмицата е {fdow}",
-    "Groups" : "Групи",
+    "Unlimited" : "Неограничено",
+    "Two-factor authentication can be enforced for all\tusers and specific groups. If they do not have a two-factor provider configured, they will be unable to log into the system." : "Ползването на двустепенното удостоверяване може да бъде наложено на всички\tпотребители или определени групи. Ако потребител не е настроил втората стъпка - няма да може да се впише.",
     "Limit to groups" : "Ограничен достъп",
-    "Official" : "Официално",
+    "Update to {version}" : "Актуализирай до {version}",
     "Disable" : "Изключване",
     "All" : "Всички",
+    "This app cannot be installed because the following dependencies are not fulfilled:" : "Приложението не може да бъде инсталирано, защото следните зависимости не са удовлетворени:",
     "View in store" : "Страница в магазина",
     "Visit website" : "Уеб страницата",
     "Report a bug" : "Докладване на грешка",
     "User documentation" : "Документация за потребители",
     "Admin documentation" : "Документация за администратори",
     "Developer documentation" : "Документация за разработчици",
-    "This app cannot be installed because the following dependencies are not fulfilled:" : "Приложението не може да бъде инсталирано, защото следните зависимости не са удовлетворени:",
+    "Update to {update}" : "Актуализирай до {update}",
     "No apps found for your version" : "Няма намерени приложения за версията, която ползвате",
     "Download and enable" : "Сваляне и включване",
     "Enable" : "Включване",
-    "The app will be downloaded from the app store" : "Приложението ще бъде свалено от магазина за приложения",
-    "New password" : "Нова парола",
-    "{size} used" : "{size} използвани",
+    "Enable untested app" : "Включване (нетествано приложение)",
+    "The app will be downloaded from the app store" : "Приложението ще бъде свалено от магазина за приложения.",
+    "This app is not marked as compatible with your Nextcloud version. If you continue you will still be able to install the app. Note that the app might not work as expected." : "Приложението не е маркирано като съвместимо с вашата версия на Nextcloud. Можете да инсталирате приложението, но е вероятно да не работи безпроблемно.",
+    "Device settings" : "Настройки за устройството",
+    "Allow filesystem access" : "Разрешен достъп до файловете",
+    "Revoke" : "Отнемане",
+    "Internet Explorer" : "Internet Explorer",
+    "Edge" : "Edge",
+    "Firefox" : "Firefox",
+    "Google Chrome" : "Google Chrome",
+    "Safari" : "Safari",
+    "Google Chrome for Android" : "Google Chrome за Android",
+    "iPhone" : "iPhone",
+    "iPad" : "iPad",
+    "This session" : "Текущата сесия",
+    "Device" : "Устройство",
+    "Last activity" : "Последна активност",
+    "Devices & sessions" : "Устройства и сесии",
+    "Web, desktop and mobile clients currently logged in to your account." : "Уеб, настолни и мобилни клиенти, които в момента са вписани чрез вашия акаунт.",
+    "App name" : "Име на приложението",
+    "Create new app password" : "Създай парола за приложението",
+    "Use the credentials below to configure your app or device." : "Ползвайте данните по-долу за да настроите вашето приложение или устройство.",
     "Username" : "Потребител",
     "Password" : "Парола",
+    "Done" : "Готово",
+    "Copied!" : "Копирано!",
+    "Copy" : "Копиране",
+    "Add user in group" : "Добавяне към група",
+    "Delete user" : "Изтриване",
+    "Disable user" : "Деактивирай",
+    "Enable user" : "Активиране",
+    "Cancel" : "Отказ",
+    "{size} used" : "{size} използвани",
     "Email" : "Имейл",
     "Group admin for" : "Групов администратор за",
     "Quota" : "Квота",
@@ -89,15 +107,22 @@ OC.L10N.register(
     "Storage location" : "Дисково пространство",
     "Last login" : "Последно вписване",
     "Default language" : "Стандартен език",
-    "Unlimited" : "Неограничено",
+    "Add a new user" : "Добавяне на новия потребител",
     "Default quota" : "Стандартна квота",
     "All languages" : "Всички езици",
     "Your apps" : "Вашите приложения",
     "Active apps" : "Включени приложения",
     "Disabled apps" : "Изключени приложения",
-    "Admins" : "Администратори",
-    "Everyone" : "Всички",
+    "Updates" : "Актуализации",
     "New user" : "Нов потребител",
+    "Default quota:" : "Стандартна квота:",
+    "Select default quota" : "Изберете стандартна квота",
+    "Show Languages" : "Показвай ползвания език",
+    "Show last login" : "Показвай последното вписване",
+    "Admins" : "Администратори",
+    "Disabled users" : "Деактивирани потребители",
+    "Everyone" : "Всички",
+    "Add group" : "Нова група",
     "Common Name" : "Познато Име",
     "Valid until" : "Валиден до",
     "Issued By" : "Издаден от",
@@ -114,51 +139,59 @@ OC.L10N.register(
     "STARTTLS" : "STARTTLS",
     "Email server" : "Имейл сървър",
     "Open documentation" : "Отвори документацията",
+    "It is important to set up this server to be able to send emails, like for password reset and notifications." : "След като настроите сървъра Nextcloud ще може да изпраща имейли за възстановяване на забравени пароли и известия.",
     "Send mode" : "Режим на изпращане",
     "Encryption" : "Криптиране",
     "From address" : "От адрес",
     "mail" : "поща",
-    "Authentication method" : "Метод за отризиране",
-    "Authentication required" : "Нужна е идентификация",
+    "Authentication method" : "Метод за удостоверяване",
+    "Authentication required" : "Нужно е удостоверяване",
     "Server address" : "Адрес на сървъра",
     "Port" : "Порт",
-    "Credentials" : "Потр. име и парола",
+    "Credentials" : "Потребител и парола",
     "SMTP Username" : "SMTP потребител",
     "SMTP Password" : "SMTP парола",
-    "Store credentials" : "Запазвай креденциите",
     "Test email settings" : "Проверка на имейл настройките",
     "Send email" : "Изпрати имейл",
-    "Security & setup warnings" : "Предупреждения за сигурност и настройки",
+    "Security & setup warnings" : "Предупреждения за сигурността и настройките",
+    "It's important for the security and performance of your instance that everything is configured correctly. To help you with that we are doing some automatic checks. Please see the linked documentation for more information." : "Важно е, от гледна точка на сигурност и производителност, всичко да бъде настроено коректно. За тази цел са създадени автоматични проверки. Прегледайте връзките, към документацията (по-долу), за допълнителна информация.",
+    "There are some errors regarding your setup." : "Открити са грешки, в инсталацията на Nextcloud.",
     "There are some warnings regarding your setup." : "Предупреждения относно текущата инсталация.",
+    "Checking for system and security issues." : "Проверка за проблеми със системата и сигурността.",
+    "Please double check the <a target=\"_blank\" rel=\"noreferrer noopener\" href=\"%1$s\">installation guides ↗</a>, and check for any errors or warnings in the <a href=\"%2$s\">log</a>." : "Прегледайте  <a target=\"_blank\" rel=\"noreferrer noopener\" href=\"%1$s\">ръководството за инсталиране ↗</a>, и проверете  <a href=\"%2$s\">журналите</a>за предупреждения и грешки. ",
     "Check the security of your Nextcloud over <a target=\"_blank\" rel=\"noreferrer noopener\" href=\"%s\">our security scan ↗</a>." : "Проверете сигурността на Nextcloud <a target=\"_blank\" rel=\"noreferrer noopener\" href=\"%s\">като заявите сканиране ↗</a>.",
     "Version" : "Версия",
+    "Two-Factor Authentication" : "Двустепенно удостоверяване",
     "Enable encryption" : "Включване на криптиране",
     "Select default encryption module:" : "Избор на модул за криптиране по подразбиране:",
     "Start migration" : "Начало на миграцията",
-    "Execute one task with each page loaded" : "Изпълни по едно задание с всяка заредена страница.",
+    "Last job ran %s." : "За последно cron задача е стартирала %s.",
+    "Last job execution ran %s. Something seems wrong." : "За последно cron задача е стартирала %s. Изглежда, че има проблем.",
+    "Execute one task with each page loaded" : "Изпълнява една задача с всяка заредена страница.",
+    "The cron.php needs to be executed by the system user \"%s\"." : "Необходимо е cron.php да се стартира от системния потребител \"%s\".",
     "Sharing" : "Споделяне",
     "Allow apps to use the Share API" : "Разреши приложенията да използват Share API",
-    "Allow users to share via link" : "Разреши потребителите да споделят с връзка",
-    "Allow public uploads" : "Разреши общодостъпно качване",
-    "Enforce password protection" : "Изискай защита с парола.",
-    "Set default expiration date" : "Задайте стандартен срок на валидност",
     "Expire after " : "Изтечи след",
     "days" : "дена",
     "Enforce expiration date" : "Изисквай дата на изтичане",
+    "Allow users to share via link" : "Разреши потребителите да споделят с връзка",
+    "Allow public uploads" : "Разреши общодостъпно качване",
+    "Enforce password protection" : "Изискай защита с парола.",
     "Allow resharing" : "Може да споделя повторно",
     "Restrict users to only share with users in their groups" : "Ограничи потребителите, така че да могат да споделят само с други потребители в своите групи.",
     "Exclude groups from sharing" : "Забрани групи да споделят",
     "These groups will still be able to receive shares, but not to initiate them." : "Тези групи ще могат да получават споделения, но няма да могат самите те да споделят.",
     "Personal" : "Лични",
     "Administration" : "Административни",
-    "Follow us on Google+" : "Следвайте ни в Google+",
     "Follow us on Twitter" : "Следвайте ни в Twitter",
+    "Follow us on Mastodon" : "Следвайте ни в Mastodon",
+    "Check out our blog" : "Посетете блога ни",
+    "Subscribe to our newsletter" : "Абонирайте се за нашия имейл бюлетин",
     "Profile picture" : "Снимка за профила",
     "Upload new" : "Качи нова",
     "Select from Files" : "Избери от файловете",
     "Remove image" : "Премахни изображението",
     "png or jpg, max. 20 MB" : "png или jpg, макс. 20 MB",
-    "Cancel" : "Отказ",
     "Choose as profile picture" : "Избор на профилна снимка",
     "Details" : "Подробности",
     "You are a member of the following groups:" : "Членувате в следните групи:",
@@ -167,6 +200,7 @@ OC.L10N.register(
     "No display name set" : "Няма настроено екранно име",
     "Your email address" : "Вашият имейл адрес",
     "No email address set" : "Не е въведен имейл адрес",
+    "For password reset and notifications" : "За възстановяване на забравена парола и известия",
     "Phone number" : "Тел. номер",
     "Your phone number" : "Вашият тел. номер",
     "Address" : "Адрес",
@@ -175,91 +209,23 @@ OC.L10N.register(
     "Twitter" : "Twitter",
     "Help translate" : "Помогнете с превода",
     "Current password" : "Текуща парола",
+    "New password" : "Нова парола",
     "Change password" : "Промени паролата",
-    "Devices & sessions" : "Устройства и сесии",
-    "Web, desktop and mobile clients currently logged in to your account." : "Уеб, настолни и мобилни клиенти, които в момента са вписани чрез вашия акаунт.",
-    "Device" : "Устройство",
-    "Last activity" : "Последна активност",
-    "App name" : "Име на приложението",
-    "Create new app password" : "Създай парола за приложението",
-    "Use the credentials below to configure your app or device." : "Ползвайте данните по-долу за да настроите вашето приложение или устройство.",
-    "Done" : "Готово",
-    "Enabled apps" : "Включени приложения",
-    "Group already exists." : "Групата вече съществува.",
-    "Unable to add group." : "Неуспешно добавяне на група.",
-    "Unable to delete group." : "Неуспешно изтриване на група.",
-    "A user with that name already exists." : "Потребител с това име вече съществува.",
-    "Unable to create user." : "Неуспешно създаване на потребител.",
-    "Unable to delete user." : "Неуспешно изтриване на потребител.",
-    "Your full name has been changed." : "Вашето име е променено.",
-    "Forbidden" : "Забранено",
-    "Invalid user" : "Невалиден протребител",
-    "Unable to change mail address" : "Неуспешна промяна на адрес на електронна поща",
-    "Email saved" : "Имейлът е запазен",
-    "Are you really sure you want add {domain} as trusted domain?" : "Наистина ли желаете да добавите {domain} в списъка със сигурни домейни?",
-    "Add trusted domain" : "Добавяне на сигурен домейн",
-    "Update to %s" : "Актуализирай до %s",
-    "Disabling app …" : "Забраняване на приложение ...",
-    "Error while disabling app" : "Грешка при изключване на приложението",
-    "Enabling app …" : "Разрешаване на приложение ...",
-    "Error while enabling app" : "Грешка при включване на приложението",
-    "Updated" : "Актуализирано",
-    "Approved" : "Одобрен",
-    "Experimental" : "Ексериментален",
-    "Unable to delete {objName}" : "Неуспешно изтриване на {objName}.",
-    "A valid group name must be provided" : "Очаква се валидно име на група",
-    "deleted {groupName}" : "{groupName} е изтрита",
-    "undo" : "възстановяване",
-    "never" : "никога",
-    "deleted {userName}" : "{userName}  е изтрит",
-    "Changing the password will result in data loss, because data recovery is not available for this user" : "Промяна на паролата ще доведе до загуба на данни, защото не е налично възстановяване за този потребител.",
-    "A valid username must be provided" : "Трябва да въведете валидно потребителско име",
-    "Error creating user: {message}" : "Грешка при създаване на потребител: {message}",
-    "A valid password must be provided" : "Трябва да въведете валидна парола",
-    "A valid email must be provided" : "Трябва да въведете валиден имейл адрес",
-    "Personal info" : "Лични данни",
-    "This is probably caused by a cache/accelerator such as Zend OPcache or eAccelerator." : "Това може да се дължи на кеш/акселератор като Zend OPache или eAccelerator.",
-    "System locale can not be set to a one which supports UTF-8." : "Системните настройки за местоположение не могат да бъдат промени на такива, подържащи UTF-8.",
-    "Please double check the <a target=\"_blank\" rel=\"noreferrer\" href=\"%s\">installation guides ↗</a>, and check for any errors or warnings in the <a href=\"%s\">log</a>." : "Прегледайте <a target=\"_blank\" rel=\"noreferrer\" href=\"%s\">ръководството за инсталиране ↗</a>, и проверете за предупреждения и грешки <a href=\"%s\">журналите</a>.",
-    "This is particularly recommended when using the desktop client for file synchronisation." : "Препоръчително, особено ако ползвате клиента за настолен компютър.",
-    "How to do backups" : "Как се правят резервни копия",
-    "Performance tuning" : "Настройване на производителността",
-    "Improving the config.php" : "Подобряване на config.php",
-    "Theming" : "Промяна на облика",
-    "Check the security of your Nextcloud over our security scan" : "Проверете сигурността на текущата инсталация на Nextcloud",
-    "This app has an update available." : "Налична е актуализация за приложението.",
-    "by %s" : "от %s",
-    "Documentation:" : "Документация:",
-    "Show description …" : "Покажи описанието ...",
-    "Hide description …" : "Скрий описанието ...",
-    "Enable only for specific groups" : "Включи само за определени групи",
-    "Online documentation" : "Онлайн документация",
-    "Commercial support" : "Платена поддръжка",
-    "You are using <strong>%s</strong> of <strong>%s</strong>" : "Ползвате <strong>%s</strong> от <strong>%s</strong>",
-    "You are using <strong>%s</strong> of <strong>%s</strong> (<strong>%s %%</strong>)" : "Ползвате <strong>%s</strong> от <strong>%s</strong> (<strong>%s %%</strong>)",
-    "You are member of the following groups:" : "Членувате в следните групи:",
-    "Get the apps to sync your files" : "Приложения за синхронизиране на файлове",
-    "Desktop client" : "Клиент за настолни компютри",
-    "Android app" : "Android приложение",
-    "iOS app" : "iOS приложениер",
-    "App passwords" : "Пароли за приложения",
-    "Follow us on Google+!" : "Следвайте ни в Google+!",
-    "Follow us on Twitter!" : "Следвайте ни в Twitter!",
-    "Check out our blog!" : "Прегледайте нашия блог!",
-    "Settings" : "Настройки",
-    "Show storage location" : "Покажи мястото на хранилището",
-    "Show email address" : "Покажи адреса на електронната поща",
-    "Send email to new user" : "Изпращай писмо към нов потребител",
-    "E-Mail" : "Имейл",
-    "Create" : "Създаване",
-    "Admin Recovery Password" : "Възстановяване на администраторската парола",
-    "Enter the recovery password in order to recover the users files during password change" : "Въведете паролата за възстановяване, за да възстановиш файловете на потребителите при промяна на паролата.",
-    "Please enter storage quota (ex: \"512 MB\" or \"12 GB\")" : "Моля, въведете квота за хранилището (напр. \"512 MB\" или \"12 GB\")",
-    "Other" : "Друга...",
-    "change full name" : "промени име",
-    "set new password" : "сложи нова парола",
-    "change email address" : "Смени адреса на елетронната поща",
-    "Default" : "Стандарт",
-    "Please double check the <a target=\"_blank\" rel=\"noreferrer noopener\" href=\"%s\">installation guides ↗</a>, and check for any errors or warnings in the <a href=\"%s\">log</a>." : "Прегледайте <a target=\"_blank\" rel=\"noreferrer noopener\" href=\"%s\">ръководството за инсталиране ↗</a>, и проверете за предупреждения и грешки <a href=\"%s\">журналите</a>."
+    "Use a second factor besides your password to increase security for your account." : "Ползвайте двустепенно удостоверяване за да повишите сигурността на профила си.",
+    "Not supported!" : "Не се поддържа!",
+    "Press ⌘-C to copy." : "За копиране натиснете ⌘-C.",
+    "Press Ctrl-C to copy." : "За копиране натиснете Ctrl-C.",
+    "Afrikaans" : "африкаанс (бурски)",
+    "Armenian" : "арменски",
+    "English" : "английски",
+    "Hungarian" : "унгарски",
+    "Japanese" : "японски",
+    "Official apps are developed by and within the community. They offer central functionality and are ready for production use." : "Официалните приложения са разработени от общността. ",
+    "Official" : "Официално",
+    "Store credentials" : "Запиши",
+    "cron.php is registered at a webcron service to call cron.php every 15 minutes over HTTP." : "Услугата webcron стартира cron.php всеки 15 минути по HTTP.",
+    "Use system cron service to call the cron.php file every 15 minutes." : "Ползва услугата cron на сървъра за да стартира cron.php всеки 15 минути.",
+    "Set default expiration date" : "Задайте стандартен срок на валидност",
+    "Follow us on Google+" : "Следвайте ни в Google+"
 },
 "nplurals=2; plural=(n != 1);");
